package pps.core.common.pattern;


import java.util.Objects;

public class Builder<T> {

    @FunctionalInterface
    public static interface FieldsValidator<T> {
        boolean valid(T instance);
    }
    @FunctionalInterface
    public static interface FillField<T> {
        void fill(T instance);
    }
    @FunctionalInterface
    public static interface Init<T> {
        void initialize(T instanse);
    }

    private final Class<T> clazz;
    private Init<T> init;
    private FieldsValidator<T> fieldsValidator;

    public Builder(Class<T> clazz) {
        this.clazz = clazz;
    }

    public Builder(Class<T> clazz, Init<T> init) {
        this.clazz = clazz;
        this.init = init;
    }

    public Builder(Class<T> clazz, Init<T> init, FieldsValidator<T> fieldsValidator) {
        this.clazz = clazz;
        this.init = init;
        this.fieldsValidator = fieldsValidator;
    }

    public static <T> Builder<T> create(Class<T> clazz) {
        return new Builder<>(clazz);
    }

    public static <T> Builder<T> create(Class<T> clazz, Init<T> init) {
        return new Builder<>(clazz, init);
    }

    public static <T> Builder<T> create(Class<T> clazz, Init<T> init, FieldsValidator<T> fieldsValidator) {
        return new Builder<>(clazz, init, fieldsValidator);
    }

    public Builder<T> addValid(FieldsValidator<T> valid){
        this.fieldsValidator = valid;
        return this;
    }

    public T build(FillField<T> fillField) {
        T instance = null;

        try {
            instance = this.clazz.newInstance();
            if (Objects.nonNull(init)){
                init.initialize(instance);
            }
        } catch (InstantiationException | IllegalAccessException e) {
            throw new RuntimeException(e.getMessage());
        }

        fillField.fill(instance);

        if (Objects.isNull(fieldsValidator)){
            return instance;
        }

        boolean validRet = this.fieldsValidator.valid(instance);
        if (validRet){
            return instance;
        }

        throw new RuntimeException(String.format("instance fields valid error, Class is %s", instance.getClass().getName()));
    }
}