package pps.core.common.pattern;

import java.util.ArrayList;
import java.util.List;
import java.util.function.Consumer;

/**
 * Node of Chain Pattern
 */
public class ChainNode<T> {
    private T node;
    private ChainNode<T> successor;

    public T getNode() {
        return node;
    }

    public void setNode(T node) {
        this.node = node;
    }

    public ChainNode<T> getSuccessor() {
        return successor;
    }

//    public void setSuccessor(ChainNode<T> successor) {
//        this.successor = successor;
//    }

    public ChainNode<T> append(T successorNode){
        ChainNode<T> successorChainNode = new ChainNode<>();
        successorChainNode.setNode(successorNode);
//        this.setSuccessor(successorChainNode);
        this.successor = successorChainNode;
        return successorChainNode;
    }

    public ChainNode<T> getLastSuccessor(){
        if( this.successor == null){
            return this;
        }else{
            return this.successor.getLastSuccessor();
        }
    }

    public ChainNode<T> setOrAppendLast(T node){
        if( this.node == null){
            this.node = node;
            return this;
        }

        ChainNode<T> lastSuccessor = this.getLastSuccessor();
        if( lastSuccessor.node == null){
            lastSuccessor.node = node;
            return lastSuccessor;
        }

        return lastSuccessor.append(node);
    }

    public void walkThroughLast(Consumer<T> consumer){
        if( this.node != null){
            consumer.accept(this.node);
        }
        if( this.successor != null){
            this.successor.walkThroughLast(consumer);
        }
    }

    public List<ChainNode<T>> walkThroughLast(){
        return walkThroughLast(new ArrayList<>());
    }
    public List<ChainNode<T>> walkThroughLast(List<ChainNode<T>> list){
        if( list == null){
            list = new ArrayList<>();
        }
        list.add(this);
        if( this.successor == null){
            return list;
        }else{
            return this.successor.walkThroughLast(list);
        }
    }

}