package pps.core.common.pdf;

import cn.hutool.core.io.file.FileReader;
import com.itextpdf.text.BaseColor;
import com.itextpdf.text.Document;
import com.itextpdf.text.DocumentException;
import com.itextpdf.text.Image;
import com.itextpdf.text.pdf.*;
import com.itextpdf.tool.xml.XMLWorkerHelper;
import gui.ava.html.parser.HtmlParser;
import gui.ava.html.parser.HtmlParserImpl;
import gui.ava.html.renderer.ImageRenderer;
import gui.ava.html.renderer.ImageRendererImpl;
import org.apache.commons.codec.binary.Base64;
import org.springframework.util.ResourceUtils;
import sun.misc.BASE64Decoder;
import xstartup.base.util.XDateUtils;
import xstartup.base.util.XStringUtils;

import java.io.*;
import java.nio.charset.Charset;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

public class Itext5HtmlToPDF {
    public static void main(String[] args) throws Exception {
        String base64 = htmlToPDF("<!DOCTYPE html>\n" +
                "<html lang=\"en\">\n" +
                "<head>\n" +
                "\t<meta charset=\"utf-8\"/>\n" +
                "\t<title>PPS</title>\n" +
                "\t<style>\n" +
                "\t\tbody {\n" +
                "\t\t\tfont-family: sans-serif, 'Microsoft Yahei';\n" +
                "\t\t}\n" +
                "\n" +
                "\t\t.container {\n" +
                "\t\t/*\twidth: 1240px;\n" +
                "\t\t\theight: 1754px;\n" +
                "\t\t\tmargin: 0 auto;*/\n" +
                "\t\t\tpadding: 16px;\n" +
                "\t\t\tbox-sizing: border-box;\n" +
                "\t\t\tbackground-color: #ffffff;\n" +
                "\t\t\tline-height: 1.5;\n" +
                "\t\t\tfont-size: 16px;\n" +
                "\t\t\tfont-weight: bold;\n" +
                "\t\t}\n" +
                "\n" +
                "\t\t.container h1 {\n" +
                "\t\t\ttext-align: center;\n" +
                "\t\t\tline-height: 1;\n" +
                "\t\t\tfont-size: 28px;\n" +
                "\t\t\tfont-weight: 900;\n" +
                "\t\t}\n" +
                "\n" +
                "\t\t.container .datecode {\n" +
                "\t\t\toverflow: hidden;\n" +
                "\t\t}\n" +
                "\n" +
                "\t\t.container .datecode span {\n" +
                "\t\t\tfloat: left;\n" +
                "\t\t}\n" +
                "\n" +
                "\t\t.container .datecode span:last-child {\n" +
                "\t\t\tfloat: right;\n" +
                "\t\t}\n" +
                "\t\t.left{\n" +
                "\t\t\ttext-align: left;\n" +
                "\t\t}\n" +
                "\t\ttable {\n" +
                "\t\t\twidth: 100%;\n" +
                "\t\t\tborder-collapse: collapse;\n" +
                "\t\t\tborder-spacing: 0;\n" +
                "\t\t}\n" +
                "\n" +
                "\t\ttable, th, td {\n" +
                "\t\t\tline-height: 2;\n" +
                "\t\t\tborder: 1px solid #000000;\n" +
                "\t\t\ttext-align: center;\n" +
                "\t\t\tpadding: 0 5px;\n" +
                "\t\t}\n" +
                "\n" +
                "\t\tth {\n" +
                "\t\t\twidth: auto;\n" +
                "\t\t\tfont-weight: bold;\n" +
                "\t\t}\n" +
                "\n" +
                "\t\ttd {\n" +
                "\t\t\twidth: auto;\n" +
                "\t\t}\n" +
                "\t</style>\n" +
                "</head>\n" +
                "<body>\n" +
                "<div class=\"container\">\n" +
                "\t<h1>国家管网集团联合管道有限责任公司西气东输分公司</h1>\n" +
                "\t<h1>天然气计量交接凭证（中卫首站）</h1>\n" +
                "\t<div class=\"datecode\">\n" +
                "\t\t<span>填写日期：2022-07-07</span>\n" +
                "\t\t<span>凭证编号：185--220708</span>\n" +
                "\t</div>\n" +
                "\t<table cellspacing=\"0\">\n" +
                "\t\t<tr>\n" +
                "\t\t\t<td class=\"left\" colspan=\"2\">承运单位：国家管网集团联合管道有限责任公司西气东输分公司</td>\n" +
                "\t\t\t<td class=\"left\" colspan=\"2\">托运单位：</td>\n" +
                "\t\t\t<td class=\"left\" colspan=\"3\">接收单位：江苏华电昆山热电有限公司</td>\n" +
                "\t\t</tr>\n" +
                "\t\t<tr>\n" +
                "\t\t\t<td class=\"left\" colspan=\"7\">计量时间：2022年7月5日 08:00 至 2022年7月6日 08:00</td>\n" +
                "\t\t</tr>\n" +
                "\t\t<tr>\n" +
                "\t\t\t<th>计量交接地点</th>\n" +
                "\t\t\t<th>流量计位号</th>\n" +
                "\t\t\t<th>前日日累（体积量）Nm3</th>\n" +
                "\t\t\t<th>前日日累（能量）MJ</th>\n" +
                "\t\t\t<th>平均高位发热量MJ/m3</th>\n" +
                "\t\t\t<th>计量气量Nm3</th>\n" +
                "\t\t\t<th>计量能量MJ</th>\n" +
                "\t\t</tr>\n" +
                "\t\t<tr>\n" +
                "\t\t\t<td>中卫首站</td>\n" +
                "\t\t\t<td>FM501</td>\n" +
                "\t\t\t<td>156140210</td>\n" +
                "\t\t\t<td>156140210</td>\n" +
                "\t\t\t<td>6436881280</td>\n" +
                "\t\t\t<td>11009989685</td>\n" +
                "\t\t\t<td>13242421311</td>\n" +
                "\t\t</tr>\n" +
                "\t\t<tr>\n" +
                "\t\t\t<td>中卫首站</td>\n" +
                "\t\t\t<td>FM501</td>\n" +
                "\t\t\t<td>156140210</td>\n" +
                "\t\t\t<td>156140210</td>\n" +
                "\t\t\t<td>6436881280</td>\n" +
                "\t\t\t<td>11009989685</td>\n" +
                "\t\t\t<td>13242421311</td>\n" +
                "\t\t</tr>\n" +
                "\t\t<tr>\n" +
                "\t\t\t<td>中卫首站</td>\n" +
                "\t\t\t<td>FM501</td>\n" +
                "\t\t\t<td>156140210</td>\n" +
                "\t\t\t<td>156140210</td>\n" +
                "\t\t\t<td>6436881280</td>\n" +
                "\t\t\t<td>11009989685</td>\n" +
                "\t\t\t<td>13242421311</td>\n" +
                "\t\t</tr>\n" +
                "\t\t<tr>\n" +
                "\t\t\t<td>中卫首站</td>\n" +
                "\t\t\t<td>FM501</td>\n" +
                "\t\t\t<td>156140210</td>\n" +
                "\t\t\t<td>156140210</td>\n" +
                "\t\t\t<td>6436881280</td>\n" +
                "\t\t\t<td>11009989685</td>\n" +
                "\t\t\t<td>13242421311</td>\n" +
                "\t\t</tr>\n" +
                "\t\t<tr>\n" +
                "\t\t\t<td colspan=\"2\">合计交接气量（小写）Nm3</td>\n" +
                "\t\t\t<td colspan=\"2\">94010</td>\n" +
                "\t\t\t<td>合计交接能量（小写）MJ</td>\n" +
                "\t\t\t<td colspan=\"2\">94010</td>\n" +
                "\t\t</tr>\n" +
                "\t\t<tr>\n" +
                "\t\t\t<td colspan=\"2\">合计交接气量（大写）Nm3</td>\n" +
                "\t\t\t<td colspan=\"2\">玖万肆仟零壹拾标方</td>\n" +
                "\t\t\t<td>合计交接能量（大写）MJ</td>\n" +
                "\t\t\t<td colspan=\"2\">玖万肆仟零壹拾标方</td>\n" +
                "\t\t</tr>\n" +
                "\t\t<tr>\n" +
                "\t\t\t<td>备注：</td>\n" +
                "\t\t\t<td colspan=\"6\"></td>\n" +
                "\t\t</tr>\n" +
                "\t\t<tr>\n" +
                "\t\t\t<td colspan=\"2\">供气单位（盖章）</td>\n" +
                "\t\t\t<td colspan=\"2\" style=\"opacity: 0;\">供气</td>\n" +
                "\t\t\t<td>接气单位(盖章)</td>\n" +
                "\t\t\t<td colspan=\"2\"></td>\n" +
                "\t\t</tr>\n" +
                "\t\t<tr>\n" +
                "\t\t\t<td colspan=\"2\">供气方计量员（签字）</td>\n" +
                "\t\t\t<td colspan=\"2\"></td>\n" +
                "\t\t\t<td>接气方计量员(签字)</td>\n" +
                "\t\t\t<td colspan=\"2\"></td>\n" +
                "\t\t</tr>\n" +
                "\t\t<tr>\n" +
                "\t\t\t<td>争议量（+/-）：</td>\n" +
                "\t\t\t<td colspan=\"6\"></td>\n" +
                "\t\t</tr>\n" +
                "\t</table>\n" +
                "</div>\n" +
                "</body>\n" +
                "</html>\n" , new HashMap<>());
        saveBase64strToFile(base64 , "D:\\pdf\\test"+ new Date().getTime() +".pdf");
    }

    public static String replaceContent(String htmlContent , Map<String,String> map){
        for (Map.Entry<String, String> entry : map.entrySet()) {
            htmlContent = htmlContent.replaceAll(entry.getKey() , entry.getValue());
        }
        return htmlContent;
    }

    public static ByteArrayOutputStream htmlSaveImage(String htmlContent){
        HtmlParser htmlParser = new HtmlParserImpl();
        //加载html模版
        htmlParser.loadHtml(htmlContent);
        //把html写入到图片
        ImageRenderer imageRenderer = new ImageRendererImpl(htmlParser);
        imageRenderer.setWidth(2048);
        imageRenderer.setHeight(1536);
        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
        imageRenderer.saveImage(byteArrayOutputStream ,true);
        return byteArrayOutputStream;
    }

    public static String imageToPDF(ByteArrayOutputStream imageByte) throws Exception{
        Document document = new Document();
        //第二步，为该Document创建一个Writer实例：
        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
        PdfWriter writer = PdfWriter.getInstance(document , byteArrayOutputStream);
        //第三步，打开当前Document
        document.open();
        //第四步，为当前Document添加内容：
        //替换内容
        document.newPage();  //在pdf创建一页
        Image png1 = Image.getInstance(imageByte.toByteArray()); //通过文件路径获取image
        float heigth = png1.getHeight();
        float width = png1.getWidth();
        int percent = getPercent2(heigth, width);
        png1.setAlignment(Image.MIDDLE);
        png1.scalePercent(percent + 3);// 表示是原来图像的比例;
        document.add(png1);
        //第五步，关闭Document
        document.close();
        byte[] buffer =byteArrayOutputStream.toByteArray();
        InputStream sbs = new ByteArrayInputStream(buffer);
        return getBase64FromInputStream(sbs);
    }

    public static String htmlToPDF(String htmlContent , Map<String,String> map ) throws Exception{
        Document document = new Document();
        //第二步，为该Document创建一个Writer实例：
        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
        PdfWriter writer = PdfWriter.getInstance(document , byteArrayOutputStream);
        //第三步，打开当前Document
        document.open();
        //第四步，为当前Document添加内容：
        //替换内容

        XMLWorkerHelper.getInstance().parseXHtml(writer, document,
                new ByteArrayInputStream(htmlContent.getBytes("UTF-8")) ,
                null , Charset.defaultCharset(),new FontProviderUtil());
        //第五步，关闭Document
        document.close();
        byte[] buffer =byteArrayOutputStream.toByteArray();
        InputStream sbs = new ByteArrayInputStream(buffer);
        return getBase64FromInputStream(sbs);
    }

    public static String getBase64FromInputStream(InputStream in) {
        // 将图片文件转化为字节数组字符串，并对其进行Base64编码处理
        byte[] data = null;
        // 读取图片字节数组
        try {
            ByteArrayOutputStream swapStream = new ByteArrayOutputStream();
            byte[] buff = new byte[100];
            int rc = 0;
            while ((rc = in.read(buff, 0, 100)) > 0) {
                swapStream.write(buff, 0, rc);
            }
            data = swapStream.toByteArray();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (in != null) {
                try {
                    in.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return new String(Base64.encodeBase64(data));
    }

    public static void saveBase64strToFile(String base64str , String outPath){
        if (base64str == null){
            return ;
        }
        BASE64Decoder decoder = new BASE64Decoder();
        try
        {
            byte[] b = decoder.decodeBuffer(base64str);
            for(int i=0;i<b.length;++i)
            {
                if(b[i]<0){
                    b[i]+=256;
                }
            }
            OutputStream out = new FileOutputStream(outPath);
            out.write(b);
            out.flush();
            out.close();
        }
        catch (Exception e){
            e.printStackTrace();
        }
    }
    public static int getPercent2(float h, float w) {
        int p = 0;
        float p2 = 0.0f;
        p2 = 530 / w * 100;
        p = Math.round(p2);
        return p;
    }
}
