package pps.core.common.provider.impl;

import com.google.common.base.Strings;
import pps.core.common.cache.ConfigCache;
import pps.core.common.utils.CounterBuilder;
import pps.core.common.utils.TraceHelper;
import pps.core.system.service.ConfigService;
import pps.core.system.service.data.GetConfigInput;
import pps.core.system.service.data.GetConfigOutput;
import xstartup.annotation.XImplement;
import xstartup.base.XContext;
import xstartup.base.util.XJsonUtils;
import xstartup.core.base.model.XNaming;
import xstartup.data.XSingleResult;
import xstartup.kernel.system.provider.spec.XConfigProvider;

/**
 * @author lixueyan
 * @date 2022/8/5 0005 17:11
 */
@XImplement
public class PpsConfigProviderCommonImpl implements XConfigProvider {
    //private static Map<String,String> configMap = new ConcurrentHashMap();

    @Override
    public String findValue(XContext context, String config, String app, String key) {
        //todo 实现自定义配置逻辑， 优先级 从低到高  数据库->properties文件
        //从Properties文件中加载配置值
        String configKey = config + "." + key;
        String configKeyWithAppName = config + "." + app + "." + key;
        String configValue = context.getProperty(configKeyWithAppName);
        if (Strings.isNullOrEmpty(configValue)) {
            configValue = context.getProperty(configKey);
        }

        //加载缓存
        final String configValue_ = configValue;
        ConfigCache configCache = ConfigCache.get(context, configKey, false, () -> {
            try {
                ConfigService configService = context.getBean(XNaming.build("pps-base-info"), ConfigService.class);
                GetConfigInput input = new GetConfigInput();
                input.setConfigKey(configKey);
                TraceHelper.appendTraceLog(context, "配置加载（服务）", "[INPUT]" + XJsonUtils.toJson(input));
                XSingleResult<GetConfigOutput> config1 = configService.getConfig(context, input);
                TraceHelper.appendTraceLog(context, "", "[RESULT]" + XJsonUtils.toJson(config1));
                if (config1.isSuccess()) {
                    CounterBuilder.globalCounterBuilder.addCounter("getConfig-Success");
                    // 从配置服务中加载
                    return config1.getResult().getConfigValue();
                } else {
                    CounterBuilder.globalCounterBuilder.addCounter("getConfig-Failed-" + configKey);
                    // 使用Properties配置文件配置
                    return configValue_;
                }
            } catch (Exception ex) {
                CounterBuilder.globalCounterBuilder.addCounter(ex, this.getClass());
                return configValue_;
            }
        });

        //返回缓存缺省值
        return configCache == null ? configValue_ : configCache.getConfigValue();
    }

}
