package pps.core.common.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.apache.commons.lang3.StringUtils;
import pps.core.common.entity.SysFileAttachmentEnt;
import pps.core.common.mapper.SysFileAttachmentMapper;
import pps.core.common.service.data.sys_file_attachment.*;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;

import java.util.List;

@XText("附件接口")
@XService
public class SysFileAttachmentService {

    @XText("删除附件")
    @XApiPost
    public XServiceResult deleteSysFileAttachment(XContext context, DeleteSysFileAttachmentInput input){
        SysFileAttachmentMapper mapper = context.getBean(SysFileAttachmentMapper.class);
        QueryWrapper<SysFileAttachmentEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysFileAttachmentEnt::getId, input.getId());
        SysFileAttachmentEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        mapper.deleteById(entity);
        return XServiceResult.OK;
    }

    @XText("更新附件业务id")
    @XApiPost
    public XServiceResult updateDispTaskPlanAttach(XContext context, UpdateSysFileAttachmentInput input){
        SysFileAttachmentMapper mapper = context.getBean(SysFileAttachmentMapper.class);
        QueryWrapper<SysFileAttachmentEnt> queryWrapper = new QueryWrapper<>();
        List<String> fileKeyList = input.getFileKeyList();
        for (String fileKey : fileKeyList) {
            queryWrapper.lambda().eq(SysFileAttachmentEnt::getFileKey, fileKey);
            SysFileAttachmentEnt entity = mapper.selectOne(queryWrapper);
            if(entity != null && StringUtils.isBlank(entity.getBsnsId())) {
                entity.setBsnsId(input.getBsnsId());
                mapper.updateById(entity);
            }
        }
        return XServiceResult.OK;
    }

    @XText("获取单个附件")
    @XApiPost
    public XSingleResult<GetSysFileAttachmentOutput> getSysFileAttachment(XContext context, GetSysFileAttachmentInput input){
        SysFileAttachmentMapper mapper = context.getBean(SysFileAttachmentMapper.class);
        QueryWrapper<SysFileAttachmentEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysFileAttachmentEnt::getId, input.getId());
        SysFileAttachmentEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetSysFileAttachmentOutput output = new GetSysFileAttachmentOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

    @XText("获取业务附件")
    @XApiPost
    public XPageResult<QuerySysFileAttachmentOutput> querySysFileAttachment(XContext context, QuerySysFileAttachmentInput input){
        SysFileAttachmentMapper mapper = context.getBean(SysFileAttachmentMapper.class);
        QueryWrapper<SysFileAttachmentEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysFileAttachmentEnt::getBsnsId, input.getBsnsId());
        return XMapperHelper.query(mapper, input, queryWrapper, QuerySysFileAttachmentOutput.class);
    }

    @XText("获取指定附件")
    @XApiPost
    public XPageResult<QuerySysFileAttachmentByIdsOutput> querySysFileAttachmentByIds(XContext context, QuerySysFileAttachmentByIdsInput input){
        SysFileAttachmentMapper mapper = context.getBean(SysFileAttachmentMapper.class);
        QueryWrapper<SysFileAttachmentEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().in(SysFileAttachmentEnt::getFileKey, input.getFileKeyList());
        queryWrapper.lambda().orderByAsc(SysFileAttachmentEnt::getCreateTime);
        input.setLimit(100);
        return XMapperHelper.query(mapper,input, queryWrapper, QuerySysFileAttachmentByIdsOutput.class);
    }

}
