package pps.core.common.utils;

import org.jxls.common.Context;
import org.jxls.expression.JexlExpressionEvaluator;
import org.jxls.transform.Transformer;
import org.jxls.util.JxlsHelper;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.nio.file.Files;
import java.util.HashMap;
import java.util.Map;

/**
 * Description:模板导出工具类
 * date: 2020/8/20 18:56
 *
 * @author zjw
 */
public class ExportExcelUtils {

    public static void outputFileData(HttpServletResponse response, byte[] fileBuff, String fileName) throws Exception{
        response.setContentType("application/octet-stream;charset=GBK");
        response.setHeader("Content-disposition", "attachment;filename=" + new String(fileName.getBytes("GBK"),"iso-8859-1"));
        OutputStream out = null;
        try {
            out = response.getOutputStream();
            out.write(fileBuff);
            out.flush();
        } catch (IOException e) {
            e.printStackTrace();
        }finally{
            if (out!=null)
                out.close();
        }
    }


    public static void outputFileData(byte[] fileBuff, String filePath) throws Exception{
        File file = new File(filePath);
        if (!file.getParentFile().exists()) {
            file.getParentFile().mkdirs();
        }
        OutputStream out = new FileOutputStream(file);
        try {
            out.write(fileBuff);
            out.flush();
        } catch (IOException e) {
            e.printStackTrace();
        }finally{
            if (out!=null)
                out.close();
        }
    }

    /**
     * 根据参数生成excel文件的二进制数据
     * @param tempPath	excel模板路径
     * @param params	模板中相关参数的Map
     * @return			生成文件的byte数组
     */
    public static byte[] genSingleExcelFileData(String tempPath,Map<String,Object> params){
        FileInputStream fileInputStream;
        try {
            fileInputStream = new FileInputStream(new File(tempPath));
        } catch (FileNotFoundException e) {
            System.out.println("-----" + tempPath + "文件未找到，导出空数据");
            return null;
        }
        Context context = new Context();
        if (params != null) {
            for (String key : params.keySet()) {
                context.putVar(key, params.get(key));
            }
        }
        ByteArrayOutputStream buff = new ByteArrayOutputStream();
        try {
            JxlsHelper jxlsHelper = JxlsHelper.getInstance();
            Transformer transformer = jxlsHelper.createTransformer(fileInputStream, buff);
            JexlExpressionEvaluator evaluator = (JexlExpressionEvaluator)transformer.getTransformationConfig().getExpressionEvaluator();
            Map<String, Object> funcs = new HashMap<String, Object>();
            funcs.put("utils", new JxlsUtils()); //添加自定义功能
            evaluator.getJexlEngine().setFunctions(funcs);
            jxlsHelper.processTemplate(context, transformer);
        } catch (IOException e) {
            e.printStackTrace();
        }
        return buff.toByteArray();
    }


    /**
     * 根据参数生成excel文件的二进制数据
     * @param inputStream	excel模板路径
     * @param params	模板中相关参数的Map
     * @return			生成文件的byte数组
     */
    public static byte[] genSingleExcelFileData(InputStream inputStream,Map<String,Object> params){
        Context context = new Context();
        if (params != null) {
            for (String key : params.keySet()) {
                context.putVar(key, params.get(key));
            }
        }
        ByteArrayOutputStream buff = new ByteArrayOutputStream();
        try {
            JxlsHelper jxlsHelper = JxlsHelper.getInstance();
            Transformer transformer = jxlsHelper.createTransformer(inputStream, buff);
            JexlExpressionEvaluator evaluator = (JexlExpressionEvaluator)transformer.getTransformationConfig().getExpressionEvaluator();
            Map<String, Object> funcs = new HashMap<String, Object>();
            funcs.put("utils", new JxlsUtils()); //添加自定义功能
            evaluator.getJexlEngine().setFunctions(funcs);
            jxlsHelper.processTemplate(context, transformer);
        } catch (IOException e) {
            e.printStackTrace();
        }
        return buff.toByteArray();
    }
}
