package pps.core.common.utils;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.atomic.AtomicLong;

public enum GlobalExecutorService {
    INSTANCE;

    private final ExecutorService executor = new ThreadPoolExecutor(10, 100, 100, TimeUnit.SECONDS, new LinkedBlockingQueue<>());

    private AtomicLong runningTaskCounter = new AtomicLong(0);
    private AtomicLong taskCounter = new AtomicLong(0);

    public void execute(Runnable runnable){
        this.executor.execute(runnable);
        runningTaskCounter.getAndAdd(1);
        taskCounter.getAndAdd(1);
    }

    public void executeDone(int count){
        runningTaskCounter.addAndGet(-count);
    }

    public long getRunningTasksCount(){
        return runningTaskCounter.get();
    }
    public long getTasksCount(){
        return taskCounter.get();
    }

}
