package pps.core.common.utils;

import com.google.common.base.Strings;

public class NameConvertHelper {

    public static boolean isEmpty(String s) {
        return s == null || s.length() == 0;
    }

    //abc-def-ghi   -> AbcDefGhi
    public static String pathToClass(String path) {
        String[] segments = path.split("-");
        StringBuilder builder = new StringBuilder(path.length());

        for (int index = 0; index < segments.length; ++index) {
            if (segments[index].length() > 1) {
                char letter = segments[index].charAt(0);
                builder.append(Character.toUpperCase(letter));
                builder.append(segments[index].substring(1).toLowerCase());
            } else {
                builder.append(segments[index].toUpperCase());
            }
        }

        return builder.toString();
    }

    //abc-def-ghi   -> abcDefGhi
    public static String pathToCamel(String value) {
        String[] segments = value.split("-");
        if (segments.length == 1) {
            return segments[0];
        } else {
            StringBuilder builder = new StringBuilder(value.length() + 5);

            for (int index = 0; index < segments.length; ++index) {
                if (segments[index].length() > 1) {
                    char letter = segments[index].charAt(0);
                    if (builder.length() == 0) {
                        builder.append(Character.toLowerCase(letter));
                    } else {
                        builder.append(Character.toUpperCase(letter));
                    }

                    builder.append(segments[index].substring(1).toLowerCase());
                } else {
                    builder.append(segments[index].toUpperCase());
                }
            }

            return builder.toString();
        }
    }

    // abc_def   -> abcDef
    public static String columnToCamel(String value) {
        String[] segments = value.split("_");
        StringBuilder builder = new StringBuilder(value.length());

        for (int index = 0; index < segments.length; ++index) {
            if (segments[index].length() > 1) {
                char letter = segments[index].charAt(0);
                if (builder.length() == 0) {
                    builder.append(Character.toLowerCase(letter));
                } else {
                    builder.append(Character.toUpperCase(letter));
                }

                builder.append(segments[index].substring(1).toLowerCase());
            } else {
                builder.append(segments[index].toUpperCase());
            }
        }

        return builder.toString();
    }

    // Abc  -> abc
    public static String toCamel(String value) {
        if (isEmpty(value)) {
            return value;
        } else {
            return 'A' <= value.charAt(0) && 'Z' >= value.charAt(0) //首字母大写？
                    ? String.format("%s%s", value.substring(0, 1).toLowerCase(), value.substring(1))
                    : value;
        }
    }

    //abc     -> Abc
    public static String toClass(String value) {
        if (isEmpty(value)) {
            return value;
        } else {
            return 'a' <= value.charAt(0) && 'z' >= value.charAt(0) //首字母小写
                    ? String.format("%s%s", value.substring(0, 1).toUpperCase(), value.substring(1))
                    : value;
        }
    }

    //AbcDef   -> abc-def
    public static String toPath(String value) {
        if (Strings.isNullOrEmpty(value)) {
            return value;
        }

        StringBuilder stringBuilder = new StringBuilder(value.length() + 5);
        int startIndex = 0;
        int index = 0;
        for (index = 0; index < value.length(); ++index) {
            if (Character.isUpperCase(value.charAt(index))) {
                if (stringBuilder.length() == 0) {
                    stringBuilder.append(value.substring(startIndex, index).toLowerCase());
                } else {
                    stringBuilder.append("-");
                    stringBuilder.append(value.substring(startIndex, index).toLowerCase());
                }

                startIndex = index;
            }
        }

        if (startIndex != index) {
            if (stringBuilder.length() == 0) {
                stringBuilder.append(value.substring(startIndex, index).toLowerCase());
            } else {
                stringBuilder.append("-");
                stringBuilder.append(value.substring(startIndex, index).toLowerCase());
            }
        }

        if (stringBuilder.length() == 0) {
            stringBuilder.append(value);
        }

        return stringBuilder.toString();
    }

    //AbcDef   -> abc_def
    public static String toColumn(String value) {
        StringBuilder stringBuilder = new StringBuilder(value.length() + 5);
        int startIndex = 0;
        int index;
        for (index = 0; index < value.length(); ++index) {
            if (Character.isUpperCase(value.charAt(index))) {
                if (stringBuilder.length() == 0) {
                    stringBuilder.append(value.substring(startIndex, index).toLowerCase());
                } else {
                    stringBuilder.append("_");
                    stringBuilder.append(value.substring(startIndex, index).toLowerCase());
                }

                startIndex = index;
            }
        }

        if (startIndex != index) {
            if (stringBuilder.length() == 0) {
                stringBuilder.append(value.substring(startIndex, index).toLowerCase());
            } else {
                stringBuilder.append("_");
                stringBuilder.append(value.substring(startIndex, index).toLowerCase());
            }
        }

        if (stringBuilder.length() == 0) {
            stringBuilder.append(value);
        }

        return stringBuilder.toString();
    }
}
