package pps.core.common.utils;

import com.google.common.base.Strings;
import xstartup.base.util.XJsonUtils;
import xstartup.base.util.XStringUtils;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;

public class SmartRecord extends LinkedHashMap<String, Object> {
    public SmartRecord() {

    }

    public SmartRecord(LinkedHashMap r) {
        if (r != null) {
            this.putAll(r);
        }
    }

    public Boolean boolValue(String fieldName) {
        return boolValue(fieldName, Boolean.FALSE);
    }
    public Boolean boolValue(String fieldName, Boolean defaultValue) {
        if (!this.containsKey(fieldName)) {
            return defaultValue;
        }

        Object v = this.get(fieldName);
        if (v == null) {
            return Boolean.FALSE;
        }

        String strV = v.toString();
        if (strV.equals("0") || strV.equals("false") || strV.equals("False")) {
            return Boolean.FALSE;
        }

        return Boolean.TRUE;
    }
    public String strValue(String fieldName) {
        return strValue(fieldName, "");
    }
    public String strValue(String fieldName, String defaultValue) {
        if (!this.containsKey(fieldName)) {
            return defaultValue;
        }

        Object v = this.get(fieldName);
        if (v == null) {
            return null;
        }

        return v.toString();
    }
    public Integer intValue(String fieldName) {
        return intValue(fieldName, null);
    }
    public Integer intValue(String fieldName, Integer defaultValue) {
        String strValue = strValue(fieldName, null);
        if (Strings.isNullOrEmpty(strValue)) {
            return defaultValue;
        } else {
            return Integer.parseInt(strValue);
        }
    }
    public Double doubleValue(String fieldName) {
        return doubleValue(fieldName, null);
    }
    public Double doubleValue(String fieldName, Double defaultValue) {
        String strValue = strValue(fieldName, null);
        if (Strings.isNullOrEmpty(strValue)) {
            return defaultValue;
        } else {
            return Double.parseDouble(strValue);
        }
    }

    public <T> T parseFromJson(String fieldName, Class<T> clz) {
        String json = strValue(fieldName);
        if (json == null || Strings.isNullOrEmpty(json)) {
            return null;
        }
        return XJsonUtils.toObject(json, clz);
    }

    public <T> T toObj(Class<T> clz) {
        String json = XJsonUtils.toJson(this.columnOrPathToClass());
        if (json == null || Strings.isNullOrEmpty(json)) {
            return null;
        }
        return XJsonUtils.toObject(json, clz);
    }

    public SmartRecord columnOrPathToClass(){
        SmartRecord r = new SmartRecord();
        this.keySet().forEach( k ->{
            r.put(XStringUtils.columnToCamel(k), this.get(k));
        });
        return r;
    }
    public static <T> List<T> toList(List<SmartRecord> queryResult,  Class<T> clz) {
        final List<T> list = new ArrayList<T>();
        queryResult.forEach(r -> {
            list.add(r.toObj(clz));
        });
        return list;
    }

    public static List<SmartRecord> buildSmartRecords(List<LinkedHashMap<String, Object>> queryResult) {
        final List<SmartRecord> list = new ArrayList<SmartRecord>();
        queryResult.forEach(r -> {
            list.add(new SmartRecord(r));
        });
        return list;
    }

    public String valueJoin(String... fieldNames) {
        StringBuilder sb = new StringBuilder();
        for (String fieldName : fieldNames) {
            String strValue = strValue(fieldName,"");
            if( sb.length() > 0){
                sb.append(",");
            }
            sb.append(strValue);
        }
        return sb.toString();
    }

}
