package pps.core.common.utils;

import com.google.common.base.Joiner;
import com.google.common.base.Splitter;
import com.google.common.base.Strings;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class SqlFilter {
    public static final List<String> BAD_KEYWORDS = Arrays.asList(
            "select", "update", "delete", "insert", "truncate"
            , "and", "or", "char", "into", "substr"
            , "ascii", "declare", "exec", "count", "master"
            , "drop", "execute", "table", "sitename", "xp_cmdshell"
            , "like", "from", "grant", "use", "group_concat"
            , "column_name", "information_schema.columns", "table_schema"
            , "union", "where", "order", "by", "add"
            , "all", "alter", "any", "as", "create"
            , "in", "join", "inner", "not", "set"
            , "--", "--;", ";--");
    public static final Pattern PATTERN_WHITESPACE = Pattern.compile("\\s+");
    public static final Splitter SPLITTER_WHITESPACE = Splitter.on(PATTERN_WHITESPACE);
    public static final Joiner JOINER_WHITESPACE = Joiner.on(" ");
    public static final Pattern BAD_FORM_PATTERN_KEYWORDS = Pattern.compile("^(" + String.join("|", BAD_KEYWORDS) + ")$", Pattern.CASE_INSENSITIVE);
    public static final String BAD_FORM_PATTERN_STR = "'|\\*|/|\\\\|--.*";//过滤掉的sql关键字，特殊字符前面需要加\\进行转义
    public static final Pattern BAD_FORM_PATTERN = Pattern.compile(BAD_FORM_PATTERN_STR, Pattern.CASE_INSENSITIVE);

    public static final String filterValue(String original) {
        if (Strings.isNullOrEmpty(original)) {
            return original;
        }
        original = original.trim();
        if (Strings.isNullOrEmpty(original)) {
            return original;
        }

        if (!PATTERN_WHITESPACE.matcher(original).find()) {
            //没有空格
            return filterValueNoSpace(original);
        }

        List<String> keywords = SPLITTER_WHITESPACE.splitToList(original);
        List<String> fixedKeywords = new ArrayList<String>(keywords.size());
        keywords.forEach(keyword -> {
            String fixed = filterValueNoSpace(keyword);
            if (!Strings.isNullOrEmpty(fixed)) {
                fixedKeywords.add(fixed);
            }
        });
        if (fixedKeywords.isEmpty()) {
            return "";
        }
        return JOINER_WHITESPACE.join(fixedKeywords);
    }

    private static String filterValueNoSpace(String original) {
        //关键字查询
        Matcher matcher = BAD_FORM_PATTERN_KEYWORDS.matcher(original);
        if (matcher.find()) {
            return "";
        }
        Matcher matcher1 = BAD_FORM_PATTERN.matcher(original);
        if( matcher1.find()){
            original = matcher1.replaceAll("");
        }
        return original;
    }
}
