package pps.core.common.utils;

import java.util.concurrent.ConcurrentHashMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public abstract class WellKnownRegexValidator {

    private static ConcurrentHashMap<WellKnownRegex, Pattern> patterns = new ConcurrentHashMap<>();

    private static Pattern getPattern(WellKnownRegex regex){
        if(patterns.containsKey(regex))
            return patterns.get(regex);
        else{
            Pattern ptn = Pattern.compile(regex.getExpression());
            patterns.put(regex, ptn);
            return ptn;
        }
    }

    public static boolean anyMaths(String txt, WellKnownRegex... regexs){
        for(WellKnownRegex regex : regexs ){
            if(getPattern(regex).matcher(txt).matches())
                return true;
        }
        return false;
    }

    public static boolean completelyMatches(String txt, WellKnownRegex... regexs){
        for(WellKnownRegex regex : regexs){
            if(!completelyMatch(txt, regex))
                return false;
        }
        return true;
    }

    public static boolean completelyMatch(String txt, WellKnownRegex regex){
        Matcher matcher = getPattern(regex).matcher(txt);
        if(!matcher.matches())
            return false;

        return matcher.group().equals(txt);
    }

    public static void main(String[] args) {
        System.out.println (WellKnownRegexValidator.completelyMatch("0.3", WellKnownRegex.FLOAT  ));
        System.out.println (WellKnownRegexValidator.completelyMatch("3", WellKnownRegex.INTEGER  ));
    }
}