package pps.core.system.cache;

import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import pps.core.common.utils.CounterBuilder;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.cache.XCacheLife;
import xstartup.cache.XCacheObject;
import xstartup.cache.XListCache;

import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 角色权限缓存
 */
public class SysDictionaryCache implements XCacheObject , XCacheLife {
    @XText("主键")
    private String id;

    @XText("父级ID")
    private String pid;

    @XText("名称")
    private String dicName;

    @XText("缩写")
    private String shortName;

    @XText("备注")
    private String remark;

    @XText("排序")
    private Integer sort;

    @XText("层级")
    private Integer levels;

    @XText("是否默认值")
    private Integer beDefault;

    @XText("是否含有下级")
    private Integer beHasChildren;

    @XText("0为系统字典，1应用字典")
    private Integer dicType;

    @XText("生效时间")
    private Date effectTime;

    @XText("失效时间")
    private Date endTime;

    @XText("创建人名称")
    private String createName;

    @XText("创建时间")
    private Date createTime;

    @XText("修改人名称")
    private String modifyName;

    @XText("修改时间")
    private Date modifyTime;

    @XText("父级编码")
    private String alias;

    @XText("字典编码")
    private String dicKey;

    @XText("是否全选")
    private String isAll;

    @XText("字典路径")
    private String dicPath;

    @XText("字典默认值")
    private String defaultValue;

    public String getDefaultValue() {
        return defaultValue;
    }

    public void setDefaultValue(String defaultValue) {
        this.defaultValue = defaultValue;
    }

    public String getId() {
        return id;
    }

    public void setId(String id) {
        this.id = id;
    }

    public String getPid() {
        return pid;
    }

    public void setPid(String pid) {
        this.pid = pid;
    }

    public String getDicName() {
        return dicName;
    }

    public void setDicName(String dicName) {
        this.dicName = dicName;
    }

    public String getShortName() {
        return shortName;
    }

    public void setShortName(String shortName) {
        this.shortName = shortName;
    }

    public String getRemark() {
        return remark;
    }

    public void setRemark(String remark) {
        this.remark = remark;
    }

    public Integer getSort() {
        return sort;
    }

    public void setSort(Integer sort) {
        this.sort = sort;
    }

    public Integer getLevels() {
        return levels;
    }

    public void setLevels(Integer levels) {
        this.levels = levels;
    }

    public Integer getBeDefault() {
        return beDefault;
    }

    public void setBeDefault(Integer beDefault) {
        this.beDefault = beDefault;
    }

    public Integer getBeHasChildren() {
        return beHasChildren;
    }

    public void setBeHasChildren(Integer beHasChildren) {
        this.beHasChildren = beHasChildren;
    }

    public Integer getDicType() {
        return dicType;
    }

    public void setDicType(Integer dicType) {
        this.dicType = dicType;
    }

    public Date getEffectTime() {
        return effectTime;
    }

    public void setEffectTime(Date effectTime) {
        this.effectTime = effectTime;
    }

    public Date getEndTime() {
        return endTime;
    }

    public void setEndTime(Date endTime) {
        this.endTime = endTime;
    }

    public String getCreateName() {
        return createName;
    }

    public void setCreateName(String createName) {
        this.createName = createName;
    }

    public Date getCreateTime() {
        return createTime;
    }

    public void setCreateTime(Date createTime) {
        this.createTime = createTime;
    }

    public String getModifyName() {
        return modifyName;
    }

    public void setModifyName(String modifyName) {
        this.modifyName = modifyName;
    }

    public Date getModifyTime() {
        return modifyTime;
    }

    public void setModifyTime(Date modifyTime) {
        this.modifyTime = modifyTime;
    }

    public String getAlias() {
        return alias;
    }

    public void setAlias(String alias) {
        this.alias = alias;
    }

    public String getDicKey() {
        return dicKey;
    }

    public void setDicKey(String dicKey) {
        this.dicKey = dicKey;
    }

    public String getIsAll() {
        return isAll;
    }

    public void setIsAll(String isAll) {
        this.isAll = isAll;
    }

    public String getDicPath() {
        return dicPath;
    }

    public void setDicPath(String dicPath) {
        this.dicPath = dicPath;
    }

    @Override
    public String getCacheKey() {
        return this.dicKey;
    }

    @Override
    public Integer getDuration() {
        //过期时间（秒）
        // 1天
//        return 24 * 3600;
        // 5分钟
        return 5 * 60;

    }

    /**
     * 检查缓存是否存在指定的code
     * @param context
     * @param alias
     * @param dicKey
     * @return
     */
    public static boolean exist(XContext context, String alias, String dicKey){
        return Tool.get(Tool.class).exist(context, alias, dicKey, SysDictionaryCache.class);
    }

    /**
     * 从缓存中读取
     * @param context
     * @param alias
     * @return
     */
    public static List<SysDictionaryCache> list(XContext context, String alias){
        return Tool.get(Tool.class).list(context, alias, SysDictionaryCache.class);
    }

    /**
     * 设置缓存
     * @param context
     * @param alias
     * @param caches
     */
    public static void set(XContext context, String alias, List<SysDictionaryCache> caches){
        CounterBuilder.globalCounterBuilder.addCounter("setCache");
        Tool.get(Tool.class).set(context, alias, caches);
    }

    /**
     * 删除缓存
     * @param context
     * @param alias
     */
    public static void delete(XContext context, String alias,String dicId){
        List<SysDictionaryCache> list = Tool.get(Tool.class).list(context, alias, SysDictionaryCache.class);
        if(ObjectUtils.isNotEmpty(list)){
            list = list.stream().filter((SysDictionaryCache b)-> !StringUtils.equals(b.getId(),dicId))
                    .collect(Collectors.toList());
        }
        CounterBuilder.globalCounterBuilder.addCounter("deleteCache");
        Tool.get(Tool.class).delete(context,alias,SysDictionaryCache.class);
        CounterBuilder.globalCounterBuilder.addCounter("setCache");
        Tool.get(Tool.class).set(context,alias,list);
    }

    /**
     * 删除缓存
     * @param context
     * @param alias
     */
    public static void deleteCache(XContext context, String alias){
        CounterBuilder.globalCounterBuilder.addCounter("deleteCache");
        Tool.get(Tool.class).delete(context,alias,SysDictionaryCache.class);
    }

    static class Tool extends XListCache<SysDictionaryCache> {
        @Override
        protected List<SysDictionaryCache> restore(XContext context, String listKey) {
            return null;
        }
    }
}
