package pps.core.system.module;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.google.common.base.Strings;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import pps.core.common.config.MqConfig;
import pps.core.common.entity.BaseSysLogEnt;
import pps.core.common.entity.BaseTransferLogEnt;
import pps.core.common.mapper.BaseSysLogMapper;
import pps.core.common.mapper.BaseTransferLogMapper;
import pps.core.common.mq.AccessLogMqMessage;
import pps.core.common.mq.SysInformSendMqMessage;
import pps.core.common.mq.TransferLogMqMessage;
import pps.core.common.provider.impl.PpsConfigProviderImpl;
import pps.core.common.utils.CommonTraceLogger;
import pps.core.common.utils.ManifestComponentInfoHelper;
import pps.core.system.cache.SysDictionaryCache;
import pps.core.system.entity.SysDictionaryEnt;
import pps.core.system.mapper.SysDictionaryMapper;
import pps.core.system.provider.impl.PpsPermissionProviderImpl;
import pps.core.system.provider.impl.PpsSsoProviderImpl;
import pps.core.system.provider.impl.PpsUserSessionInjectProviderImpl;
import pps.core.system.service.data.sys_inform.SendSysInformInput;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.XModule;
import xstartup.base.mq.*;
import xstartup.base.pattern.XProviderManager;
import xstartup.base.util.XCopyUtils;
import xstartup.base.util.XJsonUtils;
import xstartup.core.base.helper.XThreadHelper;
import xstartup.core.base.provider.spec.XPermissionProvider;
import xstartup.core.base.provider.spec.XSessionInjectProvider;
import xstartup.core.base.provider.spec.XSsoProvider;
import xstartup.data.XSingleResult;
import xstartup.kernel.system.provider.spec.XConfigProvider;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

@XText("系统模块")
public class PpsCoreSystemModule extends XModule {
    protected PpsCoreSystemModule() {
        super(1002);
    }

    @Override
    protected void onStarted(XContext context) {
        CommonTraceLogger.INSTANCE.start();
        XProviderManager.setDefaultImplement(context, XPermissionProvider.class, PpsPermissionProviderImpl.class);
        XProviderManager.setDefaultImplement(context, XSsoProvider.class, PpsSsoProviderImpl.class);
        XProviderManager.setDefaultImplement(context, XSessionInjectProvider.class, PpsUserSessionInjectProviderImpl.class);
        //配置信息 自定义实现类
        //XProviderManager.setDefaultImplement(context, XConfigProvider.class, PpsConfigProviderImpl.class);
        ManifestComponentInfoHelper.init(this.getClass().getClassLoader());

        SysDictionaryMapper dictionaryMapper = context.getBean(SysDictionaryMapper.class);
        //初始化字典缓存
        //查询所有alias不为空的
        Map<String, List<SysDictionaryCache>> dictCashMap = new HashMap();
        QueryWrapper<SysDictionaryEnt> queryDic = new QueryWrapper();
        queryDic.lambda().isNotNull(SysDictionaryEnt::getAlias).gt(SysDictionaryEnt::getEndTime, new Date());
        List<SysDictionaryEnt> sysDictionarys = dictionaryMapper.selectList(queryDic);
        sysDictionarys = sysDictionarys.stream().filter(p -> !StringUtils.equals("", p.getAlias())).collect(Collectors.toList());
        for (SysDictionaryEnt dictObj : sysDictionarys) {
            List<SysDictionaryCache> list = new ArrayList<>();
            SysDictionaryCache dictMap = new SysDictionaryCache();
            XCopyUtils.copyObject(dictObj, dictMap);
            list.add(dictMap);
            dictCashMap.put(dictObj.getAlias(), list);
        }
        //所有alias不为空的字典
        List<String> dictIds = sysDictionarys.stream().map(p -> {
            return p.getId();
        }).distinct().collect(Collectors.toList());
        int size = dictIds.size();
        while (size > 0) {
            //根据pid查询所有子字典项
            QueryWrapper<SysDictionaryEnt> queryDicChild = new QueryWrapper();
            queryDicChild.lambda().in(SysDictionaryEnt::getPid, dictIds).gt(SysDictionaryEnt::getEndTime, new Date());
            List<SysDictionaryEnt> sysDictionaryChilds = dictionaryMapper.selectList(queryDicChild);
            Set<String> aliasObjs = dictCashMap.keySet();
            //放入到缓存map中
            for (String aliasObj : aliasObjs) {
                List<SysDictionaryCache> sysDictionaryCaches = dictCashMap.get(aliasObj);
                Iterator<SysDictionaryCache> iterator = sysDictionaryCaches.iterator();
                if (iterator.hasNext()) {
                    if (ObjectUtils.isNotEmpty(sysDictionaryChilds)) {
                        SysDictionaryCache dict = iterator.next();
                        for (SysDictionaryEnt dictChild : sysDictionaryChilds) {
                            if (StringUtils.equals(dict.getId(), dictChild.getPid())) {
                                List<SysDictionaryCache> sysDictionaryList = dictCashMap.get(aliasObj);
                                SysDictionaryCache dictMap = new SysDictionaryCache();
                                XCopyUtils.copyObject(dictChild, dictMap);
                                sysDictionaryList.add(dictMap);
                                dictCashMap.put(aliasObj, sysDictionaryList);
                            }
                        }
                    }
                }
            }
            dictIds = sysDictionaryChilds.stream().map(p -> {
                return p.getId();
            }).distinct().collect(Collectors.toList());
            size = dictIds.size();
        }
        //放入到缓存redis
        Set<String> keys = dictCashMap.keySet();
        for (String key : keys) {
            List<SysDictionaryCache> sysDictionaryCaches = dictCashMap.get(key);
            SysDictionaryCache.deleteCache(context, key);
            SysDictionaryCache.set(context, key, sysDictionaryCaches);
        }


        XThreadHelper.async(() -> {
            this.fixDicPath(context);
        });

    }

    private void fixDicPath(XContext context) {
        SysDictionaryMapper mapper = context.getBean(SysDictionaryMapper.class);
        List<SysDictionaryEnt> sysDictionaryEnts = mapper.selectList(new QueryWrapper<>());
        //字典根
//        Optional<SysDictionaryEnt> root = sysDictionaryEnts.stream().filter(item -> Strings.isNullOrEmpty(item.getPid()) && item.getId().equals("1")).findFirst();
//        if (!root.isPresent()) {
//            return;
//        }
        final Map<String, SysDictionaryEnt> dics = sysDictionaryEnts.stream().collect(Collectors.toMap(SysDictionaryEnt::getId, Function.identity()));
        StringBuilder sb = new StringBuilder();
        sysDictionaryEnts.stream().forEach(item -> {
            sb.setLength(0);
            if (Strings.isNullOrEmpty(item.getPid())) {
                return;
            }
            if (!Strings.isNullOrEmpty(item.getDicPath())) {
                return;
            }

            buildPath(dics, item.getPid(), sb);
            String path = sb.toString();
            if (Strings.isNullOrEmpty(path)) {
                return;
            }

            if (path.equals(item.getDicPath())) {
                return;
            }

            SysDictionaryEnt dicForUpdate = new SysDictionaryEnt();
            dicForUpdate.setId(item.getId());
            dicForUpdate.setDicPath(path);
            mapper.updateById(dicForUpdate);
        });

    }

    private void buildPath(Map<String, SysDictionaryEnt> dics, String parentId, StringBuilder sb) {
        if (Strings.isNullOrEmpty(parentId) || "1".equals(parentId)) {
            return;
        }

        sb.insert(0, parentId);
        sb.insert(0, "/");
        if (!dics.containsKey(parentId)) {
            return;
        }

        SysDictionaryEnt parent = dics.get(parentId);
        if (parent == null) {
            return;
        }

        buildPath(dics, parent.getPid(), sb);

    }
}
