package pps.core.system.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.google.common.base.Strings;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.CollectionUtils;
import pps.cloud.system.service.SysOrganizationCloudService;
import pps.cloud.system.service.data.GetSysOrganizationViewInput;
import pps.cloud.system.service.data.GetSysOrganizationViewOutput;
import pps.core.common.entity.BaseSysConfigEnt;
import pps.core.common.mapper.BaseSysConfigMapper;
import pps.core.common.session.PpsUserSession;
import pps.core.system.service.data.GetConfigInput;
import pps.core.system.service.data.GetConfigOutput;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiPost;

import java.util.List;

@XText("配置服务")
@XService
public class ConfigServiceImpl implements ConfigService {

    @XApiPost
    @Override
    public XSingleResult<GetConfigOutput> getConfig(XContext context, GetConfigInput input) {
        try {
            //系统配置
            BaseSysConfigMapper configMapper = context.getBean(BaseSysConfigMapper.class);
            QueryWrapper<BaseSysConfigEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(BaseSysConfigEnt::getMainKeys, input.getConfigKey());
            List<BaseSysConfigEnt> baseSysConfigEnts = configMapper.selectList(queryWrapper);
            if(!CollectionUtils.isEmpty(baseSysConfigEnts)){
                try {
                    PpsUserSession session = context.getSession(PpsUserSession.class);
                    BaseSysConfigEnt configByOU = findConfigByOU(context, baseSysConfigEnts, session.getOuId());
                    GetConfigOutput output = new GetConfigOutput();
                    output.setConfigValue(configByOU.getValue());
                    return XSingleResult.success(output) ;
                }catch (Exception ex){

                }
                GetConfigOutput output = new GetConfigOutput();
                output.setConfigValue(baseSysConfigEnts.get(0).getValue());
                return XSingleResult.success(output) ;
            }else {
                return XSingleResult.error(context, XError.NotFound);
            }
        } catch (Exception ex) {
            return XSingleResult.error(ex);
        }
    }

    /**
     * 递归 向上查找是否有满足条件的组织机构
     * @param context
     * @param baseSysConfigEnts
     * @param ouId
     * @return
     */
    private BaseSysConfigEnt findConfigByOU(XContext context,List<BaseSysConfigEnt> baseSysConfigEnts,String ouId){
        if(baseSysConfigEnts.size()==1 && StringUtils.isBlank(baseSysConfigEnts.get(0).getOuId())){
            return baseSysConfigEnts.get(0);
        }
        BaseSysConfigEnt baseSysConfigEnt = baseSysConfigEnts.stream().filter(x -> x.getOuId() != null && x.getOuId().equals(ouId)).findFirst().orElse(null);
        if(baseSysConfigEnt==null){
            //获取父级机构
            SysOrganizationCloudService orgService = context.getBean(SysOrganizationCloudService.class);
            GetSysOrganizationViewInput input = new GetSysOrganizationViewInput();
            input.setId(ouId);
            XSingleResult<GetSysOrganizationViewOutput> organizationBySubOuId = orgService.getOrganizationBySubOuId(context, input);
            organizationBySubOuId.throwIfFail();
            String id = organizationBySubOuId.getResult().getId();
            if(StringUtils.isNotBlank(id)){
                return findConfigByOU(context,baseSysConfigEnts,id);
            }else {
                return baseSysConfigEnts.stream().filter(x -> Strings.isNullOrEmpty(x.getOuId())).findFirst().orElse(null);
            }
        }
        return baseSysConfigEnt;
    }
}
