package pps.core.system.service;


import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import pps.core.common.session.PpsUserSession;
import pps.core.common.utils.PatternUtil;
import pps.core.common.utils.RSAUtil;
import pps.core.common.utils.VerificationCode;
import pps.core.system.cache.FineReportThirdTokenCache;
import pps.core.system.cache.LoginSmsCodeCache;
import pps.core.system.cache.LoginVerCodeCache;
import pps.core.system.config.LoginConfig;
import pps.core.system.entity.*;
import pps.core.system.error.LoginError;
import pps.core.system.mapper.*;
import pps.core.system.service.data.FineReport.GetFineReport;
import pps.core.system.service.data.iam.GetIamInput;
import pps.core.system.service.data.sys_login.*;
import pps.core.system.utils.UserInfoUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.XSsoAuth;
import xstartup.base.XUser;
import xstartup.base.conf.XConfManager;
import xstartup.base.exception.XServiceException;
import xstartup.base.util.*;
import xstartup.core.base.provider.data.XToken;
import xstartup.data.XFileResult;
import xstartup.data.XRedirectResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.api.conf.XApiCookieConf;
import xstartup.helper.XTokenHelper;

import java.awt.image.BufferedImage;
import java.util.*;
import java.util.stream.Collectors;

import static pps.core.common.constant.LoginConstant.LOGIN_FLAG_SIMULATE;
import static pps.core.common.constant.LoginConstant.LOGIN_FLAG_TRADE;
import static pps.core.system.error.LoginError.NotExistUserName;

@XService
public class LoginService {

    /**
     * 生成third_token需要的key
     */
    private static final String KEY = "pppppppppppsssssssssssssssssssssssss";

    private static final String SPLIT = "_&_";

    /**
     * 多久过期
     */
    private static final int EXPIRE_TIME = 6000000;

    @XText("获取最后一次登录信息")
    @XApiGet
    public XSingleResult<GetLastLoginInfoOutput> getLastLoginInfo(XContext context, GetLastLoginInfoInput input) {
        context.log().info("input:{}", input);
        GetLastLoginInfoOutput output = new GetLastLoginInfoOutput();
        output.setIp(XDeviceUtils.getDefaultIp());
        return XSingleResult.success(output);
    }

    @XText("获取系统信息")
    @XApiGet(anonymous = true)
    public XSingleResult<GetSysInfoOutput> getSysInfo(XContext context, GetSysInfoInput input) {
        GetSysInfoOutput output = new GetSysInfoOutput();
        output.setEnv(context.getEnv().getText());
        return XSingleResult.success(output);
    }

    private boolean checkIsSendSmsCode(XContext context){
        Integer sendSmsCodeSwitch = new LoginConfig().sendSmsCodeSwitch.findValue(context);
        if(sendSmsCodeSwitch!=null && sendSmsCodeSwitch==1){
            return true;
        }
        return false;
    }

    @XText("登陆发送短信")
    @XApiGet(anonymous = true)
    public XServiceResult sendCode(XContext context, SendCodeInput input) {
        if(!checkIsSendSmsCode(context)){
            return XServiceResult.OK;
        }
        checkCode(context,input.getCode(),input.getCodeIden());
        XSingleResult<SysUserEnt> sysUserResult = checkAccount(context, input.getLoginName(), input.getPassword());
        SysUserEnt sysUserEnt = sysUserResult.getResult();
        String phone = sysUserEnt.getPhone();
        phone = UserInfoUtil.decrypt(phone);
        if(StringUtils.isBlank(phone)){
            return XServiceResult.error(context,LoginError.NotExistPhone);
        }
        if(!PatternUtil.checkPhone(phone)){
            return XServiceResult.error(context,LoginError.NotExistPhone);
        }
        Integer sendSmsCodeInterval = LoginConfig.sendSmsCodeInterval.findValue(context);
        if(sendSmsCodeInterval==null){
            sendSmsCodeInterval = 1;
        }
        LoginSmsCodeCache smsCodeCache = LoginSmsCodeCache.exist(context ,input.getLoginName());
        if(smsCodeCache!=null){
            Date currentDate = smsCodeCache.getCurrentDate();
            if(currentDate!=null){
                Date date = XDateUtils.addMunites(currentDate, sendSmsCodeInterval);
                if(XDateUtils.compare(date,new Date())>0){
                    return XServiceResult.error(context,LoginError.ISValidateSmsCode);
                }
            }
        }
        String smsContent = LoginConfig.smsContent.findValue(context);
        if(StringUtils.isBlank(smsContent)){
            smsContent = "您正在登陆生产运行管理系统，验证码为：{smsCode}，{valiteMinite}分钟有效！";
        }
        return XServiceResult.OK;
    }



    private XSingleResult<SysUserEnt> checkAccount(XContext context,String loginName,String password){
        SysUserMapper mapper = context.getBean(SysUserMapper.class);
        QueryWrapper<SysUserEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysUserEnt::getLoginName, loginName);
        SysUserEnt sysUserEnt = mapper.selectOne(queryWrapper);
        if(sysUserEnt == null) {
            insertError(context , loginName );
            Long count = getErrorTimes(context , loginName , sysUserEnt);
            throw new XServiceException(1000, "用户名或密码错误，还剩"+(5 - count)+"次机会");
        }
        if(StringUtils.isNotBlank(password)) {
            //password = MD5Util.encryptPassword(AESUtil.decrypt(password));
            String decrypt = sysUserEnt.getPassword();
            if (!password.equals(decrypt)) {
                insertError(context , loginName );
                Long count = getErrorTimes(context , loginName , sysUserEnt);
                throw new XServiceException(1000, "用户名或密码错误，还剩"+(5 - count)+"次机会");
            }
        }
        return XSingleResult.success(sysUserEnt);
    }

    private void checkSmsCode(XContext context,String userKey,String smsCode){
        if(!checkIsSendSmsCode(context)){
            return;
        }
        if(StringUtils.isBlank(smsCode)){
            throw new XServiceException(context,LoginError.NotValidateSmsCode);
        }
        LoginSmsCodeCache smsCodeCache = LoginSmsCodeCache.exist(context ,userKey);
        if(smsCodeCache!=null){
            String code = smsCodeCache.getCode();
            if(code.equals(smsCode)){
                LoginSmsCodeCache.delete(context,userKey);
               return;
            }
        }
        throw new XServiceException(context,LoginError.NotValidateSmsCode);
    }

    private void checkCode(XContext context,String code,String codeIden){
        if(!checkIsSendSmsCode(context)){
            return;
        }
        if(code == null) {
            throw new XServiceException(context,LoginError.NotExistCode);
        }
        LoginVerCodeCache isUse = LoginVerCodeCache.exist(context , codeIden);
        if(isUse == null) {
            throw new XServiceException(context,LoginError.NotUsedCode);
        }
        if(!isUse.getCode().toLowerCase().equals(code.toLowerCase())) {
            throw new XServiceException(context,LoginError.NotCompareCode);
        }
    }

    @XText("账号密码登录")
    @XApiPost(anonymous = true)
    public XSingleResult<LoginByAccountOutput> loginByAccount(XContext context, LoginByAccountInput input) {
        long startTime = new Date().getTime();
        Long count = getErrorTimes(context , input.getLoginName() , null);
        if(count >= 5){
            return XSingleResult.error(context,LoginError.ISLOCKED);
        }
        System.out.println("登录开始：----------------------------"+startTime);
        checkCode(context,input.getCode(),input.getCodeIden());
        checkSmsCode(context,input.getLoginName(),input.getSmsCode());
        XSingleResult<SysUserEnt> sysUserResult = checkAccount(context, input.getLoginName(), input.getPassword());
        SysUserEnt sysUserEnt = sysUserResult.getResult();
        if(sysUserEnt.getStatus() != null && sysUserEnt.getStatus() == 1 )
            return XSingleResult.error(context,LoginError.ISLOCKED);
        long getTime = new Date().getTime();
        if(sysUserEnt.getIsOuter() != null &&  sysUserEnt.getIsOuter() != 1)
            return XSingleResult.error(context,LoginError.ISOUT);
        Date now = new Date();
        SysUserOrganizationRelMapper relMapper = context.getBean(SysUserOrganizationRelMapper.class);
        QueryWrapper<SysUserOrganizationRelEnt> relQueryWrapper = new QueryWrapper<>();
        relQueryWrapper.lambda().eq(SysUserOrganizationRelEnt::getUserId, sysUserEnt.getId());
        relQueryWrapper.lambda().le(SysUserOrganizationRelEnt::getEffectTime, now);
        relQueryWrapper.lambda().gt(SysUserOrganizationRelEnt::getEndTime, now);
        SysUserOrganizationRelEnt sysUserOrganizationRelEnt = relMapper.selectOne(relQueryWrapper);
        if(sysUserOrganizationRelEnt==null){
            context.getLogger().warn("用户未挂靠组织");
            sysUserOrganizationRelEnt = new SysUserOrganizationRelEnt();
            sysUserOrganizationRelEnt.setOuId("1");
        }
        long getOrgTime = new Date().getTime();
        long getOrgTimec = getTime-getOrgTime;
        System.out.println("获取登录人组织机构信息：--------------------------上一步时间差--"+getOrgTimec+"------当前时间-"+getOrgTime);

        XApiCookieConf apiCookieConf = XConfManager.getConf(XApiCookieConf.class);
        Integer expiresIn = apiCookieConf.getExpires();

        String userName = sysUserEnt.getUserName(); //姓名
        Long orgId = 1L; //组织机构id
        Long roleId0 = 1L; //角色id1
        Long roleId1 = 1L; //角色id2
        Long userId = IdWorker.getId(); //用户id
        String userCode = sysUserEnt.getId();//这个是用户字符串类型的唯一标志
        String ssoToken = null; //sso系统授权的token
        String ssoRefreshToken = null; //sso系统授权的刷新token
        String uniqueId = XUuidUtils.randomUUID();
        XUser user = XUser.createUser(XUser.ENTRY_API,
                orgId,
                new Long[]{roleId0, roleId1},
                userId,
                userCode,
                sysUserEnt.getLoginName(),
                userName,
                new XSsoAuth(uniqueId, ssoToken, ssoRefreshToken));
        XSingleResult<XToken> saveUserResult = XTokenHelper.saveUser(context, user, expiresIn);
        saveUserResult.throwIfException();
        /*//todo:设置对应角色的权限缓存
        List list = new ArrayList();
        RolePermissionCache insertCache = new RolePermissionCache("insert","插入");
        RolePermissionCache updateCache = new RolePermissionCache("update","修改");
        list.add(insertCache);
        list.add(updateCache);
        for(Long roleId : user.getRoles()){
            RolePermissionCache.set(context, roleId, list);
        }*/
        LoginByAccountOutput output = new LoginByAccountOutput();
        output.setExpiresIn(expiresIn);
        output.setToken(saveUserResult.getResult().getToken());
        PpsUserSession session = context.getSession(PpsUserSession.class);
        output.setSession(session);
        long endTime = new Date().getTime();
        long cost = endTime - startTime;
        long costOrg = endTime - getOrgTime;
        System.out.println("登录结束：-------------------上一步时间差-"+costOrg+"--------方法总时间"+cost+"------当前时间-"+endTime);
        //登录成功后，删除失败记录
        deleteError(context , sysUserEnt);
        return XSingleResult.success(output);
    }



    @XText("忘记密码")
    @XApiPost(anonymous = true)
    public XSingleResult<ForgetPwdOutput> forgetPwd(XContext context, ForgetPwdInput input) {
        if(XStringUtils.isNotEmpty(input.getCode()))
            checkCode(context,input.getCode(),input.getCodeIden());
        Long count = getForgetErrorTimes(context);
        if(count >= 5){
            return XSingleResult.error(500 , "查询失败超过5次");
        }
        XSingleResult<SysUserEnt> sysUserResult = checkForgetAccount(context, input.getLoginName());
        SysUserEnt sysUserEnt = sysUserResult.getResult();
        String phone = sysUserEnt.getPhone();
        phone = UserInfoUtil.decrypt(phone);
        if(StringUtils.isBlank(phone)){
            return XSingleResult.error(context,LoginError.NotExistPhone);
        }

        if(!PatternUtil.checkPhone(phone)){
            return XSingleResult.error(context,LoginError.NotExistPhone);
        }

        ForgetPwdOutput output = new ForgetPwdOutput();
        output.setPhone(phone.substring(phone.length()-4));
        return XSingleResult.success(output);
    }



    @XText("修改密码")
    @XApiPost(anonymous = true)
    public XServiceResult modifyPwd(XContext context, ModifyPwdInput input) {
        checkSmsCode(context,input.getLoginName()+"-pwd",input.getSmsCode());

        XSingleResult<SysUserEnt> sysUserResult = checkAccount(context, input.getLoginName(), null);
        SysUserEnt sysUserEnt = sysUserResult.getResult();
        sysUserEnt.setPassword(input.getPassword());
        SysUserMapper userMapper = context.getBean(SysUserMapper.class);
        userMapper.updateById(sysUserEnt);
        return XServiceResult.OK;
    }

    @XText("模拟用户登陆")
    @XApiPost
    public XSingleResult<LoginByAccountOutput> loginBySimulate(XContext context, LoginBySimulateInput input) {
        String loginName = input.getLoginName(); //账号
        SysUserMapper mapper = context.getBean(SysUserMapper.class);
        QueryWrapper<SysUserEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysUserEnt::getLoginName, input.getLoginName());
        SysUserEnt sysUserEnt = mapper.selectOne(queryWrapper);
        if(sysUserEnt == null) {
            return XSingleResult.error(context,NotExistUserName);
        }

        Date now = new Date();
        SysUserOrganizationRelMapper relMapper = context.getBean(SysUserOrganizationRelMapper.class);
        QueryWrapper<SysUserOrganizationRelEnt> relQueryWrapper = new QueryWrapper<>();
        relQueryWrapper.lambda().eq(SysUserOrganizationRelEnt::getUserId, sysUserEnt.getId());
        relQueryWrapper.lambda().le(SysUserOrganizationRelEnt::getEffectTime, now);
        relQueryWrapper.lambda().gt(SysUserOrganizationRelEnt::getEndTime, now);
        SysUserOrganizationRelEnt sysUserOrganizationRelEnt = relMapper.selectOne(relQueryWrapper);
        if(sysUserOrganizationRelEnt==null){
            sysUserOrganizationRelEnt = new SysUserOrganizationRelEnt();
            sysUserOrganizationRelEnt.setOuId("1");
        }
        XApiCookieConf apiCookieConf = XConfManager.getConf(XApiCookieConf.class);
        Integer expiresIn = apiCookieConf.getExpires();
        String userName = sysUserEnt.getUserName(); //姓名
        Long orgId = LOGIN_FLAG_SIMULATE; //组织机构id
        Long roleId0 = 1L; //角色id1
        Long roleId1 = 1L; //角色id2
        Long userId = IdWorker.getId(); //用户id
        String userCode = sysUserEnt.getId();//这个是用户字符串类型的唯一标志
        String ssoToken = null; //sso系统授权的token
        String ssoRefreshToken = null; //sso系统授权的刷新token
        String uniqueId = XUuidUtils.randomUUID();
        XUser user = XUser.createUser(XUser.ENTRY_API,
                orgId,
                new Long[]{roleId0, roleId1},
                userId,
                userCode,
                loginName,
                userName,
                new XSsoAuth(uniqueId, ssoToken, ssoRefreshToken));
        XSingleResult<XToken> saveUserResult = XTokenHelper.saveUser(context, user, expiresIn);
        saveUserResult.throwIfException();

        LoginByAccountOutput output = new LoginByAccountOutput();
        output.setExpiresIn(expiresIn);
        output.setToken(saveUserResult.getResult().getToken());
        PpsUserSession session = context.getSession(PpsUserSession.class);
        output.setSession(session);
        return XSingleResult.success(output);
    }

    @XText("退出")
    @XApiPost
    public XServiceResult logout(XContext context){
        XUser user = context.getUser();
        XTokenHelper.deleteUser(context, user);
        context.updateToken(context.getWebRequest().getToken(),0);
        return XServiceResult.OK;
    }

    @XText("测试token是否有效")
    @XApiPost
    public XSingleResult<TestTokenOutput> testToken(XContext context) {
        PpsUserSession session = null;
        try {
            session = context.getSession(PpsUserSession.class);
            context.log().info("session:{}", session);
        }catch (XServiceException e){
            return XSingleResult.error(context,XError.NotAuth);
        }
        TestTokenOutput output = new TestTokenOutput();
        output.setUserId(session.getUserCode());
        output.setUserName(session.getUserName());
        return XSingleResult.success(output);
    }


    @XText("Iam重定向")
    @XApiAnonymous
    @XApiGet
    public XRedirectResult iamToLogin(XContext context , GetIamInput input) {
//        XWebRequest request = context.getWebRequest();
//        request.getHeader("host")
        return XRedirectResult.success(LoginConfig.iamToLoginUrl.getValue(context) + "?code=" + input.getCode() +"&client_secret=" + input.getClient_secret());
    }



    @XText("验证码")
    @XApiAnonymous
    @XApiGet
    public XFileResult getVerificationCode(XContext context , VerificationCodeInput input ) {
        Map<String, Object> codeMap = VerificationCode.generateCodeAndPic();
        // 将四位数字的验证码保存到Session中。
        LoginVerCodeCache cache = new LoginVerCodeCache();
        cache.setCodeKey(input.getCodeIden());
        cache.setCode(codeMap.get("code").toString());
        LoginVerCodeCache.set(context,cache);
        return XFileResult.createWithPng(input.getCodeIden()+".png" , (BufferedImage)codeMap.get("codePic"));
    }



    /**
     * 帆软获取third_token
     * @param context
     * @param input
     * @return
     */
    @XText("帆软获取third_token")
    @XApiPost
    public XSingleResult<GetFineReport> getThirdToken(XContext context , TestThirdToken input ) {
        PpsUserSession session = context.getSession(PpsUserSession.class);
        SysDictionaryMapper dictionaryMapper = context.getBean(SysDictionaryMapper.class);
        GetFineReport report = new GetFineReport();
        boolean exist = FineReportThirdTokenCache.exist(context, session.getLoginName(), session.getLoginName());
        if(exist){
            List<FineReportThirdTokenCache> list = FineReportThirdTokenCache.list(context, session.getLoginName());
            if(ObjectUtils.isNotEmpty(list)){
                FineReportThirdTokenCache cache = list.get(0);
                report.setThirdToken(cache.getCacheCode());
            }

        }else{
            String string = getString(context);
            System.out.println(string);
            /*String key = "";
            QueryWrapper<SysDictionaryEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(SysDictionaryEnt::getAlias,"FINE_REPORT_KEY")
            .gt(SysDictionaryEnt::getEndTime,new Date());
            List<SysDictionaryEnt> sysDictionaryEnts = dictionaryMapper.selectList(queryWrapper);
            if(ObjectUtils.isNotEmpty(sysDictionaryEnts)){
                SysDictionaryEnt dictionaryEnt = sysDictionaryEnts.get(0);
                if(ObjectUtils.isNotEmpty(dictionaryEnt)){
                    QueryWrapper<SysDictionaryEnt> query = new QueryWrapper<>();
                    query.lambda().eq(SysDictionaryEnt::getPid,dictionaryEnt.getId())
                            .gt(SysDictionaryEnt::getEndTime,new Date());
                    List<SysDictionaryEnt> dicList = dictionaryMapper.selectList(query);
                    if(ObjectUtils.isNotEmpty(dicList)){
                        SysDictionaryEnt sysDictionaryEnt = dicList.get(0);
                        if(ObjectUtils.isNotEmpty(sysDictionaryEnt) && StringUtils.isNotBlank(sysDictionaryEnt.getDicKey())){
                            key = sysDictionaryEnt.getDicKey();
                        }
                    }
                }
            }*/
            Date expireTime = new Date(new Date().getTime() + EXPIRE_TIME);
            String third_token = "";
            try{
                third_token = io.jsonwebtoken.Jwts.builder()
                        .setExpiration(expireTime)
                        .signWith(io.jsonwebtoken.SignatureAlgorithm.HS256, Base64.getEncoder().encodeToString(KEY.getBytes("UTF-8")))
                        .setSubject(string)
                        .compact();
            }catch (Exception e){
                e.printStackTrace();
            }

            List<FineReportThirdTokenCache> list = new ArrayList<>();
            FineReportThirdTokenCache cache = new FineReportThirdTokenCache();
            cache.setCacheKey(session.getLoginName());
            cache.setCacheCode(third_token);
            list.add(cache);
            FineReportThirdTokenCache.set(context,session.getLoginName(),list);
            // third_token 返回前端.
            // url?tenant=租户id&third_token=third_token  跳转到指定报表
            // cookie中生成 fine_auth_token和tenantId, 可以正常打开地址
            // tenant=test1&third_token=third_token
            System.out.println(third_token);
            report.setThirdToken(third_token);
        }
        return XSingleResult.success(report);
    }

    @XText("交易平台登录")
    @XApiAnonymous
    @XApiPost
    public XSingleResult<TradeLoginOutput> loginByTrade(XContext context, TradeLoginInput input) {
        String env = context.getEnv().getText();
        if("prod".equals(env)){
            String clientIp = context.getWebRequest().getClientIp();
            String ipWhiteValue = LoginConfig.ipWhite.findValue(context);
            if(StringUtils.isNotBlank(ipWhiteValue) && !ipWhiteValue.contains(clientIp)){
                return XSingleResult.error(context,LoginError.Trade_IpWhiteError);
            }
        }
        String loginParam = input.getLoginParam();
        if(StringUtils.isBlank(loginParam)) {
            return XSingleResult.error(context,LoginError.Trade_ParamIsNull);
        }
        String loginInfo = null;
        try {
            String privateKey = LoginConfig.rsaPrivateKey.findValue(context);
            privateKey = XEncryptUtils.decrypt(privateKey);
            loginInfo = RSAUtil.RsaDecrypt(loginParam, privateKey);
        }catch (Exception e){
            context.getLogger().error("参数解密失败",e);
            return XSingleResult.error(context,LoginError.Trade_ParamRSAError);
        }
        TradeLoginInfo tradeLoginInfo = null;
        try {
            tradeLoginInfo = XJsonUtils.toObject(loginInfo, TradeLoginInfo.class);
            if(!(tradeLoginInfo!=null && StringUtils.isNotBlank(tradeLoginInfo.getLoginName()))){
                return XSingleResult.error(context,LoginError.Trade_ParamFormatError);
            }
        }catch (Exception e){
            context.getLogger().error("参数格式有误",e);
            return XSingleResult.error(context,LoginError.Trade_ParamFormatError);
        }

        SysUserMapper mapper = context.getBean(SysUserMapper.class);
        QueryWrapper<SysUserEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysUserEnt::getLoginName, tradeLoginInfo.getLoginName())
                .eq(SysUserEnt::getIsTrade,1)
                .last("limit 1");
        SysUserEnt sysUserEnt = mapper.selectOne(queryWrapper);
        if(sysUserEnt == null)
            return XSingleResult.error(context,LoginError.Trade_LoginError);

        XApiCookieConf apiCookieConf = XConfManager.getConf(XApiCookieConf.class);
        Integer expiresIn = apiCookieConf.getExpires();

        String userName = sysUserEnt.getUserName(); //姓名
        Long orgId = LOGIN_FLAG_TRADE; //组织机构id - 交易平台登陆专用值
        Long roleId0 = 1L; //角色id1
        Long roleId1 = 1L; //角色id2
        Long userId = IdWorker.getId(); //用户id
        String userCode = sysUserEnt.getId();//这个是用户字符串类型的唯一标志
        String ssoToken = null; //sso系统授权的token
        String ssoRefreshToken = null; //sso系统授权的刷新token
        String uniqueId = XUuidUtils.randomUUID();
        XUser user = XUser.createUser(XUser.ENTRY_API,
                orgId,
                new Long[]{roleId0, roleId1},
                userId,
                userCode,
                sysUserEnt.getLoginName(),
                userName,
                new XSsoAuth(uniqueId, ssoToken, ssoRefreshToken));
        XSingleResult<XToken> saveUserResult = XTokenHelper.saveUser(context, user, expiresIn);
        saveUserResult.throwIfException();

        TradeLoginOutput output = new TradeLoginOutput();
        output.setExpiresIn(expiresIn);
        output.setToken(saveUserResult.getResult().getToken());
        return XSingleResult.success(output);
    }

    /**
     * 部门参数规则(department):
     * 部门ID长度构建部门树：
     * 默认通过部门ID长度构建部分树，会根据department和id字段长度来自动构建部门树结构
     * 例如：id 0 对应中国， 02 对应北京，021 对应朝阳区，根据id长度匹配父子部门
     * 特殊根部门ID：
     * 例如：id 00 对应中国， 02 对应北京，021 对应朝阳区，这时候就需要填入 00 来标记特殊根部门
     * @return
     */


    /**
     *  平台用户类型(platformType)：
     *  1 PC端
     *  2 BI查看 (仅BI)
     *  4 移动端
     *  5 BI数据分析（只可创建excel和自助数据集） (仅BI)
     *  6 BI数据处理（可创建sql/db/excel/自助数据集)(仅BI)
     * @return
     */
    public static String getString(XContext context){
        PpsUserSession session = context.getSession(PpsUserSession.class);
        HashMap<String, Object> map = new HashMap<>();
        HashMap<String, Object> department = new HashMap<>();

        String ouName = getDepartment(session.getOuId(), context);
        department.put("name", ouName != null? ouName : session.getOuName());
        department.put("id", session.getOuId());

        map.put("username", session.getLoginName());
        map.put("department", department);

       /* ArrayList<String> list1 = new ArrayList<>();
        ArrayList<Integer> list2 = new ArrayList<>();
        list1.add("职员");
        list2.add(1);
        map.put("roles", list1);
        map.put("platformType", list2);*/
        String jsonString = JSONUtil.toJsonStr(map);
        return jsonString;
    }

    private static String getDepartment(String ouId, XContext context) {
        try {
            SysOrganizationRelMapper bean = context.getBean(SysOrganizationRelMapper.class);
            QueryWrapper<SysOrganizationRelEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("ou_id", ouId);
            queryWrapper.lambda().lt(SysOrganizationRelEnt::getEffectTime,new Date());
            queryWrapper.lambda().gt(SysOrganizationRelEnt::getEndTime,new Date());

            SysOrganizationRelEnt sysOrganizationRelEnt = bean.selectOne(queryWrapper);
            String ouIdPath = sysOrganizationRelEnt.getOuIdPath();
            if (StringUtils.isEmpty(ouIdPath)){
                return null;
            }

            String[] split = ouIdPath.split("/");

            SysOrganizationMapper mapper = context.getBean(SysOrganizationMapper.class);
            QueryWrapper<SysOrganizationEnt> wrapper = new QueryWrapper<>();
            wrapper.in("id", split);
            wrapper.eq("is_deleted", "1");
            List<SysOrganizationEnt> sysOrganizationEnts = mapper.selectList(wrapper);

            Map<String, SysOrganizationEnt> personMap =  sysOrganizationEnts.stream().collect(Collectors.toMap(
                    ent -> ent.getId(),
                    ent -> ent,
                    (p, n) -> p
            ));

            StringBuffer buffer = new StringBuffer();

            for (int i = 0; i < split.length; i++) {
                String ouName = personMap.get(split[i]).getOuName();

                if (i == split.length - 1){
                    buffer.append(ouName);
                }else {
                    buffer.append(ouName).append(SPLIT);
                }
            }

            return buffer.toString();
        } catch (Exception e) {
            context.log().info("获取机构path错误");
            return null;
        }

    }

    private void deleteError(XContext context , SysUserEnt sysUserEnt ){
        //删除登录日志错误信息
        SysUserLoginLogMapper userLoginLogMapper = context.getBean(SysUserLoginLogMapper.class);
        QueryWrapper<SysUserLoginLogEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysUserLoginLogEnt::getLoginName,sysUserEnt.getLoginName());
        userLoginLogMapper.delete(queryWrapper);
        SysUserMapper mapper = context.getBean(SysUserMapper.class);
        sysUserEnt.setStatus(0);
        sysUserEnt.setModifyTime(new Date());
        mapper.updateById(sysUserEnt);
    }

    private void insertError(XContext context , String loginName ){
        //记录登录日志错误信息
        SysUserLoginLogMapper userLoginLogMapper = context.getBean(SysUserLoginLogMapper.class);
        SysUserLoginLogEnt logEnt = new SysUserLoginLogEnt();
        logEnt.setId(UUID.randomUUID().toString());
        logEnt.setErrorMsg("账户、密码错误");
        logEnt.setLoginName(loginName);
        logEnt.setCreateTime(new Date());
        userLoginLogMapper.insert(logEnt);
    }

    private XSingleResult<SysUserEnt> checkForgetAccount(XContext context,String loginName){
        SysUserMapper mapper = context.getBean(SysUserMapper.class);
        QueryWrapper<SysUserEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysUserEnt::getLoginName, loginName);
        SysUserEnt sysUserEnt = mapper.selectOne(queryWrapper);
        if(sysUserEnt == null) {
            insertForgetError(context , loginName );
            Long count = getForgetErrorTimes(context );
            throw new XServiceException(1000, "用户名错误，还剩"+(5 - count)+"次机会");
        }
        return XSingleResult.success(sysUserEnt);
    }

    private void insertForgetError(XContext context , String loginName ){
        //记录登录日志错误信息
        SysUserForgetLogMapper userForgetLogMapper = context.getBean(SysUserForgetLogMapper.class);
        SysUserForgetLogEnt logEnt = new SysUserForgetLogEnt();
        logEnt.setId(UUID.randomUUID().toString());
        logEnt.setIpAddress(context.getWebRequest().getClientIp());
        logEnt.setLoginName(loginName);
        logEnt.setCreateTime(new Date());
        userForgetLogMapper.insert(logEnt);
    }

    private Long getErrorTimes(XContext context , String loginName , SysUserEnt sysUserEnt ){
        SysUserLoginLogMapper userLoginLogMapper = context.getBean(SysUserLoginLogMapper.class);
        QueryWrapper<SysUserLoginLogEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysUserLoginLogEnt::getLoginName,loginName);
        queryWrapper.lambda().ge(SysUserLoginLogEnt::getCreateTime, XDateUtils.getDayBeginTime(new Date()));
        Long count = userLoginLogMapper.selectCount(queryWrapper);
        if(count >= 5 && sysUserEnt != null){
            SysUserMapper mapper = context.getBean(SysUserMapper.class);
            sysUserEnt.setStatus(2);
            sysUserEnt.setModifyTime(new Date());
            mapper.updateById(sysUserEnt);
        }
        return count;
    }

    private Long getForgetErrorTimes(XContext context ){
        SysUserForgetLogMapper userLoginLogMapper = context.getBean(SysUserForgetLogMapper.class);
        QueryWrapper<SysUserForgetLogEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysUserForgetLogEnt::getIpAddress,context.getWebRequest().getClientIp());
        queryWrapper.lambda().ge(SysUserForgetLogEnt::getCreateTime, XDateUtils.addMunites(new Date() , -5));
        Long count = userLoginLogMapper.selectCount(queryWrapper);
        return count;
    }
}
