package pps.core.system.service;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.Date;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.CollectionUtils;
import pps.core.system.cache.SysAreaCache;
import pps.core.system.cache.SysOrganizationCache;
import pps.core.system.constant.SysAreaCode;
import pps.core.system.constant.SysOrganizationCode;
import pps.core.system.mapper.*;
import pps.core.system.entity.*;
import pps.core.system.service.data.sys_area.*;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import pps.core.system.service.data.sys_organization.GetSysOrganizationInput;
import pps.core.system.service.data.sys_organization.QuerySysOrganizationViewOutputTree;
import xstartup.annotation.XService;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.error.XError;
import xstartup.base.data.CustomQueryInput;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import xstartup.feature.mybatis.helper.XMapperHelper;

import java.util.List;
import java.util.stream.Collectors;

@XService
@XText("地区管理")
public class SysAreaService {

    @XText("查询地区树")
    @XApiAnonymous
    @XApiPost
    public XListResult<GetSysAreaViewOutputTree> getSysAreaTree(XContext context, GetSysAreaInput input) {
        //先从缓存中获取
        List<SysAreaCache> sysAreaCaches = SysAreaCache.list(context, SysAreaCode.SYS_AREA_CACHE);
        if(ObjectUtils.isNotEmpty(input.getLev()) && input.getLev() ==1){
            List<SysAreaCache> items = new ArrayList<>();
            for(SysAreaCache ent:sysAreaCaches){
                if (!CollectionUtils.isEmpty(ent.getChildren())){
                    items.addAll(ent.getChildren());
                }

            }
            sysAreaCaches = items;
        }
        //缓存不存在从数据库获取
        if (CollectionUtils.isEmpty(sysAreaCaches)) {
            List<SysAreaCache> caches = selectSysAreaTree(context, input);
            List<GetSysAreaViewOutputTree> resultTree = XCopyUtils.copyNewList(caches, GetSysAreaViewOutputTree.class);
            if(ObjectUtils.isNotEmpty(input.getLev()) && input.getLev() ==1){
                List<GetSysAreaViewOutputTree> items = new ArrayList<>();
                for(GetSysAreaViewOutputTree ent:resultTree){
                    if (!CollectionUtils.isEmpty(ent.getChildren())){
                        items.addAll(ent.getChildren());
                    }
                }
                resultTree = items;
            }
            return XListResult.success(resultTree);
        }
        List<GetSysAreaViewOutputTree> resultTree = XCopyUtils.copyNewList(sysAreaCaches, GetSysAreaViewOutputTree.class);
        return XListResult.success(resultTree);
    }

    private List<SysAreaCache>  selectSysAreaTree(XContext context,GetSysAreaInput input){
        SysAreaViewMapper mapper = context.getBean(SysAreaViewMapper.class);
        SysAreaView record = new SysAreaView();
        XCopyUtils.copyObject(input, record);
        List<SysAreaView> areaViews = mapper.selectListAll(record);
        List<GetSysAreaViewOutputTree> list = XCopyUtils.copyNewList(areaViews, GetSysAreaViewOutputTree.class);
        List<GetSysAreaViewOutputTree> trees = new ArrayList<>();

        for (GetSysAreaViewOutputTree tree : list) {
            if (tree.getLev().equals(0)) {
                trees.add(tree);
            }
            //子集
            for (GetSysAreaViewOutputTree it : list) {
                if (tree.getId().equals(it.getParentId())) {
                    if (tree.getChildren() == null) {
                        tree.setChildren(new ArrayList<>());
                    }
                    tree.getChildren().add(it);
                }
            }
        }
        List<SysAreaCache> caches = XCopyUtils.copyNewList(trees, SysAreaCache.class);
        //放入redis缓存中
        SysAreaCache.set(context,SysAreaCode.SYS_AREA_CACHE,caches);
        return caches;
    }

    @XApiAnonymous
    @XApiPost
    public XServiceResult createSysArea(XContext context, CreateSysAreaInput input){
        SysAreaMapper mapper = context.getBean(SysAreaMapper.class);
        SysAreaEnt entity = new SysAreaEnt();
        XCopyUtils.copyObject(input, entity);
        mapper.insert(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiPost
    public XServiceResult updateSysArea(XContext context, UpdateSysAreaInput input){
        SysAreaMapper mapper = context.getBean(SysAreaMapper.class);
        QueryWrapper<SysAreaEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysAreaEnt::getId, input.getId());
        SysAreaEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiPost
    public XServiceResult deleteSysArea(XContext context, DeleteSysAreaInput input){
        SysAreaMapper mapper = context.getBean(SysAreaMapper.class);
        QueryWrapper<SysAreaEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysAreaEnt::getId, input.getId());
        SysAreaEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        mapper.deleteById(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetSysAreaOutput> getSysArea(XContext context, GetSysAreaInput input){
        SysAreaMapper mapper = context.getBean(SysAreaMapper.class);
        QueryWrapper<SysAreaEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysAreaEnt::getId, input.getId());
        SysAreaEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetSysAreaOutput output = new GetSysAreaOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QuerySysAreaOutput> querySysArea(XContext context, QuerySysAreaInput input){
        SysAreaMapper mapper = context.getBean(SysAreaMapper.class);
        QueryWrapper<SysAreaEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysAreaEnt::getId, input.getId())
            .like(SysAreaEnt::getName, input.getName())
            .eq(SysAreaEnt::getParentId, input.getParentId())
            .eq(SysAreaEnt::getLev, input.getLev());
        return XMapperHelper.query(mapper, input, queryWrapper, QuerySysAreaOutput.class);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QuerySysAreaOutput> dynamicQuerySysArea(XContext context, CustomQueryInput input){
        SysAreaMapper mapper = context.getBean(SysAreaMapper.class);
        return XMapperHelper.query(mapper, input, SysAreaEnt.class, QuerySysAreaOutput.class);
    }

    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetSysAreaViewOutput> getSysAreaView(XContext context, GetSysAreaViewInput input){
        SysAreaViewMapper mapper = context.getBean(SysAreaViewMapper.class);
        SysAreaView record = new SysAreaView();
        XCopyUtils.copyObject(input, record);
        SysAreaView view = mapper.selectOne(record);
        if(view == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetSysAreaViewOutput output = new GetSysAreaViewOutput();
        XCopyUtils.copyObject(view, output);
        return XSingleResult.success(output);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QuerySysAreaViewOutput> querySysAreaView(XContext context, QuerySysAreaViewInput input){
        SysAreaViewMapper mapper = context.getBean(SysAreaViewMapper.class);
        SysAreaView record = new SysAreaView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<SysAreaView> list = mapper.selectList(record);
        PageInfo<SysAreaView> pageInfo = new PageInfo<>(list);
        List<QuerySysAreaViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySysAreaViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

}
