package pps.core.system.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import pps.cloud.system.service.SystemDictionaryService;
import pps.cloud.system.service.data.*;
import pps.core.system.cache.SysDictionaryCache;
import pps.core.common.pattern.Builder;
import pps.core.system.entity.*;
import pps.core.system.mapper.SysDictionaryMapper;
import pps.core.system.mapper.SysDictionaryViewMapper;
import pps.core.system.mapper.SysOrganizationDictionaryMapper;
import pps.core.system.mapper.SysOrganizationRelMapper;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.helper.XTransactionHelper;

import java.util.*;
import java.util.stream.Collectors;

@XService
public class SysDictionaryServiceImpl implements SystemDictionaryService  {
    @XText("根据父级id(可以多个id)获取字典")
    @XApiPost
    @Override
    public XPageResult<QuerySysDictionaryViewOutput> querySysDictionarysByParentIds(XContext context, QuerySysDictionaryViewInput input) {
        SysDictionaryViewMapper mapper = context.getBean(SysDictionaryViewMapper.class);
        SysDictionaryView record = new SysDictionaryView();
        XCopyUtils.copyObject(input, record);
        String pidStr = record.getPid();
        StringBuffer codes = new StringBuffer();
        if(StringUtils.isNotBlank(pidStr)){
            String[] split = pidStr.split(",");
            for(int i=0;i< split.length;i++){
                if(StringUtils.isNotBlank(split[i]) && !split[i].startsWith("'")){
                    split[i] = "'"+split[i]+"'";
                }
                if(i==0){
                    codes.append(split[i]);
                }else{
                    codes.append(","+split[i]);
                }
            }
        }
        record.setPid(codes.toString());
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<SysDictionaryView> list = mapper.selectListsByParentIds(record);
        PageInfo<SysDictionaryView> pageInfo = new PageInfo<>(list);
        List<QuerySysDictionaryViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySysDictionaryViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    @Override
    public XListResult<QuerySysDictionaryViewOutput> querySysDictionarysByAlias(XContext context, CreateSysDictionaryInput input) {
        SysDictionaryMapper mapper = context.getBean(SysDictionaryMapper.class);
        SysOrganizationDictionaryMapper orgDicmapper = context.getBean(SysOrganizationDictionaryMapper.class);
        SysOrganizationRelMapper orgRelMapper = context.getBean(SysOrganizationRelMapper.class);
        List<SysDictionaryCache> list = SysDictionaryCache.list(context, input.getAlias());
        QueryWrapper<SysDictionaryEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysDictionaryEnt::getAlias,input.getAlias());
        SysDictionaryEnt sysDictionaryEnt = mapper.selectOne(queryWrapper);
        list = list.stream().filter((SysDictionaryCache dic)->StringUtils.equals(dic.getPid(),sysDictionaryEnt.getId()) && !StringUtils.equals(dic.getAlias(),input.getAlias()))
                .collect(Collectors.toList());
        if(ObjectUtils.isEmpty(list)){
            list = new ArrayList<>();
            SysDictionaryCache.set(context,input.getAlias(),list);
            //查询alias字典
            QueryWrapper<SysDictionaryEnt> query = new QueryWrapper();
            query.lambda().eq(SysDictionaryEnt::getAlias,input.getAlias()).gt(SysDictionaryEnt::getEndTime,new Date());
            SysDictionaryEnt sysDictionary = mapper.selectOne(query);
            if(ObjectUtils.isNotEmpty(sysDictionary)){
                SysDictionaryCache sysDictionaryCache = new SysDictionaryCache();
                XCopyUtils.copyObject(sysDictionary,sysDictionaryCache);
                list.add(sysDictionaryCache);
                //查询alias字典下级字典项
                QueryWrapper<SysDictionaryEnt> queryItems = new QueryWrapper();
                if(ObjectUtils.isNotEmpty(sysDictionary.getIsOrg()) && sysDictionary.getIsOrg() == 1){
                    //查询当前登录人的组织机构层级路径
                    QueryWrapper<SysOrganizationRelEnt> queryOrgRel = new QueryWrapper<>();
                    queryOrgRel.lambda().eq(SysOrganizationRelEnt::getOuId,input.getOuId())
                            .gt(SysOrganizationRelEnt::getEndTime,new Date());
                    SysOrganizationRelEnt sysOrganizationRelEnt = orgRelMapper.selectOne(queryOrgRel);
                    //根据字典id查询
                    QueryWrapper<SysOrganizationDictionaryEnt> queryOrgDic = new QueryWrapper();
                    queryOrgDic.lambda().eq(SysOrganizationDictionaryEnt::getParentDicCode, sysDictionary.getId())
                            .gt(SysOrganizationDictionaryEnt::getEndTime, new Date());
                    List<SysOrganizationDictionaryEnt> sysOrganizationDictionaryEnts = orgDicmapper.selectList(queryOrgDic);
                    String ouIdPath = sysOrganizationRelEnt.getOuIdPath();
                    String ouId = sysOrganizationRelEnt.getOuId();
                    if(StringUtils.isNotBlank(ouIdPath)){
                        List<String> dicIds = sysOrganizationDictionaryEnts.stream().map(p -> {
                            if(ouIdPath.indexOf(p.getOuId())!= -1 && p.getUseLowerLevel()==1){
                                return p.getDicCode();
                            }else if(ouId.indexOf(p.getOuId())!= -1){
                                return p.getDicCode();
                            }else{
                                return "1";
                            }
                        }).collect(Collectors.toList());
                        queryItems.lambda().in(SysDictionaryEnt::getId, dicIds);
                    }
                }
                queryItems.lambda().eq(SysDictionaryEnt::getPid,sysDictionaryCache.getId()).gt(SysDictionaryEnt::getEndTime,new Date());
                List<SysDictionaryEnt> sysDictionaryList = mapper.selectList(queryItems);
                if(ObjectUtils.isNotEmpty(sysDictionaryList)){
                    if(ObjectUtils.isNotEmpty(list)){
                        for(SysDictionaryEnt ent:sysDictionaryList){
                            SysDictionaryCache cache = new SysDictionaryCache();
                            XCopyUtils.copyObject(ent,cache);
                            list.add(cache);
                        }
                    }
                    Collections.sort(sysDictionaryList, new Comparator<SysDictionaryEnt>() {
                        @Override
                        public int compare(SysDictionaryEnt o1, SysDictionaryEnt o2) {
                            if(ObjectUtils.isNotEmpty(o1.getSort()) && ObjectUtils.isNotEmpty(o2.getSort())){
                                return o1.getSort()-o2.getSort();
                            }else{
                                return 0;
                            }
                        }
                    });
                    List<QuerySysDictionaryViewOutput> outputs = XCopyUtils.copyNewList(sysDictionaryList, QuerySysDictionaryViewOutput.class);
                    return XListResult.success(outputs);
                }
            }
        }
        if(sysDictionaryEnt != null && ObjectUtils.isNotEmpty(sysDictionaryEnt.getIsOrg()) && sysDictionaryEnt.getIsOrg() == 1){
            //查询当前登录人的组织机构层级路径
            QueryWrapper<SysOrganizationRelEnt> queryOrgRel = new QueryWrapper<>();
            queryOrgRel.lambda().eq(SysOrganizationRelEnt::getOuId,input.getOuId())
                    .gt(SysOrganizationRelEnt::getEndTime,new Date());
            SysOrganizationRelEnt sysOrganizationRelEnt = orgRelMapper.selectOne(queryOrgRel);
            //根据字典id查询
            QueryWrapper<SysOrganizationDictionaryEnt> queryOrgDic = new QueryWrapper();
            queryOrgDic.lambda().eq(SysOrganizationDictionaryEnt::getParentDicCode, sysDictionaryEnt.getId())
                    .gt(SysOrganizationDictionaryEnt::getEndTime, new Date());
            List<SysOrganizationDictionaryEnt> sysOrganizationDictionaryEnts = orgDicmapper.selectList(queryOrgDic);
            String ouIdPath = sysOrganizationRelEnt.getOuIdPath();
            String ouId = sysOrganizationRelEnt.getOuId();
            if(StringUtils.isNotBlank(ouIdPath)){
                List<String> dicIds = sysOrganizationDictionaryEnts.stream().map(p -> {
                    if(ouIdPath.indexOf(p.getOuId())!= -1 && p.getUseLowerLevel()==1){
                        return p.getDicCode();
                    }else if(ouId.indexOf(p.getOuId())!= -1){
                        return p.getDicCode();
                    }else{
                        return "1";
                    }
                }).collect(Collectors.toList());
                list = list.stream().filter(i->dicIds.contains(i.getId())).collect(Collectors.toList());
            }

        }
        Collections.sort(list, new Comparator<SysDictionaryCache>() {
            @Override
            public int compare(SysDictionaryCache o1, SysDictionaryCache o2) {
                if(ObjectUtils.isNotEmpty(o1.getSort()) && ObjectUtils.isNotEmpty(o2.getSort())){
                    return o1.getSort()-o2.getSort();
                }else{
                    return 0;
                }
            }
        });
        List<QuerySysDictionaryViewOutput> outputs = XCopyUtils.copyNewList(list, QuerySysDictionaryViewOutput.class);
        return XListResult.success(outputs);
    }

    @Override
    public XSingleResult<QuerySysDictionaryViewOutput> querySysDictionarysById(XContext context, QuerySysDictionaryViewInput input) {
        SysDictionaryMapper mapper = context.getBean(SysDictionaryMapper.class);
        QueryWrapper<SysDictionaryEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysDictionaryEnt::getId, input.getId());
        SysDictionaryEnt ent = mapper.selectOne(queryWrapper);
        QuerySysDictionaryViewOutput output = new QuerySysDictionaryViewOutput();
        XCopyUtils.copyObject(ent, output);
        return XSingleResult.success(output);
    }

    @Override
    public XPageResult<QuerySysDictionaryViewOutput> querySysDictionarysByDicKeys(XContext context, QuerySysDictionaryViewInput input) {
        SysDictionaryViewMapper mapper = context.getBean(SysDictionaryViewMapper.class);
        SysDictionaryView record = new SysDictionaryView();
        XCopyUtils.copyObject(input, record);
//        String dicKey = record.getDicKey();
        String parentDicKey = record.getPid();
//        StringBuffer dicKeys = new StringBuffer();
        StringBuffer parentDicKeys = new StringBuffer();
        /*if(StringUtils.isNotBlank(dicKey)){
            String[] split = dicKey.split(",");
            for(int i=0;i< split.length;i++){
                if(StringUtils.isNotBlank(split[i]) && !split[i].startsWith("'")){
                    split[i] = "'"+split[i]+"'";
                }
                if(i==0){
                    dicKeys.append(split[i]);
                }else{
                    dicKeys.append(","+split[i]);
                }
            }
        }*/
        if(StringUtils.isNotBlank(parentDicKey)){
            String[] split = parentDicKey.split(",");
            for(int i=0;i< split.length;i++){
                if(StringUtils.isNotBlank(split[i]) && !split[i].startsWith("'")){
                    split[i] = "'"+split[i]+"'";
                }
                if(i==0){
                    parentDicKeys.append(split[i]);
                }else{
                    parentDicKeys.append(","+split[i]);
                }
            }
        }
//        record.setDicKey(dicKeys.toString());
        record.setPid(parentDicKeys.toString());
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<SysDictionaryView> list = mapper.selectListsByByDicKeys(record);
        PageInfo<SysDictionaryView> pageInfo = new PageInfo<>(list);
        List<QuerySysDictionaryViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySysDictionaryViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    @Override
    public XListResult<QuerySysDictionaryViewOutput> queryChildSysDictionarysByParentAlias(XContext context, QuerySysDictionaryViewInput input) {
//        SysDictionaryViewMapper mapper = context.getBean(SysDictionaryViewMapper.class);
//        SysDictionaryView record = new SysDictionaryView();
//        XCopyUtils.copyObject(input, record);
//        String alias = record.getAlias();
//        StringBuffer codes = new StringBuffer();
//        if(StringUtils.isNotBlank(alias)){
//            String[] split = alias.split(",");
//            for(int i=0;i< split.length;i++){
//                if(StringUtils.isNotBlank(split[i]) && !split[i].startsWith("'")){
//                    split[i] = "'"+split[i]+"'";
//                }
//                if(i==0){
//                    codes.append(split[i]);
//                }else{
//                    codes.append(","+split[i]);
//                }
//            }
//        }
//        record.setAlias(codes.toString());
//        PageHelper.startPage(input.getPage(), input.getLimit());
//        List<SysDictionaryView> list = mapper.selectListsByParentCodes(record);
//        PageInfo<SysDictionaryView> pageInfo = new PageInfo<>(list);
//        List<QuerySysDictionaryViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySysDictionaryViewOutput.class);
//        return XPageResult.success(outputs, input, pageInfo.getTotal());
        List<QuerySysDictionaryViewOutput> resultList = new ArrayList<>();
        SysDictionaryMapper sysDictionaryMapper = context.getBean(SysDictionaryMapper.class);
        List<SysDictionaryEnt> list = sysDictionaryMapper.selectList(Builder.create(QueryWrapper.class).build(queryWrapper -> {
            queryWrapper.eq("alias", input.getAlias());
        }));
        if( list.isEmpty()){
            return  XListResult.success(resultList);
        }

        SysDictionaryEnt parentDictionaryEnt = list.get(0);
        list = sysDictionaryMapper.selectList(Builder.create(QueryWrapper.class).build(queryWrapper ->{
            queryWrapper.eq("pid", parentDictionaryEnt.getId() );
        }));

        resultList = XCopyUtils.copyNewList(list, QuerySysDictionaryViewOutput.class);
        return XListResult.success(resultList);
    }

    @XText("创建字典")
    @Override
    public XServiceResult createSysDictionary(XContext context, CreateSysDictionaryInput input) {
        return XTransactionHelper.begin(context,()-> {
            SysDictionaryMapper mapper = context.getBean(SysDictionaryMapper.class);
            //如果输入的alias为空，查询新增项的上级alias
            String alias = "";
            if(StringUtils.isNotBlank(input.getAlias())){
                alias = input.getAlias();
                List<SysDictionaryCache> list1 = SysDictionaryCache.list(context, alias);
                if(ObjectUtils.isNotEmpty(list1)){
                    return XServiceResult.error(200, "字典标识已存在！");
                }
            }else{
                alias = selectSysDictionaryAlias(context, input.getPid());
            }
            boolean exist = SysDictionaryCache.exist(context,alias, input.getDicKey());
            if(exist){
                return XServiceResult.error(200, "字典编码已存在！");
            }else{
                //缓存不存在数据库查询
                QueryWrapper<SysDictionaryEnt> queryWrapper = new QueryWrapper<>();
                queryWrapper.lambda().eq(SysDictionaryEnt::getDicKey, input.getDicKey())
                        .gt(SysDictionaryEnt::getEndTime, new Date());
                if (StringUtils.isNotBlank(input.getPid())) {
                    queryWrapper.lambda().eq(SysDictionaryEnt::getPid, input.getPid());
                } else {
                    queryWrapper.lambda().eq(SysDictionaryEnt::getPid, "1");
                }
                SysDictionaryEnt ent = mapper.selectOne(queryWrapper);
                if (ent != null) {
                    return XServiceResult.error(200, "字典编码已存在！");
                }
            }
            SysDictionaryEnt entity = new SysDictionaryEnt();
            String dicId = UUID.randomUUID().toString();
            XCopyUtils.copyObject(input, entity);
            if (StringUtils.equals("1",input.getPid())) {
                if (StringUtils.isBlank(input.getAlias())) {
                    return XServiceResult.error(200, "新增一级字典，字典标识不能为空！");
                }
                entity.setPid("1");
                entity.setDicPath(null);
            } else {
                QueryWrapper<SysDictionaryEnt> wrapper = new QueryWrapper<>();
                wrapper.lambda().eq(SysDictionaryEnt::getId, input.getPid());
                SysDictionaryEnt sysDictionaryEnt = mapper.selectOne(wrapper);
                //查看有没有dicPath,如果没有说明是一级，直接dicPath=/父级id
                if (ObjectUtils.isNotEmpty(sysDictionaryEnt)) {
                    if (StringUtils.isNotBlank(sysDictionaryEnt.getDicPath())) {
                        entity.setDicPath(sysDictionaryEnt.getDicPath() + "/" + sysDictionaryEnt.getId());
                    } else {
                        entity.setDicPath("/" + sysDictionaryEnt.getId());
                    }
                }
            }
            if(input.getSort() == null){
                QueryWrapper<SysDictionaryEnt> queryMaxSortEntity = new QueryWrapper<>();
                queryMaxSortEntity.lambda().gt(SysDictionaryEnt::getEndTime, new Date());
                if(StringUtils.isBlank(input.getPid())){
                    queryMaxSortEntity.lambda().eq(SysDictionaryEnt::getPid, "1")
                            .isNotNull(SysDictionaryEnt::getSort).orderByDesc(SysDictionaryEnt::getSort);
                }else{
                    queryMaxSortEntity.lambda().eq(SysDictionaryEnt::getPid, input.getPid());
                }
                //查询最大的sort值
                queryMaxSortEntity.lambda().isNotNull(SysDictionaryEnt::getSort).orderByDesc(SysDictionaryEnt::getSort);
                List<SysDictionaryEnt> sysDictionarys = mapper.selectList(queryMaxSortEntity);
                if(ObjectUtils.isNotEmpty(sysDictionarys)){
                    SysDictionaryEnt sysDictionaryEnt1 = sysDictionarys.get(0);
                    if(sysDictionaryEnt1.getSort() != null){
                        entity.setSort(sysDictionaryEnt1.getSort()+1);
                    }else{
                        entity.setSort(1);
                    }
                }else{
                    entity.setSort(1);
                }
            }
            entity.setEffectTime(new Date());
            entity.setId(dicId);
//        entity.setcr(context.getSession(PpsUserSession.class).getId());
            entity.setCreateName(context.getUser().getName());
            entity.setCreateTime(new Date());
            mapper.insert(entity);
            //更新字典缓存
            List<SysDictionaryCache> list = SysDictionaryCache.list(context, input.getAlias());
            SysDictionaryCache sysDictionaryCache = new SysDictionaryCache();
            XCopyUtils.copyObject(entity,sysDictionaryCache);
            list.add(sysDictionaryCache);
            SysDictionaryCache.set(context,alias,list);
            return XServiceResult.OK;
        });
    }

    @XText("创建字典")
    @Override
    public XSingleResult<QuerySysDictionaryViewOutput> createSysDictionaryAndReturn(XContext context, CreateSysDictionaryInput input) {
        return XTransactionHelper.begin(context,()-> {
            SysDictionaryMapper mapper = context.getBean(SysDictionaryMapper.class);
            //如果输入的alias为空，查询新增项的上级alias
            String alias = "";
            if(StringUtils.isNotBlank(input.getAlias())){
                alias = input.getAlias();
                List<SysDictionaryCache> list1 = SysDictionaryCache.list(context, alias);
                if(ObjectUtils.isNotEmpty(list1)){
                    return XSingleResult.error(200, "字典标识已存在！");
                }
            }else{
                alias = selectSysDictionaryAlias(context, input.getPid());
            }
            boolean exist = SysDictionaryCache.exist(context,alias, input.getDicKey());
            if(exist){
                return XSingleResult.error(200, "字典编码已存在！");
            }else{
                //缓存不存在数据库查询
                QueryWrapper<SysDictionaryEnt> queryWrapper = new QueryWrapper<>();
                queryWrapper.lambda().eq(SysDictionaryEnt::getDicKey, input.getDicKey())
                        .gt(SysDictionaryEnt::getEndTime, new Date());
                if (StringUtils.isNotBlank(input.getPid())) {
                    queryWrapper.lambda().eq(SysDictionaryEnt::getPid, input.getPid());
                } else {
                    queryWrapper.lambda().eq(SysDictionaryEnt::getPid, "1");
                }
                SysDictionaryEnt ent = mapper.selectOne(queryWrapper);
                if (ent != null) {
                    return XSingleResult.error(200, "字典编码已存在！");
                }
            }
            SysDictionaryEnt entity = new SysDictionaryEnt();
            String dicId = UUID.randomUUID().toString();
            XCopyUtils.copyObject(input, entity);
            if (StringUtils.equals("1",input.getPid())) {
                if (StringUtils.isBlank(input.getAlias())) {
                    return XSingleResult.error(200, "新增一级字典，字典标识不能为空！");
                }
                entity.setPid("1");
                entity.setDicPath(null);
            } else {
                QueryWrapper<SysDictionaryEnt> wrapper = new QueryWrapper<>();
                wrapper.lambda().eq(SysDictionaryEnt::getId, input.getPid());
                SysDictionaryEnt sysDictionaryEnt = mapper.selectOne(wrapper);
                //查看有没有dicPath,如果没有说明是一级，直接dicPath=/父级id
                if (ObjectUtils.isNotEmpty(sysDictionaryEnt)) {
                    if (StringUtils.isNotBlank(sysDictionaryEnt.getDicPath())) {
                        entity.setDicPath(sysDictionaryEnt.getDicPath() + "/" + sysDictionaryEnt.getId());
                    } else {
                        entity.setDicPath("/" + sysDictionaryEnt.getId());
                    }
                }
            }
            if(input.getSort() == null){
                QueryWrapper<SysDictionaryEnt> queryMaxSortEntity = new QueryWrapper<>();
                queryMaxSortEntity.lambda().gt(SysDictionaryEnt::getEndTime, new Date());
                if(StringUtils.isBlank(input.getPid())){
                    queryMaxSortEntity.lambda().eq(SysDictionaryEnt::getPid, "1")
                            .isNotNull(SysDictionaryEnt::getSort).orderByDesc(SysDictionaryEnt::getSort);
                }else{
                    queryMaxSortEntity.lambda().eq(SysDictionaryEnt::getPid, input.getPid());
                }
                //查询最大的sort值
                queryMaxSortEntity.lambda().isNotNull(SysDictionaryEnt::getSort).orderByDesc(SysDictionaryEnt::getSort);
                List<SysDictionaryEnt> sysDictionarys = mapper.selectList(queryMaxSortEntity);
                if(ObjectUtils.isNotEmpty(sysDictionarys)){
                    SysDictionaryEnt sysDictionaryEnt1 = sysDictionarys.get(0);
                    if(sysDictionaryEnt1.getSort() != null){
                        entity.setSort(sysDictionaryEnt1.getSort()+1);
                    }else{
                        entity.setSort(1);
                    }
                }else{
                    entity.setSort(1);
                }
            }
            entity.setEffectTime(new Date());
            entity.setId(dicId);
            entity.setCreateName(context.getUser().getName());
            entity.setCreateTime(new Date());
            mapper.insert(entity);
            //更新字典缓存
            List<SysDictionaryCache> list = SysDictionaryCache.list(context, input.getAlias());
            SysDictionaryCache sysDictionaryCache = new SysDictionaryCache();
            XCopyUtils.copyObject(entity,sysDictionaryCache);
            list.add(sysDictionaryCache);
            SysDictionaryCache.set(context,alias,list);
            QuerySysDictionaryViewOutput out = XCopyUtils.copyNewObject(entity , QuerySysDictionaryViewOutput.class);
            return XSingleResult.success(out);
        });
    }

    @XText("删除字典")
    @Override
    public XServiceResult deleteSysDictionary(XContext context, DeleteSysDictionaryInput input) {
        SysDictionaryMapper mapper = context.getBean(SysDictionaryMapper.class);
        QueryWrapper<SysDictionaryEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysDictionaryEnt::getId, input.getId());
        SysDictionaryEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        SysDictionaryEnt sysDictionaryEnt = new SysDictionaryEnt();
        sysDictionaryEnt.setId(entity.getId());
        sysDictionaryEnt.setEndTime(new Date());
        mapper.updateById(sysDictionaryEnt);
        if(StringUtils.isBlank(entity.getAlias())){
            String alias = selectSysDictionaryAlias(context, entity.getPid());
            SysDictionaryCache.delete(context,alias,entity.getId());
        }else{
            SysDictionaryCache.delete(context,entity.getAlias(),entity.getId());
        }
        return XServiceResult.OK;
    }

    @XText("修改字典排序")
    @Override
    public XServiceResult updateDictionarySortById(XContext context, QuerySysDictionaryOutput input) {
        SysDictionaryViewMapper viewMapper = context.getBean(SysDictionaryViewMapper.class);
        SysDictionaryMapper mapper = context.getBean(SysDictionaryMapper.class);
        QueryWrapper<SysDictionaryEnt> query = new QueryWrapper();
        //查询当前字典项
        QueryWrapper<SysDictionaryEnt> queryWrapper = new QueryWrapper();
        queryWrapper.lambda().eq(SysDictionaryEnt::getId,input.getId());
        SysDictionaryEnt sysDictionaryEnt = mapper.selectOne(queryWrapper);
        //查询上一个或者下一个字典项
        query.lambda().eq(SysDictionaryEnt::getPid,sysDictionaryEnt.getPid());
        if(StringUtils.equals("up",input.getSortFlag())){
            query.lambda().lt(SysDictionaryEnt::getSort,sysDictionaryEnt.getSort())
                    .orderByDesc(SysDictionaryEnt::getSort);
        }else{
            query.lambda().gt(SysDictionaryEnt::getSort,sysDictionaryEnt.getSort())
                    .orderByAsc(SysDictionaryEnt::getSort);
        }
        List<SysDictionaryEnt> sysDictionaryEnts = mapper.selectList(query);
        Integer sort = 0;
        if(ObjectUtils.isNotEmpty(sysDictionaryEnts)){
            SysDictionaryEnt sysDictionaryEnt1 = sysDictionaryEnts.get(0);
            //交换排序值
            sort = sysDictionaryEnt1.getSort();
            sysDictionaryEnt1.setSort(sysDictionaryEnt.getSort());
            mapper.updateById(sysDictionaryEnt1);
        }else{
            sort = sysDictionaryEnt.getSort();
        }
        sysDictionaryEnt.setSort(sort);
        mapper.updateById(sysDictionaryEnt);
        return XServiceResult.OK;
    }

    //递归查询当前项最近的alias
    public String selectSysDictionaryAlias(XContext context, String pid){
        SysDictionaryMapper mapper = context.getBean(SysDictionaryMapper.class);
        String alias = "";
        while(StringUtils.isBlank(alias)){
            QueryWrapper<SysDictionaryEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(SysDictionaryEnt::getId, pid);
            SysDictionaryEnt sysDictionaryEnt = mapper.selectOne(queryWrapper);
            if(ObjectUtils.isNotEmpty(sysDictionaryEnt)) {
                if (StringUtils.isNotBlank(sysDictionaryEnt.getAlias())) {
                    alias = sysDictionaryEnt.getAlias();
                } else {
                    pid = sysDictionaryEnt.getPid();
                }
            }else{
                return "";
            }
        }
        return alias;
    }

    @Override
    @XText("根据dicPath获取所有字典列表")
    public XListResult<QuerySysDictionaryViewOutput> queryChildByDicPath(XContext context, QuerySysDictionaryViewInput input) {
        SysDictionaryViewMapper mapper = context.getBean(SysDictionaryViewMapper.class);
        SysDictionaryView record = new SysDictionaryView();
        XCopyUtils.copyObject(input, record);
        List<SysDictionaryView> list = mapper.selectListsByDicPath(record);
        List<QuerySysDictionaryViewOutput> outputs = XCopyUtils.copyNewList(list, QuerySysDictionaryViewOutput.class);
        return XListResult.success(outputs);
    }

    @Override
    @XText("根据dicPath获取所有字典列表(含本身)")
    public XListResult<QuerySysDictionaryViewOutput> queryChildByDicPathIncludeSelf(XContext context, QuerySysDictionaryViewInput input) {
        SysDictionaryViewMapper mapper = context.getBean(SysDictionaryViewMapper.class);
        SysDictionaryView record = new SysDictionaryView();
        XCopyUtils.copyObject(input, record);
        List<SysDictionaryView> list = mapper.selectListsByDicPathIncludeSelf(record);
        List<QuerySysDictionaryViewOutput> outputs = XCopyUtils.copyNewList(list, QuerySysDictionaryViewOutput.class);
        return XListResult.success(outputs);
    }

    @Override
    public XSingleResult<QuerySysDictionaryViewOutput> queryByAlias(XContext context,
                                                                    QuerySysDictionaryViewInput input) {
        SysDictionaryMapper mapper = context.getBean(SysDictionaryMapper.class);
        QueryWrapper<SysDictionaryEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysDictionaryEnt::getAlias, input.getAlias());
        SysDictionaryEnt ent = mapper.selectOne(queryWrapper);
        QuerySysDictionaryViewOutput output = new QuerySysDictionaryViewOutput();
        XCopyUtils.copyObject(ent, output);
        return XSingleResult.success(output);
    }

    @Override
    public XSingleResult<QuerySysDictionaryViewOutput> queryByDicName(XContext context,
                                                                      QuerySysDictionaryViewInput input) {
        SysDictionaryMapper mapper = context.getBean(SysDictionaryMapper.class);
        QueryWrapper<SysDictionaryEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysDictionaryEnt::getDicName, input.getDicName());
        SysDictionaryEnt ent = mapper.selectOne(queryWrapper);
        QuerySysDictionaryViewOutput output = new QuerySysDictionaryViewOutput();
        XCopyUtils.copyObject(ent, output);
        return XSingleResult.success(output);
    }

    @Override
    public XSingleResult<QuerySysDictionaryViewOutput> queryByDicKey(XContext context, QuerySysDictionaryViewInput input){
        SysDictionaryMapper mapper = context.getBean(SysDictionaryMapper.class);
        QueryWrapper<SysDictionaryEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysDictionaryEnt::getDicKey, input.getDicKey());
        queryWrapper.last(" limit 1");
        SysDictionaryEnt ent = mapper.selectOne(queryWrapper);
        QuerySysDictionaryViewOutput output = new QuerySysDictionaryViewOutput();
        XCopyUtils.copyObject(ent, output);
        return XSingleResult.success(output);
    }
    @Override
    public XServiceResult deleteCache(XContext context, DeleteCacheInput input) {
        SysDictionaryCache.deleteCache(context, input.getAlias());
        return XServiceResult.OK;
    }
}
