package pps.core.system.service;


import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.springframework.util.CollectionUtils;
import pps.cloud.system.service.data.GetSysOrganizationViewInput;
import pps.cloud.system.service.data.GetSysOrganizationViewOutput;
import pps.core.common.config.MqConfig;
import pps.core.common.mq.PushDispLngShippingScheduleDateValueMessage;
import pps.core.common.mq.SysInformSendMqMessage;
import pps.core.common.session.PpsUserSession;
import pps.core.system.constant.SysInformHandleTypeConstant;
import pps.core.system.constant.SysInformReceiveTypeConstant;
import pps.core.system.entity.*;
import pps.core.system.mapper.*;
import pps.core.system.service.data.sys_inform.*;
import pps.core.system.service.data.sys_inform_receive_type.GetSysInformReceiveTypeInput;
import pps.core.system.service.data.sys_inform_receive_type.GetSysInformReceiveTypeOutput;
import pps.core.system.service.data.sys_inform_receive_type_detail.GetSysInformReceiveTypeDetailOutput;
import pps.core.system.service.data.sys_inform_receiver.UpdateSysInformReceiverListInput;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.mq.XMqFactory;
import xstartup.base.mq.XMqProduceQueue;
import xstartup.base.mq.XMqProducer;
import xstartup.base.util.XCopyUtils;
import xstartup.base.util.XStringUtils;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.helper.XTransactionHelper;

import java.util.*;
import java.util.stream.Collectors;

@XService
@XText("消息通知")
public class SysInformService {

    @XText("查询通知列表数据")
    @XApiGet
    public XPageResult<QuerySysInformViewOutput> selectSysInformPage(XContext context, QuerySysInformViewInput input){
        SysInformViewMapper mapper = context.getBean(SysInformViewMapper.class);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        input.setOuId(session.getOuId());
        SysInformView record = new SysInformView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<SysInformView> list = mapper.selectList(record);
        PageInfo<SysInformView> pageInfo = new PageInfo<>(list);
        List<QuerySysInformViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySysInformViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    @XText("创建/更新接收方数据状态")
    @XApiPost
    public XServiceResult updateSysInform(XContext context, UpdateSysInformListInput input){
        return XTransactionHelper.begin(context , ()->{
            //判断创建和更新操作
            Date date = new Date();
            PpsUserSession session = context.getSession(PpsUserSession.class);
            SysInformMapper mapper = context.getBean(SysInformMapper.class);
            SysInformReceiveTypeMapper receiveTypeMapper = context.getBean(SysInformReceiveTypeMapper.class);
            SysInformReceiveTypeDetailMapper receiveTypeDetailMapper = context.getBean(SysInformReceiveTypeDetailMapper.class);
            SysInformEnt sysInformEnt = XCopyUtils.copyNewObject(input ,SysInformEnt.class);
            if(!XStringUtils.isEmpty(sysInformEnt.getId())){
                QueryWrapper<SysInformEnt> wrapper = new QueryWrapper<>();
                wrapper.lambda().eq(SysInformEnt::getId , input.getId());
                SysInformEnt ent = mapper.selectOne(wrapper);
                if(ent == null)
                    return XServiceResult.error(context, XError.NotFound);
                //更新时，先删除历史接收方式数据
                HashMap<String, Object> deleteMapper = new HashMap();
                deleteMapper.put("inform_id", sysInformEnt.getId());
                receiveTypeMapper.deleteByMap(deleteMapper);
                receiveTypeDetailMapper.deleteByMap(deleteMapper);
            }
            sysInformEnt.setOuId(session.getOuId());
            sysInformEnt.setOuName(session.getOuName());
            sysInformEnt.setModifyById(session.getId());
            sysInformEnt.setUserId(session.getId());
            sysInformEnt.setUserName(session.getUserName());
            sysInformEnt.setStatus(0);
            sysInformEnt.setSendType(1);
            sysInformEnt.setModifyTime(date);
            sysInformEnt.setModifyByName(session.getUserName());
            if(XStringUtils.isEmpty(sysInformEnt.getId())){
                sysInformEnt.setId(UUID.randomUUID().toString());
                sysInformEnt.setCreateById(session.getId());
                sysInformEnt.setCreateTime(date);
                sysInformEnt.setCreateByName(session.getUserName());
                mapper.insert(sysInformEnt);
            }else
                mapper.updateById(sysInformEnt);
            //插入子数据
            if(!CollectionUtils.isEmpty(input.getReceiveTypeList())){
                for(GetSysInformReceiveTypeInput receiveTypeInput : input.getReceiveTypeList()){
                    SysInformReceiveTypeEnt receiveTypeEnt = new SysInformReceiveTypeEnt();
                    receiveTypeEnt.setId(UUID.randomUUID().toString());
                    receiveTypeEnt.setRecieveType(receiveTypeInput.getRecieveType());
                    receiveTypeEnt.setRecieveTypeName(SysInformReceiveTypeConstant.findByValue(receiveTypeInput.getRecieveType()).getDesc());
                    receiveTypeEnt.setInformId(sysInformEnt.getId());
                    receiveTypeInput.getReceiveTypeDetailList().stream().forEach(item->{
                        SysInformReceiveTypeDetailEnt detailEnt = XCopyUtils.copyNewObject(item , SysInformReceiveTypeDetailEnt.class);
                        detailEnt.setId(UUID.randomUUID().toString());
                        detailEnt.setInformId(sysInformEnt.getId());
                        detailEnt.setInformRecieveTypeId(receiveTypeEnt.getId());
                        if(receiveTypeEnt.getRecieveType() == 8)
                            detailEnt.setObjId("call " + detailEnt.getObjId() + "();");
                        receiveTypeDetailMapper.insert(detailEnt);
                    });
                    receiveTypeMapper.insert(receiveTypeEnt);
                }
            }
            if( !CollectionUtils.isEmpty(input.getFileKeyList())){
                List<String> fileKeyList=input.getFileKeyList();
                SysInformFileMapper fileMapper=context.getBean(SysInformFileMapper.class);
                fileMapper.deleteBySysInformId(sysInformEnt.getId());
                //插入附件
                if(!fileKeyList.isEmpty())
                {
                    List<SysInformFileEnt> fileList=new ArrayList<>();
                    for (int i = 0; i < fileKeyList.size(); i++) {
                        String fileId =  fileKeyList.get(i);
                        SysInformFileEnt fileEntity=new SysInformFileEnt();
                        fileEntity.setId(UUID.randomUUID().toString());
                        fileEntity.setFileId(fileId);
                        fileEntity.setSysInformId(sysInformEnt.getId());
                        fileList.add(fileEntity);
                    }
                    fileMapper.insertBatch(fileList);
                }
            }
            return XServiceResult.OK;
        });
    }

    @XText("删除接收方数据状态")
    @XApiPost
    public XServiceResult deleteSysInform(XContext context, DeleteSysInformInput input){
        return XTransactionHelper.begin(context , ()->{
            //判断创建和更新操作
            Date date = new Date();
            PpsUserSession session = context.getSession(PpsUserSession.class);
            SysInformMapper mapper = context.getBean(SysInformMapper.class);
            QueryWrapper<SysInformEnt> wrapper = new QueryWrapper<>();
            wrapper.lambda().eq(SysInformEnt::getId , input.getId());
            SysInformEnt ent = mapper.selectOne(wrapper);
            if(ent == null)
                return XServiceResult.error(context, XError.NotFound);
            if(ent.getStatus() >= 1)
                return XServiceResult.error(200, "数据状态不支持删除！");
            ent.setStatus(2);
            ent.setModifyById(session.getId());
            ent.setModifyTime(date);
            ent.setModifyByName(session.getUserName());
            mapper.updateById(ent);
            return XServiceResult.OK;
        });
    }

    @XText("获取通知详情")
    @XApiGet
    public XSingleResult<GetSysInformOutput> getSysInform(XContext context, GetSysInformInput input){
        SysInformMapper mapper = context.getBean(SysInformMapper.class);
        QueryWrapper<SysInformEnt> wrapper = new QueryWrapper<>();
        wrapper.lambda().eq(SysInformEnt::getId , input.getId());
        SysInformEnt ent = mapper.selectOne(wrapper);
        if(ent == null)
            return XSingleResult.error(context, XError.NotFound);
        GetSysInformOutput output = XCopyUtils.copyNewObject(ent , GetSysInformOutput.class);
        //查询子级
        SysInformReceiveTypeMapper receiveTypeMapper = context.getBean(SysInformReceiveTypeMapper.class);
        QueryWrapper<SysInformReceiveTypeEnt> receiveTypeWrapper = new QueryWrapper<>();
        receiveTypeWrapper.lambda().eq(SysInformReceiveTypeEnt::getInformId , ent.getId());
        List<SysInformReceiveTypeEnt> receiveTypeList = receiveTypeMapper.selectList(receiveTypeWrapper);
        if(!CollectionUtils.isEmpty(receiveTypeList)){
            SysInformReceiveTypeDetailMapper receiveTypeDetailMapper = context.getBean(SysInformReceiveTypeDetailMapper.class);
            List<GetSysInformReceiveTypeOutput> receiveTypeOutputs = XCopyUtils.copyNewList(receiveTypeList , GetSysInformReceiveTypeOutput.class);
            receiveTypeOutputs.stream().forEach(item->{
                //查询子级的子级
                QueryWrapper<SysInformReceiveTypeDetailEnt> receiveTypeDetailWrapper = new QueryWrapper<>();
                receiveTypeDetailWrapper.lambda().eq(SysInformReceiveTypeDetailEnt::getInformId , ent.getId());
                receiveTypeDetailWrapper.lambda().eq(SysInformReceiveTypeDetailEnt::getInformRecieveTypeId , item.getId());
                List<SysInformReceiveTypeDetailEnt> receiveTypeDetailList = receiveTypeDetailMapper.selectList(receiveTypeDetailWrapper);
                List<GetSysInformReceiveTypeDetailOutput> receiveTypeDetailOutputList = XCopyUtils.copyNewList(receiveTypeDetailList , GetSysInformReceiveTypeDetailOutput.class);
                item.setReceiveTypeDetailList(receiveTypeDetailOutputList);
            });
            output.setReceiveTypeList(receiveTypeOutputs);
        }
        SysInformFileMapper fileMapper=context.getBean(SysInformFileMapper.class);
        LambdaQueryWrapper<SysInformFileEnt> queryFileWrapper = new LambdaQueryWrapper<>();
        queryFileWrapper.eq(SysInformFileEnt::getSysInformId, input.getId());
        List<SysInformFileEnt> fileList=fileMapper.selectList(queryFileWrapper);
        output.setFileKeyList(fileList.stream().map(SysInformFileEnt::getFileId).collect(Collectors.toList()));
        return XSingleResult.success(output);
    }

    @XText("获取消息通知类型")
    @XApiGet
    public XListResult<GetSysInformTypeOutput> getInformTypes(XContext context, GetSysInformTypeOutput input){
        List<GetSysInformTypeOutput> outputs = new ArrayList<>();
        SysInformData sysInformData = new SysInformData();
        if(input.getType() == 0){
            //操作状态
            List<SysInformHandleTypeConstant> list = sysInformData.getHandleTypsSetting();
            list.stream().forEach(item->{
                GetSysInformTypeOutput output = new GetSysInformTypeOutput();
                output.setKey(item.getValue());
                output.setValue(item.getDesc());
                outputs.add(output);
            });
        }else{
            //接收方式
            List<SysInformReceiveTypeConstant> list = sysInformData.getReceiveTypsSetting();
            list.stream().forEach(item->{
                GetSysInformTypeOutput output = new GetSysInformTypeOutput();
                output.setKey(item.getValue());
                output.setValue(item.getDesc());
                outputs.add(output);
            });
        }
        return XListResult.success(outputs);
    }

    @XText("通过当前用户接收类型")
    @XApiGet
    public XListResult<GetSysReceiveTypeOutput> getReceiveTypesByUser(XContext context, GetSysInformTypeOutput input){
        List<GetSysReceiveTypeOutput> outputs = new ArrayList<>();
        SysInformData sysInformData = new SysInformData();
        //接收方式
        PpsUserSession session = context.getSession(PpsUserSession.class);
        SysOrganizationMapper mapper = context.getBean(SysOrganizationMapper.class);
        QueryWrapper<SysOrganizationEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysOrganizationEnt::getId, session.getOuId());
        SysOrganizationEnt ent = mapper.selectOne(queryWrapper);
        List<SysInformReceiveTypeConstant> list = sysInformData.getReceiveTypsSetting();
        for(SysInformReceiveTypeConstant item : list){
            GetSysReceiveTypeOutput output = new GetSysReceiveTypeOutput();
            output.setOuId(session.getOuId());
            output.setOuName(session.getOuName());
            if(item.getValue() >= 5 && item.getValue() < 8)//去掉角色-提交人*
                continue;
            if((ent.getOuLevel().equals("REGION_FIRM")
                    ||ent.getOuLevel().equals("4097")) && item.getValue() >=6){
                //1.所属企业
                continue;
            }else if(ent.getOuLevel().equals("ADMINISTRATIVE")
                    ||ent.getOuLevel().equals("4099")){
                //二级单位，获取所属企业id
                if(item.getValue() == 6)//如果是二级单位，跳过
                    continue;
                if(item.getValue() == 7){
                    //如果是所属企业，获取
                    SysOrganizationView view = new SysOrganizationView();
                    view.setOuLevel("REGION_FIRM");
                    GetSysOrganizationViewOutput organizationViewOutput = getSysOrganizationByOuLevelAndUserId(context , view);
                    output.setOuId(organizationViewOutput.getId());
                    output.setOuName(organizationViewOutput.getOuName());
                }
            }else if(ent.getOuLevel().equals("STATION")
                    ||ent.getOuLevel().equals("4106")){
                //3.场站，获取二级单位和所属企业的id
                SysOrganizationView view = new SysOrganizationView();
                if(item.getValue() == 6){
                    //如果是二级单位
                    view.setOuLevel("ADMINISTRATIVE");
                }
                if(item.getValue() == 7){
                    //如果是所属企业
                    view.setOuLevel("REGION_FIRM");
                }
                GetSysOrganizationViewOutput organizationViewOutput = getSysOrganizationByOuLevelAndUserId(context , view);
                output.setOuId(organizationViewOutput.getId());
                output.setOuName(organizationViewOutput.getOuName());
            }else if(item.getValue() >=6 && item.getValue() < 8 ){
                //其他
                continue;
            }
            output.setKey(item.getValue());
            output.setValue(item.getDesc());
            outputs.add(output);
        }
        return XListResult.success(outputs);
    }

    @XText("通过组织机构接收类型")
    @XApiGet
    public XListResult<GetSysReceiveTypeOutput> getReceiveTypesByOrg(XContext context, GetSysInformTypeOutput input){
        List<GetSysReceiveTypeOutput> outputs = new ArrayList<>();
        SysInformData sysInformData = new SysInformData();
        //接收方式
        if(XStringUtils.isEmpty(input.getOuId())){
            List<SysInformReceiveTypeConstant> list = sysInformData.getReceiveTypsSetting();
            for(SysInformReceiveTypeConstant item : list){
                GetSysReceiveTypeOutput output = new GetSysReceiveTypeOutput();
                if((item.getValue()>=5 && item.getValue()<=7) || item.getValue() == 9)
                    continue;
                output.setKey(item.getValue());
                output.setValue(item.getDesc());
                outputs.add(output);
            }
        }else {
            SysOrganizationMapper mapper = context.getBean(SysOrganizationMapper.class);
            QueryWrapper<SysOrganizationEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(SysOrganizationEnt::getId, input.getOuId());
            SysOrganizationEnt ent = mapper.selectOne(queryWrapper);
            List<SysInformReceiveTypeConstant> list = sysInformData.getReceiveTypsSetting();
            for(SysInformReceiveTypeConstant item : list){
                GetSysReceiveTypeOutput output = new GetSysReceiveTypeOutput();
                output.setOuId(ent.getId());
                output.setOuName(ent.getOuName());
                if((ent.getOuLevel().equals("REGION_FIRM")
                        ||ent.getOuLevel().equals("4097") || item.getValue() == 9) && item.getValue() >=6 && item.getValue() < 8){
                    //1.所属企业
                    continue;
                }else if(ent.getOuLevel().equals("ADMINISTRATIVE")
                        ||ent.getOuLevel().equals("4099")){
                    //二级单位，获取所属企业id
                    if(item.getValue() == 6)//如果是二级单位，跳过
                        continue;
                    if(item.getValue() == 7){
                        //如果是所属企业，获取
                        SysOrganizationView view = new SysOrganizationView();
                        view.setOuLevel("REGION_FIRM");
                        view.setId(input.getOuId());
                        GetSysOrganizationViewOutput organizationViewOutput = getSysOrganizationByOuLevelAndOuId(context , view);
                        output.setOuId(organizationViewOutput.getId());
                        output.setOuName(organizationViewOutput.getOuName());
                    }
                }else if(ent.getOuLevel().equals("STATION")
                        ||ent.getOuLevel().equals("4106")){
                    //3.场站，获取二级单位和所属企业的id
                    SysOrganizationView view = new SysOrganizationView();
                    view.setId(input.getOuId());
                    if(item.getValue() == 6){
                        //如果是二级单位
                        view.setOuLevel("ADMINISTRATIVE");
                        GetSysOrganizationViewOutput organizationViewOutput = getSysOrganizationByOuLevelAndOuId(context , view);
                        output.setOuId(organizationViewOutput.getId());
                        output.setOuName(organizationViewOutput.getOuName());
                    }
                    if(item.getValue() == 7){
                        //如果是所属企业
                        view.setOuLevel("REGION_FIRM");
                        GetSysOrganizationViewOutput organizationViewOutput = getSysOrganizationByOuLevelAndOuId(context , view);
                        output.setOuId(organizationViewOutput.getId());
                        output.setOuName(organizationViewOutput.getOuName());
                    }
                    if(item.getValue() == 9)
                        continue;
                }else if((item.getValue() >=6 && item.getValue() < 8) || item.getValue() == 9){
                    //其他
                    continue;
                }
                output.setKey(item.getValue());
                output.setValue(item.getDesc());
                outputs.add(output);
            }
        }

        return XListResult.success(outputs);
    }

    @XText("发送消息")
    @XApiPost
    public XServiceResult sendSysInform(XContext context, SendSysInformInput input){
        return XTransactionHelper.begin(context , ()->{
            //判断创建和更新操作
            Date date = new Date();
            PpsUserSession session = context.getSession(PpsUserSession.class);
            SysInformMapper mapper = context.getBean(SysInformMapper.class);
            QueryWrapper<SysInformEnt> wrapper = new QueryWrapper<>();
            wrapper.lambda().eq(SysInformEnt::getId , input.getId());
            SysInformEnt ent = mapper.selectOne(wrapper);
            if(ent == null)
                return XServiceResult.error(context, XError.NotFound);
            if(ent.getStatus() >= 1)
                return XServiceResult.error(200, "数据状态不支持发送！");
            SysInformSendMqMessage sysInformSendMqMessage =new SysInformSendMqMessage();
            sysInformSendMqMessage.setId(input.getId());
            sendMqSysInform(context, sysInformSendMqMessage);
            //执行完成后，更新状态
            ent.setStatus(1);
            ent.setModifyById(session.getId());
            ent.setModifyTime(date);
            ent.setSendTime(date);
            ent.setModifyByName(session.getUserName());
            mapper.updateById(ent);
            return XServiceResult.OK;
        });
    }

    public void sendMqSysInform(XContext context, SysInformSendMqMessage sysInformSendMqMessage) {
        String schedulTopic = MqConfig.sysInformSendMq.getValue(context);
        XMqProduceQueue queue = new XMqProduceQueue.Builder().queue(schedulTopic).build();
        XMqProducer<SysInformSendMqMessage> producer = XMqFactory.createProducer(context, queue, SysInformSendMqMessage.class);
        producer.publish(context, sysInformSendMqMessage).logIfFail(context);
    }

    public GetSysOrganizationViewOutput getSysOrganizationByOuLevelAndUserId(XContext context, SysOrganizationView view) {
        SysOrganizationViewMapper viewMapper = context.getBean(SysOrganizationViewMapper.class);
        view.setEndTime(new Date());
        view.setUserId(context.getSession(PpsUserSession.class).getId());
        SysOrganizationView sysOrganizationView = viewMapper.getSysOrganizationByOuLevelAndUserId(view);
        GetSysOrganizationViewOutput organizationViewOutput = XCopyUtils.copyNewObject(sysOrganizationView, GetSysOrganizationViewOutput.class);
        return organizationViewOutput;
    }

    public GetSysOrganizationViewOutput getSysOrganizationByOuLevelAndOuId(XContext context, SysOrganizationView view) {
        SysOrganizationViewMapper viewMapper = context.getBean(SysOrganizationViewMapper.class);
        view.setEndTime(new Date());
        view.setId(view.getId());
        SysOrganizationView sysOrganizationView = viewMapper.getSysOrganizationByOuLevelAndOuId(view);
        GetSysOrganizationViewOutput organizationViewOutput = XCopyUtils.copyNewObject(sysOrganizationView, GetSysOrganizationViewOutput.class);
        return organizationViewOutput;
    }
}
