package pps.core.system.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.CollectionUtils;
import pps.cloud.base.service.BaseBaseStationService;
import pps.cloud.base.service.data.BaseOuIdInput;
import pps.cloud.base.service.data.QueryBaseStationInput;
import pps.cloud.base.service.data.QueryBaseStationOutput;
import pps.cloud.system.service.SystemDictionaryService;
import pps.cloud.system.service.data.CreateSysDictionaryInput;
import pps.cloud.system.service.data.GetOuListTreeOutput;
import pps.cloud.system.service.data.QuerySysDictionaryViewOutput;
import pps.cloud.system.service.data.SelectRegionFirmInput;
import pps.core.common.constant.Constant;
import pps.core.common.constant.UserConstant;
import pps.core.common.session.PpsUserSession;
import pps.core.common.session.SysOrganization;
import pps.core.system.cache.SysOrganizationCache;
import pps.core.system.constant.OuLevelConstant;
import pps.core.system.constant.SysOrganizationCode;
import pps.core.system.entity.*;
import pps.core.system.mapper.*;
import pps.core.system.service.data.sys_organization.*;
import pps.core.system.service.data.sys_organization_rel.GetOuInfoByConditionInput;
import pps.core.system.service.data.sys_organization_rel.GetSysOrganizationRelInput;
import pps.core.system.service.data.sys_organization_rel.QuerySysOrganizationRelInput;
import pps.core.system.service.data.sys_organization_rel.UpdateSysOrganizationRelInput;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.exception.XServiceException;
import xstartup.base.util.XCopyUtils;
import xstartup.base.util.XDateUtils;
import xstartup.base.util.XStringUtils;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.helper.XTransactionHelper;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

@XText("组织机构接口")
@XService
public class SysOrganizationService {


    @XText("根据ouName查询组织机构树和场站")
    @XApiPost
    public XListResult<QuerySysOrganizationViewOutputTree> selectOuTreeAndStationByOuNameAndStationName(XContext context, GetSysOrganizationInput input) {
        List<String> ouIds = new ArrayList<>();
        BaseBaseStationService stationService = context.getBean(BaseBaseStationService.class);
        QueryBaseStationInput stationInput = new QueryBaseStationInput();
        //根据模糊场站名称查场站
        stationInput.setStationName(input.getOuName());
        XListResult<QueryBaseStationOutput> listResult = stationService.selectStationAllByStationName(context, stationInput);
        List<QueryBaseStationOutput> stationLikeList = listResult.getResult();
        if (!CollectionUtils.isEmpty(stationLikeList)) {
            for (QueryBaseStationOutput baseStationView : stationLikeList) {
                if (XStringUtils.isNotEmpty(baseStationView.getOuId())) {
                    ouIds.add(baseStationView.getOuId());
                }
            }
        }

        SysOrganizationViewMapper mapper = context.getBean(SysOrganizationViewMapper.class);
        SysOrganizationView record = new SysOrganizationView();
        XCopyUtils.copyObject(input, record);
        record.setIsDeleted(SysOrganizationCode.NORMAL);
        record.setEndTime(new Date());
        record.setOuIds(ouIds);
        List<SysOrganizationView> viewList = mapper.selectOuByOuIdsAndOuName(record);
        List<QuerySysOrganizationViewOutputTree> list = XCopyUtils.copyNewList(viewList, QuerySysOrganizationViewOutputTree.class);


        List<QuerySysOrganizationViewOutputTree> trees = new ArrayList<>();
        PpsUserSession session = context.getSession(PpsUserSession.class);
        Integer isSuperAdmin = session.getIsSuperAdmin();
        for (QuerySysOrganizationViewOutputTree tree : list) {

            //如果是超级管理员查所以组织机构
            if (UserConstant.ISSUPERADMIN_TRUE.equals(isSuperAdmin)) {
                if (tree.getParentOuId().equals("00000000-0000-0000-0000-000000000000")) {
                    trees.add(tree);
                }
            } else {
                if (tree.getId().equals(session.getOuId())) {
                    trees.add(tree);
                }
            }

            //子集
            for (QuerySysOrganizationViewOutputTree it : list) {

                if (OuLevelConstant.CHANG_ZHAN.getCode().equals(it.getOuLevel()) && !CollectionUtils.isEmpty(stationLikeList)) {
                    List<QuerySysOrganizationViewOutputTree> stationList = new ArrayList<>();
                    for (QueryBaseStationOutput baseStationView : stationLikeList) {
                        if (it.getId().equals(baseStationView.getOuId())) {
                            QuerySysOrganizationViewOutputTree outputTree = new QuerySysOrganizationViewOutputTree();
                            outputTree.setId(baseStationView.getId());
                            outputTree.setOuName(baseStationView.getStationName());
                            outputTree.setType("station");
                            stationList.add(outputTree);
                        }
                    }
                    it.setChildren(stationList);
                }
                if (tree.getId().equals(it.getParentOuId())) {
                    if (tree.getChildren() == null) {
                        tree.setChildren(new ArrayList<>());
                    }
                    tree.getChildren().add(it);
                }
            }
            if (tree.getChildren() != null && tree.getChildren().size() > 1) {
                List<QuerySysOrganizationViewOutputTree> children = tree.getChildren();
                children.forEach(outputTree -> {
                    if (outputTree.getSort() == null) {
                        outputTree.setSort(999);
                    }
                });
                children = children.stream().sorted(Comparator.comparing(QuerySysOrganizationViewOutputTree::getSort)).collect(Collectors.toList());
                tree.setChildren(children);
            }
        }
        trees.forEach(outputTree -> {
            if (outputTree.getSort() == null) {
                outputTree.setSort(999);
            }
        });
        trees = trees.stream().sorted(Comparator.comparing(QuerySysOrganizationViewOutputTree::getSort)).collect(Collectors.toList());

        return XListResult.success(trees);
    }

    @XText("根据父级id查询儿子级--最后一级查的是场站")
    @XApiPost
    public XListResult<GetSysOrganizationViewOutput> selectOuInfoOrStationByParentOuId(XContext context, GetSysOrganizationRelInput input) {
        SysOrganizationViewMapper mapper = context.getBean(SysOrganizationViewMapper.class);
        SysOrganizationView record = new SysOrganizationView();
        XCopyUtils.copyObject(input, record);
        record.setEndTime(new Date());
        record.setIsDeleted(SysOrganizationCode.NORMAL);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        Integer isSuperAdmin = session.getIsSuperAdmin();
        List<SysOrganizationView> viewList = new ArrayList<>();

        if (XStringUtils.isNotEmpty(input.getOuLevel()) && OuLevelConstant.CHANG_ZHAN.getCode().equals(input.getOuLevel())) {
            if (XStringUtils.isNotEmpty(input.getParentOuId())) {
                BaseBaseStationService stationService = context.getBean(BaseBaseStationService.class);
                BaseOuIdInput ouIdInput = new BaseOuIdInput();
                ouIdInput.setOuId(input.getParentOuId());
                XListResult<QueryBaseStationOutput> viewsResult = stationService.selectStationListByOuId(context, ouIdInput);
                List<QueryBaseStationOutput> stationViews = viewsResult.getResult();
                if (CollectionUtils.isEmpty(stationViews)) {
                    return XListResult.success(new ArrayList<>());
                }
                for (QueryBaseStationOutput stationView : stationViews) {
                    SysOrganizationView organizationView = new SysOrganizationView();
                    organizationView.setId(stationView.getId());
                    organizationView.setOuName(stationView.getStationName());
                    organizationView.setType("station");
                    viewList.add(organizationView);
                }
                List<GetSysOrganizationViewOutput> outputs = XCopyUtils.copyNewList(viewList, GetSysOrganizationViewOutput.class);
                return XListResult.success(outputs);

            }
        }
        if (XStringUtils.isEmpty(input.getParentOuId())) {
            //如果是超级管理员查所以组织机构
            if (UserConstant.ISSUPERADMIN_TRUE.equals(isSuperAdmin) || OuLevelConstant.YOU_QI_DIAO_KONG_ZHONG_XIN.getCode().equals(session.getOwnerOrgLevel())) {
                record.setParentOuId("00000000-0000-0000-0000-000000000000");
                viewList = mapper.selectOuSonByParentOuId(record);
            } else {
                record.setId(session.getOuId());
                viewList = mapper.selectOuByOuId(record);
            }
        } else {
            viewList = mapper.selectOuSonByParentOuId(record);
        }


        List<GetSysOrganizationViewOutput> outputs = XCopyUtils.copyNewList(viewList, GetSysOrganizationViewOutput.class);
        return XListResult.success(outputs);
    }


    @XText("根据用户id查询地区公司")
    @XApiPost
    public XListResult<GetSysOrganizationViewOutput> selectRegionFirmByUserId(XContext context, SelectRegionFirmInput input) {

        PpsUserSession session = context.getSession(PpsUserSession.class);
        SysOrganizationMapper bean = context.getBean(SysOrganizationMapper.class);
        QueryWrapper<SysOrganizationEnt> wrapper = new QueryWrapper<>();
        String ouId = session.getOuId();
        if (StringUtils.isNotBlank(input.getOuId())) {
            ouId = input.getOuId();
        }
        wrapper.lambda().eq(SysOrganizationEnt::getId, ouId).eq(SysOrganizationEnt::getIsDeleted, SysOrganizationCode.NORMAL);
        SysOrganizationEnt ou = bean.selectOne(wrapper);
        if (ou == null) {
            return XListResult.success(new ArrayList<>());
        }
        List<GetSysOrganizationViewOutput> outputs = new ArrayList<>();

        //查询当前用户所属组织机构的上级是有油气调控中心
        SysOrganizationViewMapper viewMapper = context.getBean(SysOrganizationViewMapper.class);
        SysOrganizationView view = new SysOrganizationView();
        view.setEndTime(new Date());
        view.setId(ouId);
        view.setOuLevel(OuLevelConstant.YOU_QI_DIAO_KONG_ZHONG_XIN.getCode());
        SysOrganizationView organization = viewMapper.getSysOrganizationByOuLevelAndOuId(view);

        if (OuLevelConstant.ZHUAN_YE_GONG_SI.getCode().equals(ou.getOuLevel()) || organization != null && OuLevelConstant.YOU_QI_DIAO_KONG_ZHONG_XIN.getCode().equals(organization.getOuLevel())) {
            wrapper = new QueryWrapper<>();
            wrapper.lambda().eq(SysOrganizationEnt::getOuLevel, OuLevelConstant.DI_QU_GONG_SI.getCode())
                    .eq(SysOrganizationEnt::getIsDeleted, SysOrganizationCode.NORMAL)
                    .orderBy(true, true, SysOrganizationEnt::getId);
            List<SysOrganizationEnt> organizationEnts = bean.selectList(wrapper);
            outputs = XCopyUtils.copyNewList(organizationEnts, GetSysOrganizationViewOutput.class);
        } else {
            view = new SysOrganizationView();
            view.setEndTime(new Date());
            view.setId(ouId);
            view.setOuLevel(OuLevelConstant.DI_QU_GONG_SI.getCode());
            SysOrganizationView organization1 = viewMapper.getSysOrganizationByOuLevelAndOuId(view);
            GetSysOrganizationViewOutput output = new GetSysOrganizationViewOutput();
            XCopyUtils.copyObject(organization1, output);
            outputs.add(output);
        }

        return XListResult.success(outputs);
    }

    @XText("根据ouName查询组织机构树")
    @XApiPost
    public XListResult<QuerySysOrganizationViewOutputTree> selectSysOrganizationTreeByOuName(XContext context, GetSysOrganizationInput input) {
        SysOrganizationViewMapper mapper = context.getBean(SysOrganizationViewMapper.class);
        SysOrganizationView record = new SysOrganizationView();
        XCopyUtils.copyObject(input, record);
        record.setIsDeleted(SysOrganizationCode.NORMAL);
        record.setEndTime(new Date());
        List<SysOrganizationView> viewList = mapper.selectOuTreeByOuName(record);
        List<QuerySysOrganizationViewOutputTree> list = XCopyUtils.copyNewList(viewList, QuerySysOrganizationViewOutputTree.class);
        List<QuerySysOrganizationViewOutputTree> trees = new ArrayList<>();
        PpsUserSession session = context.getSession(PpsUserSession.class);
        Integer isSuperAdmin = session.getIsSuperAdmin();
        for (QuerySysOrganizationViewOutputTree tree : list) {

            //如果是超级管理员查所以组织机构
            if (UserConstant.ISSUPERADMIN_TRUE.equals(isSuperAdmin)) {
                if (tree.getParentOuId().equals("00000000-0000-0000-0000-000000000000")) {
                    trees.add(tree);
                }
            } else {
                if (tree.getId().equals(session.getOuId())) {
                    trees.add(tree);
                }
            }

            //子集
            for (QuerySysOrganizationViewOutputTree it : list) {
                if (tree.getId().equals(it.getParentOuId())) {
                    if (tree.getChildren() == null) {
                        tree.setChildren(new ArrayList<>());
                    }
                    tree.getChildren().add(it);
                }
            }
            if (tree.getChildren() != null && tree.getChildren().size() > 1) {
                List<QuerySysOrganizationViewOutputTree> children = tree.getChildren();
                children.forEach(outputTree -> {
                    if (outputTree.getSort() == null) {
                        outputTree.setSort(999);
                    }
                });
                children = children.stream().sorted(Comparator.comparing(QuerySysOrganizationViewOutputTree::getSort)).collect(Collectors.toList());
                tree.setChildren(children);
            }
        }
        trees.forEach(outputTree -> {
            if (outputTree.getSort() == null) {
                outputTree.setSort(999);
            }
        });
        trees = trees.stream().sorted(Comparator.comparing(QuerySysOrganizationViewOutputTree::getSort)).collect(Collectors.toList());

        return XListResult.success(trees);
    }


    @XText("根据条件组织机构树")
    @XApiPost
    public XListResult<QuerySysOrganizationViewOutputTree> getOuTreeByCondition(XContext context, GetOuInfoByConditionInput input) {
        SysOrganizationViewMapper mapper = context.getBean(SysOrganizationViewMapper.class);
        SysOrganizationView record = new SysOrganizationView();
        XCopyUtils.copyObject(input, record);
        record.setIsDeleted(SysOrganizationCode.NORMAL);
        record.setEndTime(new Date());
        List<SysOrganizationView> viewList = mapper.selectOuTreeByOuName(record);
        List<QuerySysOrganizationViewOutputTree> list = XCopyUtils.copyNewList(viewList, QuerySysOrganizationViewOutputTree.class);
        List<QuerySysOrganizationViewOutputTree> trees = new ArrayList<>();
        String rootId = null;
        String ouId = null;
        if (input.getShowAll() != null && input.getShowAll()) {
            rootId = "00000000-0000-0000-0000-000000000000";
        } else {
            PpsUserSession session = context.getSession(PpsUserSession.class);
            ouId = session.getOuId();
        }

        for (QuerySysOrganizationViewOutputTree tree : list) {
            if (rootId != null) {
                if (tree.getParentOuId().equals(rootId)) {
                    trees.add(tree);
                }
            } else {
                if (tree.getId().equals(ouId)) {
                    trees.add(tree);
                }
            }
            //子集
            for (QuerySysOrganizationViewOutputTree it : list) {
                if (tree.getId().equals(it.getParentOuId())) {
                    if (tree.getChildren() == null) {
                        tree.setChildren(new ArrayList<>());
                    }
                    tree.getChildren().add(it);
                }
            }
            if (tree.getChildren() != null && tree.getChildren().size() > 1) {
                List<QuerySysOrganizationViewOutputTree> children = tree.getChildren();
                children.forEach(outputTree -> {
                    if (outputTree.getSort() == null) {
                        outputTree.setSort(999);
                    }
                });
                children = children.stream().sorted(Comparator.comparing(QuerySysOrganizationViewOutputTree::getSort)).collect(Collectors.toList());
                tree.setChildren(children);
            }
        }
        trees.forEach(outputTree -> {
            if (outputTree.getSort() == null) {
                outputTree.setSort(999);
            }
        });
        trees = trees.stream().sorted(Comparator.comparing(QuerySysOrganizationViewOutputTree::getSort)).collect(Collectors.toList());

        return XListResult.success(trees);
    }


    @XText("根据条件查询组织机构")
    @XApiPost
    public XListResult<GetSysOrganizationViewOutput> getOuInfoByCondition(XContext context, GetOuInfoByConditionInput input) {
        SysOrganizationViewMapper mapper = context.getBean(SysOrganizationViewMapper.class);
        SysOrganizationView record = new SysOrganizationView();
        XCopyUtils.copyObject(input, record);
        record.setEndTime(new Date());
        record.setIsDeleted(SysOrganizationCode.NORMAL);
        String rootId = null;
        String ouId = null;
        if (input.getShowAll() != null && input.getShowAll()) {
            rootId = "00000000-0000-0000-0000-000000000000";
        } else {
            PpsUserSession session = context.getSession(PpsUserSession.class);
            ouId = session.getOuId();
        }
        List<SysOrganizationView> viewList = new ArrayList<>();
        if (XStringUtils.isEmpty(input.getParentOuId())) {
            //如果是超级管理员查所以组织机构
            if (rootId != null) {
                record.setParentOuId(rootId);
                viewList = mapper.selectOuSonByParentOuId(record);
            } else {
                record.setId(ouId);
                viewList = mapper.selectOuByOuId(record);
            }
        } else {
            viewList = mapper.selectOuSonByParentOuId(record);
        }
        List<GetSysOrganizationViewOutput> outputs = XCopyUtils.copyNewList(viewList, GetSysOrganizationViewOutput.class);
        return XListResult.success(outputs);
    }

    @XText("根据父级id查询儿子级")
    @XApiPost
    public XListResult<GetSysOrganizationViewOutput> selectOuInfoByParentOuId(XContext context, GetSysOrganizationRelInput input) {
        SysOrganizationViewMapper mapper = context.getBean(SysOrganizationViewMapper.class);
        SysOrganizationView record = new SysOrganizationView();
        XCopyUtils.copyObject(input, record);
        record.setIsDeleted(SysOrganizationCode.NORMAL);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        Integer isSuperAdmin = session.getIsSuperAdmin();
        List<SysOrganizationView> viewList;
        if (XStringUtils.isEmpty(input.getParentOuId())) {
            //如果是超级管理员查所以组织机构
            if (UserConstant.ISSUPERADMIN_TRUE.equals(isSuperAdmin)) {
                record.setParentOuId("00000000-0000-0000-0000-000000000000");
                viewList = mapper.selectOuSonByParentOuId(record);
            } else {
                record.setId(session.getOuId());
                viewList = mapper.selectOuByOuId(record);
            }
        } else {
            viewList = mapper.selectOuSonByParentOuId(record);
        }
        List<GetSysOrganizationViewOutput> outputs = XCopyUtils.copyNewList(viewList, GetSysOrganizationViewOutput.class);
        return XListResult.success(outputs);
    }

    /**
     * 组织机构管理--动态查询列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQuerySysOrganizationOutput}>
     */
    @XText("组织机构管理--动态查询列表")
    @XApiGet
    public XListResult<DynamicQuerySysOrganizationOutput> querySysOrganizationList(XContext context, DynamicQuerySysOrganizationInput input) {
        SysOrganizationViewMapper mapper = context.getBean(SysOrganizationViewMapper.class);
        SysOrganizationView view;
        if (StringUtils.isAllBlank(input.getId(), input.getParentOuId(), input.getOuName())) {
            //如果条件都为空,则从登录信息获取条件
            view = new SysOrganizationView();
            PpsUserSession session = context.getSession(PpsUserSession.class);
            if (UserConstant.ISSUPERADMIN_TRUE.equals(session.getIsSuperAdmin())) {
                view.setParentOuId("00000000-0000-0000-0000-000000000000");
            } else {
                view.setId(session.getOuId());
            }
        } else {
            view = XCopyUtils.copyNewObject(input, SysOrganizationView.class);
        }
        List<SysOrganizationView> viewList = mapper.selectOrgListDynamicQuery(view);
        List<DynamicQuerySysOrganizationOutput> outputs = XCopyUtils.copyNewList(viewList, DynamicQuerySysOrganizationOutput.class);
        return XListResult.success(outputs);
    }

    @XText("根据父级id查询儿子级-特殊")
    @XApiPost
    public XListResult<GetSysOrganizationViewOutput> selectSpecailOuInfoByParentOuId(XContext context, GetSysOrganizationRelInput input) {
        SysOrganizationViewMapper mapper = context.getBean(SysOrganizationViewMapper.class);
        SysOrganizationView record = new SysOrganizationView();
        XCopyUtils.copyObject(input, record);
        record.setEndTime(new Date());
        record.setIsDeleted(SysOrganizationCode.NORMAL);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        Integer isSuperAdmin = session.getIsSuperAdmin();
        List<SysOrganizationView> viewList = new ArrayList<>();
        SysDictionaryViewMapper dictionaryViewMapper = context.getBean(SysDictionaryViewMapper.class);
        SysDictionaryView dictionaryView = new SysDictionaryView();
        dictionaryView.setAlias("sys_inform_org");
        Boolean isSpecial = false;
        List<SysDictionaryView> dictionaryViews = dictionaryViewMapper.selectListsByAlias(dictionaryView);
        if (!CollectionUtils.isEmpty(dictionaryViews)) {
            SysDictionaryView isExit = dictionaryViews.stream().filter(item -> XDateUtils.compare(item.getEndTime(), new Date()) >= 0 && item.getRemark().equals(session.getOuId())).findFirst().orElse(null);
            if (isExit != null)
                isSpecial = true;
        }
        if (XStringUtils.isEmpty(input.getParentOuId())) {
            //如果是超级管理员查所以组织机构
            if (UserConstant.ISSUPERADMIN_TRUE.equals(isSuperAdmin) || isSpecial) {
                record.setParentOuId("00000000-0000-0000-0000-000000000000");
                viewList = mapper.selectOuSonByParentOuId(record);
            } else {
                record.setId(session.getOuId());
                viewList = mapper.selectOuByOuId(record);
            }
        } else {
            viewList = mapper.selectOuSonByParentOuId(record);
        }
        List<GetSysOrganizationViewOutput> outputs = XCopyUtils.copyNewList(viewList, GetSysOrganizationViewOutput.class);
        return XListResult.success(outputs);
    }

    @XText("根据登录用户所属组织机构ID查询组织机构子级树")
    @XApiPost
    public XListResult<QuerySysOrganizationViewOutputTree> getSysOrganizationTreeByUserId(XContext context, GetSysOrganizationInput input) {
        SysOrganizationViewMapper mapper = context.getBean(SysOrganizationViewMapper.class);
        SysOrganizationView view = new SysOrganizationView();
        PpsUserSession session = context.getSession(PpsUserSession.class);
        view.setUserId(session.getId());
        view.setEndTime(new Date());
        List<SysOrganizationView> viewList = mapper.selectOuAllByUserId(view);
        List<QuerySysOrganizationViewOutputTree> list = XCopyUtils.copyNewList(viewList, QuerySysOrganizationViewOutputTree.class);
        List<QuerySysOrganizationViewOutputTree> trees = new ArrayList<>();
        for (QuerySysOrganizationViewOutputTree tree : list) {
            if (tree.getId().equals(session.getOuId())) {
                trees.add(tree);
            }
            //子集
            for (QuerySysOrganizationViewOutputTree it : list) {
                if (tree.getId().equals(it.getParentOuId())) {
                    if (tree.getChildren() == null) {
                        tree.setChildren(new ArrayList<>());
                    }
                    tree.getChildren().add(it);
                }
            }
            if (tree.getChildren() != null && tree.getChildren().size() > 1) {
                List<QuerySysOrganizationViewOutputTree> children = tree.getChildren();
                children.forEach(outputTree -> {
                    if (outputTree.getSort() == null) {
                        outputTree.setSort(999);
                    }
                });
                children = children.stream().sorted(Comparator.comparing(QuerySysOrganizationViewOutputTree::getSort)).collect(Collectors.toList());
                tree.setChildren(children);
            }
        }
        trees.forEach(outputTree -> {
            if (outputTree.getSort() == null) {
                outputTree.setSort(999);
            }
        });
        trees = trees.stream().sorted(Comparator.comparing(QuerySysOrganizationViewOutputTree::getSort)).collect(Collectors.toList());

        return XListResult.success(trees);
    }

    @XText("查询组织机构树")
    @XApiPost
    public XListResult<QuerySysOrganizationViewOutputTree> getSysOrganizationTree(XContext context, GetSysOrganizationInput input) {
        //先从缓存中获取
        List<SysOrganizationCache> sysOrganizationCaches = SysOrganizationCache.list(context, SysOrganizationCode.SYS_ORGANIZATION_CACHE);
        //缓存不存在从数据库获取
        if (CollectionUtils.isEmpty(sysOrganizationCaches)) {
            List<SysOrganizationCache> caches = selectSysOrganizationTree(context, input);
            List<QuerySysOrganizationViewOutputTree> resultTree = XCopyUtils.copyNewList(caches, QuerySysOrganizationViewOutputTree.class);

            return XListResult.success(resultTree);
        }
        List<QuerySysOrganizationViewOutputTree> resultTree = XCopyUtils.copyNewList(sysOrganizationCaches, QuerySysOrganizationViewOutputTree.class);

        return XListResult.success(resultTree);
    }

    private List<SysOrganizationCache> selectSysOrganizationTree(XContext context, GetSysOrganizationInput input) {
        SysOrganizationViewMapper mapper = context.getBean(SysOrganizationViewMapper.class);
        SysOrganizationView record = new SysOrganizationView();
        XCopyUtils.copyObject(input, record);
        record.setIsDeleted(SysOrganizationCode.NORMAL);
        record.setEndTime(new Date());
        List<SysOrganizationView> viewList = mapper.selectOrganizationEffective(record);
        List<QuerySysOrganizationViewOutputTree> list = XCopyUtils.copyNewList(viewList, QuerySysOrganizationViewOutputTree.class);
        List<QuerySysOrganizationViewOutputTree> trees = new ArrayList<>();

        for (QuerySysOrganizationViewOutputTree tree : list) {
            if (tree.getParentOuId().equals("00000000-0000-0000-0000-000000000000")) {
                trees.add(tree);
            }
            //子集
            for (QuerySysOrganizationViewOutputTree it : list) {
                if (tree.getId().equals(it.getParentOuId())) {
                    if (tree.getChildren() == null) {
                        tree.setChildren(new ArrayList<>());
                    }
                    tree.getChildren().add(it);
                }
            }
            if (tree.getChildren() != null && tree.getChildren().size() > 1) {
                List<QuerySysOrganizationViewOutputTree> children = tree.getChildren();
                children.forEach(outputTree -> {
                    if (outputTree.getSort() == null) {
                        outputTree.setSort(999);
                    }
                });
                children = children.stream().sorted(Comparator.comparing(QuerySysOrganizationViewOutputTree::getSort)).collect(Collectors.toList());
                tree.setChildren(children);
            }
        }
        trees.forEach(outputTree -> {
            if (outputTree.getSort() == null) {
                outputTree.setSort(999);
            }
        });
        trees = trees.stream().sorted(Comparator.comparing(QuerySysOrganizationViewOutputTree::getSort)).collect(Collectors.toList());
        List<SysOrganizationCache> caches = XCopyUtils.copyNewList(trees, SysOrganizationCache.class);
        //放入redis缓存中
        SysOrganizationCache.set(context, SysOrganizationCode.SYS_ORGANIZATION_CACHE, caches);
        return caches;
    }

    @XText("查询组织机构单节点树")
    @XApiPost
    public XListResult<QuerySysOrganizationViewOutputTree> selectSysOrganizationTreeByParentOuId(XContext context, GetSysOrganizationInput input) {
        SysOrganizationViewMapper mapper = context.getBean(SysOrganizationViewMapper.class);
        SysOrganizationView record = new SysOrganizationView();
        XCopyUtils.copyObject(input, record);
        record.setIsDeleted(SysOrganizationCode.NORMAL);
        List<SysOrganizationView> viewList = mapper.selectOrganizationByParentOuIdOrOuId(record);
        List<QuerySysOrganizationViewOutputTree> list = XCopyUtils.copyNewList(viewList, QuerySysOrganizationViewOutputTree.class);
        List<QuerySysOrganizationViewOutputTree> trees = new ArrayList<>();
        if ("00000000-0000-0000-0000-000000000000".equals(input.getId())) {
            trees.add(list.get(0));
        } else {
            for (QuerySysOrganizationViewOutputTree tree : list) {
                if (tree.getId().equals(input.getId())) {
                    trees.add(tree);
                }
                //子集
                for (QuerySysOrganizationViewOutputTree it : list) {
                    if (tree.getId().equals(it.getParentOuId())) {
                        if (tree.getChildren() == null) {
                            tree.setChildren(new ArrayList<>());
                        }
                        tree.getChildren().add(it);
                    }
                }
                if (tree.getChildren() != null && tree.getChildren().size() > 1) {
                    List<QuerySysOrganizationViewOutputTree> children = tree.getChildren();
                    children.forEach(outputTree -> {
                        if (outputTree.getSort() == null) {
                            outputTree.setSort(999);
                        }
                    });
                    children = children.stream().sorted(Comparator.comparing(QuerySysOrganizationViewOutputTree::getSort)).collect(Collectors.toList());
                    tree.setChildren(children);
                }
            }
        }

        trees.forEach(outputTree -> {
            if (outputTree.getSort() == null) {
                outputTree.setSort(999);
            }
        });
        trees = trees.stream().sorted(Comparator.comparing(QuerySysOrganizationViewOutputTree::getSort)).collect(Collectors.toList());

        return XListResult.success(trees);
    }

    @XText("查询组织机构列表")
    @XApiPost
    public XPageResult<QuerySysOrganizationViewOutput> querySysOrganizationViewPage(XContext context, QuerySysOrganizationViewInput input) {
        SysOrganizationViewMapper mapper = context.getBean(SysOrganizationViewMapper.class);
        SystemDictionaryService dictService = context.getBean(SystemDictionaryService.class);
        SysOrganizationView record = new SysOrganizationView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        record.setIsDeleted(SysOrganizationCode.NORMAL);
        record.setEndTime(new Date());
        List<SysOrganizationView> list = mapper.selectSysOrganizationList(record);
        if (!CollectionUtils.isEmpty(list)) {
            List<String> parentIds = list.stream().map(SysOrganizationView::getId).collect(Collectors.toList());
            List<SysOrganizationView> childList = mapper.selectCountByParentOuIds(parentIds, new Date());
            Map<String, List<SysOrganizationView>> groupChildList = childList.stream().collect(Collectors.groupingBy(SysOrganizationView::getParentOuId));
            list.stream().forEach(item -> {
                if (groupChildList.get(item.getId()) != null)
                    item.setIsHasChildren(1);
                else
                    item.setIsHasChildren(0);
            });
        }
        //单位级别
        CreateSysDictionaryInput unitLevelInput = new CreateSysDictionaryInput();
        unitLevelInput.setAlias("UNIT_LEVEL");
        XListResult<QuerySysDictionaryViewOutput> unitLevel = dictService.querySysDictionarysByAlias(context, unitLevelInput);

        //介质类型
        CreateSysDictionaryInput mediaTypesInput = new CreateSysDictionaryInput();
        mediaTypesInput.setAlias("MEDIUM_TYPE");
        XListResult<QuerySysDictionaryViewOutput> mediaTypes = dictService.querySysDictionarysByAlias(context, mediaTypesInput);


        //国家
        CreateSysDictionaryInput countryInput = new CreateSysDictionaryInput();
        countryInput.setAlias("COUNTRIES");
        XListResult<QuerySysDictionaryViewOutput> countrys = dictService.querySysDictionarysByAlias(context, countryInput);

        for (SysOrganizationView view : list) {

            //单位级别
            if (view.getOuLevel() != null) {
                if (unitLevel.getResult() != null) {
                    List<QuerySysDictionaryViewOutput> items = unitLevel.getResult();
                    for (QuerySysDictionaryViewOutput item : items) {
                        if (view.getOuLevel().equals(item.getDicKey())) {
                            view.setLevelDicName(item.getDicName());
                        }
                    }
                }
                if (view.getLevelDicName() == null) {
                    view.setLevelDicName(view.getOuLevel());
                }
            }

            //介质类型
            if (view.getMediaType() != null) {
                if (mediaTypes.getResult() != null) {
                    List<QuerySysDictionaryViewOutput> items = mediaTypes.getResult();
                    String[] split = view.getMediaType().split(",");
                    StringBuilder buffer = new StringBuilder();
                    for (String mediaType : split) {
                        for (QuerySysDictionaryViewOutput item : items) {
                            if (mediaType.equals(item.getDicKey())) {
                                buffer.append(",");
                                buffer.append(item.getDicName());
                            }
                        }
                    }
                    if (buffer.length() > 0) {
                        String mediaTypeName = buffer.substring(1);
                        view.setPropertyName(mediaTypeName);
                    }

                }

                if (view.getPropertyName() == null) {
                    view.setPropertyName(view.getMediaType());
                }
            }

            //国家
            if (view.getCountry() != null) {
                if (countrys.getResult() != null) {
                    List<QuerySysDictionaryViewOutput> items = countrys.getResult();
                    for (QuerySysDictionaryViewOutput item : items) {
                        if (view.getCountry().equals(item.getDicKey())) {
                            view.setCountryName(item.getDicName());
                        }
                    }
                }

                if (view.getCountryName() == null) {
                    view.setCountryName(view.getCountry());
                }
            }

        }
        PageInfo<SysOrganizationView> pageInfo = new PageInfo<>(list);
        List<QuerySysOrganizationViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySysOrganizationViewOutput.class);
        outputs = outputs.stream().sorted(Comparator.comparing(QuerySysOrganizationViewOutput::getSort)).collect(Collectors.toList());
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    @XText("查询组织机构列表")
    @XApiPost
    public XPageResult<QuerySysOrganizationViewOutput> querySysOrganizationViewIncluedOwnerPage(XContext context, QuerySysOrganizationViewInput input) {
        SysOrganizationViewMapper mapper = context.getBean(SysOrganizationViewMapper.class);
        SystemDictionaryService dictService = context.getBean(SystemDictionaryService.class);
        SysOrganizationView record = new SysOrganizationView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        record.setIsDeleted(SysOrganizationCode.NORMAL);
        List<SysOrganizationView> list = mapper.selectSysOrganizationList(record);
        if (CollectionUtils.isEmpty(list) && StringUtils.isNotBlank(record.getId())) {
            SysOrganizationView owner = mapper.selectOne(record);
            List<SysOrganizationView> parentList = mapper.selectOrganizationBySubOuIds(Arrays.asList(record.getId()));
            if (!CollectionUtils.isEmpty(parentList)) {
                SysOrganizationView parent = parentList.get(0);
                owner.setParentOuId(parent.getParentOuId());
                owner.setPartentName(parent.getPartentName());
            }
            list.add(owner);
        }
        //单位级别
        CreateSysDictionaryInput unitLevelInput = new CreateSysDictionaryInput();
        unitLevelInput.setAlias("UNIT_LEVEL");
        XListResult<QuerySysDictionaryViewOutput> unitLevel = dictService.querySysDictionarysByAlias(context, unitLevelInput);
        unitLevel.throwIfFail();
        if (!CollectionUtils.isEmpty(list)) {
            List<String> parentIds = list.stream()
                    .map(SysOrganizationView::getId)
                    .collect(Collectors.toList());
            List<SysOrganizationView> childList = mapper.selectCountByParentOuIds(parentIds, null);
            Map<String, List<SysOrganizationView>> groupChildList = childList.stream()
                    .collect(Collectors.groupingBy(SysOrganizationView::getParentOuId));
            for (SysOrganizationView view : list) {
                if (Objects.isNull(view.getSort())) {
                    view.setSort(0);
                }
                if (Objects.nonNull(groupChildList.get(view.getId()))) {
                    view.setIsHasChildren(1);
                } else {
                    view.setIsHasChildren(0);
                }
                //单位级别
                if (!CollectionUtils.isEmpty(unitLevel.getResult())) {
                    view.setLevelDicName(unitLevel.getResult().stream()
                            .filter(row -> StringUtils.equals(row.getDicKey(), view.getOuLevel()))
                            .map(QuerySysDictionaryViewOutput::getDicName)
                            .findAny()
                            .orElse(null));
                }
            }
        }
        PageInfo<SysOrganizationView> pageInfo = new PageInfo<>(list);
        List<QuerySysOrganizationViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySysOrganizationViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    @XText("组织机构排序")
    @XApiPost
    public XServiceResult updateSysOrganizationRelSort(XContext context, UpdateSysOrganizationRelInput input) throws XServiceException {

        return XTransactionHelper.begin(context, () -> {
            SysOrganizationRelMapper mapper = context.getBean(SysOrganizationRelMapper.class);
            Date date = new Date();
            QueryWrapper<SysOrganizationRelEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(SysOrganizationRelEnt::getOuId, input.getId());
            queryWrapper.lambda().le(true, SysOrganizationRelEnt::getEffectTime, date);
            queryWrapper.lambda().ge(true, SysOrganizationRelEnt::getEndTime, date);
            SysOrganizationRelEnt entity = mapper.selectOne(queryWrapper);
            if (entity == null) {
                return XServiceResult.error(context, XError.NotFound);
            }
            Integer entitySort = entity.getSort();


            QueryWrapper<SysOrganizationRelEnt> sortQueryWrapper = new QueryWrapper<>();
            sortQueryWrapper.lambda().eq(SysOrganizationRelEnt::getParentOuId, entity.getParentOuId());
            sortQueryWrapper.lambda().le(true, SysOrganizationRelEnt::getEffectTime, date);
            sortQueryWrapper.lambda().ge(true, SysOrganizationRelEnt::getEndTime, date);

            //判断是上移还是下移
            if (input.getSortFlag().equals("up")) {
                //sort-=1;
                sortQueryWrapper.lambda().le(SysOrganizationRelEnt::getSort, entitySort);
                sortQueryWrapper.lambda().orderByDesc(SysOrganizationRelEnt::getSort);
            } else {
                //sort+=1;
                sortQueryWrapper.lambda().ge(SysOrganizationRelEnt::getSort, entitySort);
                sortQueryWrapper.lambda().orderByAsc(SysOrganizationRelEnt::getSort);
            }


            List<SysOrganizationRelEnt> relEnts = mapper.selectList(sortQueryWrapper);

            PpsUserSession session = context.getSession(PpsUserSession.class);
            if (!CollectionUtils.isEmpty(relEnts) && relEnts.size() > 1) {
                SysOrganizationRelEnt relEnt = relEnts.get(1);
                Integer relEntSort = relEnt.getSort();
                if (input.getSortFlag().equals("up")) {
                    if (relEntSort != null && relEntSort.equals(entitySort)) {
                        relEntSort -= 1;
                    }
                } else {
                    if (relEntSort != null && relEntSort.equals(entitySort)) {
                        relEntSort += 1;
                    }
                }

                //上方或下方有数据进行交换序号
                relEnt.setSort(entitySort);
                relEnt.setModifyTime(date);
                relEnt.setModifyById(session.getId());
                relEnt.setModifyByName(session.getUserName());
                mapper.updateById(relEnt);
                entity.setSort(relEntSort);
            }

            entity.setModifyById(session.getId());
            entity.setModifyByName(session.getUserName());
            entity.setModifyTime(date);
            mapper.updateById(entity);
            //更新缓存中组织机构树
            selectSysOrganizationTree(context, new GetSysOrganizationInput());

            //返回执行成功的结果，事务提交
            return XServiceResult.OK;
        });
    }

    @XText("新增组织机构信息")
    @XApiPost
    public XServiceResult createSysOrganization(XContext context, CreateSysOrganizationInput input) throws XServiceException {

        return XTransactionHelper.begin(context, () -> {
            SysOrganizationMapper mapper = context.getBean(SysOrganizationMapper.class);
            SysOrganizationRelMapper relMapper = context.getBean(SysOrganizationRelMapper.class);
            SysOrganizationEnt entity = new SysOrganizationEnt();
            XCopyUtils.copyObject(input, entity);
            Date date = new Date();
            String uuid = UUID.randomUUID().toString();
            entity.setId(uuid);
            entity.setIsDeleted(SysOrganizationCode.NORMAL);
            entity.setCreateTime(date);
            PpsUserSession session = context.getSession(PpsUserSession.class);
            if (session != null) {
                entity.setCreateById(session.getId());
                entity.setCreateByName(session.getUserName());
            }
            mapper.insert(entity);

            //先修改父级为有子集状态
            if (input.getParentOuId() != null) {
                QueryWrapper<SysOrganizationRelEnt> queryWrapper = new QueryWrapper<>();
                queryWrapper.lambda().le(true, SysOrganizationRelEnt::getEffectTime, new Date());
                queryWrapper.lambda().ge(true, SysOrganizationRelEnt::getEndTime, new Date());
                queryWrapper.lambda().eq(SysOrganizationRelEnt::getOuId, input.getParentOuId());
                SysOrganizationRelEnt parent = relMapper.selectOne(queryWrapper);
                if (parent != null) {
                    parent.setIsHasChildren(Constant.ISHASHCHILDREN_TRUE);
                    parent.setModifyTime(new Date());
                    if (session != null) {
                        parent.setModifyById(session.getId());
                        parent.setModifyByName(session.getUserName());
                    }
                    relMapper.updateById(parent);
                }
            }

            //新增组织机构关系表
            insertSysOrganizationRel(context, date, uuid, input.getParentOuId(), input.getSort(), session);

            //新增组织机构属性表
            insertSysOrganizationProperty(context, date, input.getId(), "GAS", session);
            //更新缓存中组织机构树
            selectSysOrganizationTree(context, new GetSysOrganizationInput());

            //返回执行成功的结果，事务提交
            return XServiceResult.OK;
        });
    }

    @XText("获取当前父级下最大的序号")
    @XApiPost
    public XSingleResult<QuerySysOrganizationRelInput> selectMaxSort(XContext context, GetSysOrganizationRelInput input) {
        SysOrganizationRelViewMapper relViewMapper = context.getBean(SysOrganizationRelViewMapper.class);
        SysOrganizationRelView relView = relViewMapper.selectMaxSort(input.getParentOuId());
        QuerySysOrganizationRelInput output = new QuerySysOrganizationRelInput();
        if (relView != null) {
            Integer sort1 = relView.getSort();
            output.setSort(++sort1);
        } else {
            output.setSort(1);
        }
        return XSingleResult.success(output);
    }

    @XText("修改组织机构信息")
    @XApiPost
    public XServiceResult updateSysOrganization(XContext context, UpdateSysOrganizationInput input) throws XServiceException {

        return XTransactionHelper.begin(context, () -> {
            SysOrganizationMapper mapper = context.getBean(SysOrganizationMapper.class);
            SysOrganizationRelMapper relMapper = context.getBean(SysOrganizationRelMapper.class);
            SysOrganizationPropertyMapper propertyMapper = context.getBean(SysOrganizationPropertyMapper.class);
            QueryWrapper<SysOrganizationEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(SysOrganizationEnt::getId, input.getId());
            SysOrganizationEnt entity = mapper.selectOne(queryWrapper);
            if (entity == null) {
                return XServiceResult.error(context, XError.NotFound);

            }
            XCopyUtils.copyObject(input, entity);
            Date date = new Date();
            entity.setModifyTime(date);
            PpsUserSession session = context.getSession(PpsUserSession.class);
            if (session != null) {
                entity.setModifyById(session.getId());
                entity.setModifyByName(session.getUserName());
            }
            mapper.updateById(entity);


            //修改组织机构关系表
            QueryWrapper<SysOrganizationRelEnt> relQueryWrapper = new QueryWrapper<>();
            relQueryWrapper.lambda().eq(SysOrganizationRelEnt::getOuId, input.getId());
            relQueryWrapper.lambda().le(SysOrganizationRelEnt::getEffectTime, new Date());
            relQueryWrapper.lambda().ge(SysOrganizationRelEnt::getEndTime, new Date());
            SysOrganizationRelEnt relEntity = relMapper.selectOne(relQueryWrapper);
            if (relEntity == null) {
                throw new XServiceException("数据有误！");
            }
            try {
                //先修改原父级为无子集状态
                if (input.getParentOuId() != null && !input.getParentOuId().equals(relEntity.getParentOuId())) {
                    QueryWrapper<SysOrganizationRelEnt> originalParentWrapper = new QueryWrapper<>();
                    originalParentWrapper.lambda().le(SysOrganizationRelEnt::getEffectTime, new Date());
                    originalParentWrapper.lambda().ge(SysOrganizationRelEnt::getEndTime, new Date());
                    originalParentWrapper.lambda().eq(SysOrganizationRelEnt::getOuId, relEntity.getParentOuId());
                    SysOrganizationRelEnt originalParent = relMapper.selectOne(originalParentWrapper);
                    if (originalParent != null) {
                        originalParent.setIsHasChildren(Constant.ISHASHCHILDREN_FALSE);
                        originalParent.setModifyTime(new Date());
                        if (session != null) {
                            originalParent.setModifyById(session.getId());
                            originalParent.setModifyByName(session.getUserName());
                        }
                        relMapper.updateById(originalParent);
                    }

                    //修改新父级为有子集状态
                    QueryWrapper<SysOrganizationRelEnt> nowParentWrapper = new QueryWrapper<>();
                    nowParentWrapper.lambda().eq(SysOrganizationRelEnt::getOuId, input.getParentOuId());
                    nowParentWrapper.lambda().le(SysOrganizationRelEnt::getEffectTime, new Date());
                    nowParentWrapper.lambda().ge(SysOrganizationRelEnt::getEndTime, new Date());
                    SysOrganizationRelEnt nowParent = relMapper.selectOne(nowParentWrapper);
                    if (nowParent != null) {
                        nowParent.setIsHasChildren(Constant.ISHASHCHILDREN_TRUE);
                        nowParent.setModifyTime(new Date());
                        if (session != null) {
                            nowParent.setModifyById(session.getId());
                            nowParent.setModifyByName(session.getUserName());
                        }
                        relMapper.updateById(nowParent);
                    }
                    relEntity.setEndTime(date);
                    //新增组织机构关系表
                    insertSysOrganizationRel(context, date, input.getId(), input.getParentOuId(), input.getSort(), session);
                }
                relEntity.setSort(input.getSort());
                relEntity.setModifyTime(date);
                if (session != null) {
                    relEntity.setModifyById(session.getId());
                    relEntity.setModifyByName(session.getUserName());
                }

                relMapper.updateById(relEntity);

                //修改组织机构属性表
                QueryWrapper<SysOrganizationPropertyEnt> propertyQueryWrapper = new QueryWrapper<>();
                propertyQueryWrapper.lambda().eq(SysOrganizationPropertyEnt::getOuId, input.getId())
                        .le(SysOrganizationPropertyEnt::getEffectTime, new Date())
                        .ge(SysOrganizationPropertyEnt::getEndTime, new Date());
                List<SysOrganizationPropertyEnt> propertyEntitys = propertyMapper.selectList(propertyQueryWrapper);

                //组织机构属性数据全部改为失效
                propertyEntitys.forEach(sysOrganizationPropertyEnt -> {
                    sysOrganizationPropertyEnt.setEndTime(date);
                    sysOrganizationPropertyEnt.setModifyTime(date);
                    if (session != null) {
                        sysOrganizationPropertyEnt.setModifyById(session.getId());
                        sysOrganizationPropertyEnt.setModifyByName(session.getUserName());
                    }
                    propertyMapper.updateById(sysOrganizationPropertyEnt);
                });
                insertSysOrganizationProperty(context, date, input.getId(), "GAS", session);
//                if (XStringUtils.isNotEmpty(input.getMediaType())){
//                    String[] split = input.getMediaType().split(",");
//                    StringBuilder builder = new StringBuilder();
//                    for (String mediaType : split) {
//                        //用来判断库中是否存在选中的介质类型
//                        int record=0;
//                        for (SysOrganizationPropertyEnt propertyEntity : propertyEntitys) {
//                            //如果页面和库中的介质类型相等的话改为有效
//                            if (mediaType.equals(propertyEntity.getPropertyValue())){
//                                record+=1;
//                                //介质类型
//                                //propertyEntity.setPropertyValue(input.getMediumType());
//                                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
//                                String datetime="9999-12-31";
//                                try {
//                                    propertyEntity.setEndTime(sdf.parse(datetime));
//                                } catch (ParseException e) {
//                                    e.printStackTrace();
//                                }
//                                propertyMapper.updateById(propertyEntity);
//                            }
//                        }
//                        if (record ==0){
//                            builder.append(",");
//                            builder.append(mediaType);
//
//                        }
//                    }
//                    if (builder.length()>0){
//                        //新增组织机构属性表
//                        String mediaType = builder.substring(1);
//
//                    }
//                }

            } catch (Exception e) {
                throw new XServiceException(e);
            }

            //更新缓存中组织机构树
            selectSysOrganizationTree(context, new GetSysOrganizationInput());

            //返回执行成功的结果，事务提交
            return XServiceResult.OK;
        });
    }

    @XText("删除组织机构")
    @XApiPost
    public XServiceResult deleteSysOrganization(XContext context, DeleteSysOrganizationInput input) throws XServiceException {

        return XTransactionHelper.begin(context, () -> {
            SysOrganizationMapper mapper = context.getBean(SysOrganizationMapper.class);
            SysUserOrganizationRelMapper userOuRelMapper = context.getBean(SysUserOrganizationRelMapper.class);
            QueryWrapper<SysOrganizationEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(SysOrganizationEnt::getId, input.getId());
            SysOrganizationEnt entity = mapper.selectOne(queryWrapper);
            if (entity == null) {
                return XServiceResult.error(context, XError.NotFound);
            }

            QueryWrapper<SysUserOrganizationRelEnt> wrapper = new QueryWrapper<>();
            wrapper.lambda().eq(SysUserOrganizationRelEnt::getOuId, entity.getId())
                    .le(true, SysUserOrganizationRelEnt::getEffectTime, new Date())
                    .ge(true, SysUserOrganizationRelEnt::getEndTime, new Date());
            List<SysUserOrganizationRelEnt> relEnts = userOuRelMapper.selectList(wrapper);
            if (!CollectionUtils.isEmpty(relEnts)) {
                return XServiceResult.error(200, "当前组织机构下已绑定人员禁止删除！");
            }
            entity.setModifyTime(new Date());
            PpsUserSession session = context.getSession(PpsUserSession.class);
            if (session != null) {
                entity.setModifyById(session.getId());
                entity.setModifyByName(session.getUserName());
            }
            entity.setIsDeleted(SysOrganizationCode.DELETE);
            mapper.updateById(entity);

            //组织机构关系表修改为失效状态
            SysOrganizationRelMapper relmapper = context.getBean(SysOrganizationRelMapper.class);
            QueryWrapper<SysOrganizationRelEnt> relWrapper = new QueryWrapper<>();
            relWrapper.lambda().eq(SysOrganizationRelEnt::getOuId, input.getId());
            relWrapper.lambda().le(true, SysOrganizationRelEnt::getEffectTime, new Date());
            relWrapper.lambda().ge(true, SysOrganizationRelEnt::getEndTime, new Date());
            SysOrganizationRelEnt relEnt = relmapper.selectOne(relWrapper);
            try {
                relEnt.setEndTime(new Date());
                relEnt.setModifyTime(new Date());
                if (session != null) {
                    relEnt.setModifyById(session.getId());
                    relEnt.setModifyByName(session.getUserName());
                }
                relmapper.updateById(relEnt);
            } catch (Exception e) {
                throw new XServiceException(e);
            }

            //组织机构属性表修改为失效状态
            SysOrganizationPropertyMapper propertyMapper = context.getBean(SysOrganizationPropertyMapper.class);
            QueryWrapper<SysOrganizationPropertyEnt> propertyWrapper = new QueryWrapper<>();
            propertyWrapper.lambda().eq(SysOrganizationPropertyEnt::getOuId, input.getId());
            propertyWrapper.lambda().le(true, SysOrganizationPropertyEnt::getEffectTime, new Date());
            propertyWrapper.lambda().ge(true, SysOrganizationPropertyEnt::getEndTime, new Date());
            List<SysOrganizationPropertyEnt> propertyEnts = propertyMapper.selectList(propertyWrapper);
            propertyEnts.forEach(propertyEnt -> {
                propertyEnt.setEndTime(new Date());
                propertyEnt.setModifyTime(new Date());
                if (session != null) {
                    propertyEnt.setModifyById(session.getId());
                    propertyEnt.setModifyByName(session.getUserName());
                }
            });

            propertyEnts.forEach(propertyMapper::updateById);
            //更新缓存中组织机构树
            selectSysOrganizationTree(context, new GetSysOrganizationInput());

            //返回执行成功的结果，事务提交
            return XServiceResult.OK;
        });
    }

    //新增组织机构属性表
    public void insertSysOrganizationProperty(XContext context, Date date, String uuid, String mediaTypes, PpsUserSession session) {
        //新增组织机构属性表
        SysOrganizationPropertyMapper propertyMapper = context.getBean(SysOrganizationPropertyMapper.class);
        String[] split = mediaTypes.split(",");
        List<SysOrganizationPropertyEnt> list = new ArrayList<>();
        for (String mediumType : split) {
            SysOrganizationPropertyEnt propertyEnt = new SysOrganizationPropertyEnt();
            propertyEnt.setId(UUID.randomUUID().toString());
            propertyEnt.setOuId(uuid);
            propertyEnt.setPropertyValue(mediumType);
            propertyEnt.setEffectTime(date);
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            String datetime = "9999-12-31";
            try {
                propertyEnt.setEndTime(sdf.parse(datetime));
            } catch (ParseException e) {
                e.printStackTrace();
            }
            propertyEnt.setCreateTime(date);
            if (session != null) {
                propertyEnt.setCreateById(session.getId());
                propertyEnt.setCreateByName(session.getUserName());
            }
            list.add(propertyEnt);

        }
        if (!CollectionUtils.isEmpty(list)) {
            list.forEach(propertyMapper::insert);
        }

    }

    //新增组织机构关系表
    public void insertSysOrganizationRel(XContext context, Date date, String ouId, String parentOuId, Integer sort, PpsUserSession session) {
        SysOrganizationRelMapper relMapper = context.getBean(SysOrganizationRelMapper.class);
        SysOrganizationRelViewMapper relViewMapper = context.getBean(SysOrganizationRelViewMapper.class);


        //新增组织机构关系表
        SysOrganizationRelEnt relEnt = new SysOrganizationRelEnt();
        relEnt.setId(UUID.randomUUID().toString());
        relEnt.setOuId(ouId);
        relEnt.setParentOuId(parentOuId);
        QueryWrapper<SysOrganizationRelEnt> relWrapper = new QueryWrapper<>();
        relWrapper.lambda().eq(SysOrganizationRelEnt::getOuId, parentOuId)
                .le(SysOrganizationRelEnt::getEffectTime, date)
                .ge(true, SysOrganizationRelEnt::getEndTime, date);
        SysOrganizationRelEnt relEnt1 = relMapper.selectOne(relWrapper);
        if (relEnt1 == null) {
            relEnt.setOuIdPath(ouId);
        } else {
            String ouIdPath = relEnt1.getOuIdPath();
            ouIdPath = ouIdPath + "/" + ouId;
            relEnt.setOuIdPath(ouIdPath);
            relEnt.setLevels(relEnt1.getLevels() == null ? 1 : relEnt1.getLevels() + 1);
        }

        //如果用户不填写序号需要取表中最大的序号然后+1作为当前数据的序号
        if (sort == null) {
            SysOrganizationRelView relView = relViewMapper.selectMaxSort(parentOuId);
            if (relView != null) {
                Integer sort1 = relView.getSort();
                relEnt.setSort(++sort1);
            } else {
                relEnt.setSort(1);
            }
        } else {
            relEnt.setSort(sort);
        }
        relEnt.setEffectTime(date);
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        String datetime = "9999-12-31";
        try {
            relEnt.setEndTime(sdf.parse(datetime));
        } catch (ParseException e) {
            e.printStackTrace();
        }
        relEnt.setCreateTime(date);
        if (session != null) {
            relEnt.setCreateById(session.getId());
            relEnt.setCreateByName(session.getUserName());
        }
        relMapper.insert(relEnt);
        //判断当前组织机构是否存在下级，如果存在下级，更新下级组织机构的路径
        updateSysOrganizationChildRel(context, date, relEnt, session);
    }

    public void updateSysOrganizationChildRel(XContext context, Date date, SysOrganizationRelEnt relEnt, PpsUserSession session) {
        SysOrganizationRelMapper relMapper = context.getBean(SysOrganizationRelMapper.class);
        QueryWrapper<SysOrganizationRelEnt> relWrapper = new QueryWrapper<>();
        relWrapper.lambda().like(SysOrganizationRelEnt::getOuIdPath, relEnt.getOuId() + "/");
        List<SysOrganizationRelEnt> childList = relMapper.selectList(relWrapper);
        if (!CollectionUtils.isEmpty(childList)) {
            childList.stream().forEach(item -> {
                int index = item.getOuIdPath().indexOf(relEnt.getOuId());
                item.setOuIdPath(relEnt.getOuIdPath() + item.getOuIdPath().substring(index + relEnt.getOuId().length()));
                item.setModifyById(session.getId());
                item.setModifyByName(session.getUserName());
                item.setLevels(relEnt.getLevels() + 1);
                item.setModifyTime(date);
                relMapper.updateById(item);
            });
        }
    }

    @XText("获取用户组织机构及授权的组织机构")
    @XApiPost
    public XListResult<GetOuListTreeOutput> getOuListWithSelfAndAuth(XContext context, GetOuListWithSelfAndAuthInput input) {
        PpsUserSession session = context.getSession(PpsUserSession.class);
        String ouId = session.getOuId();
        if (StringUtils.isBlank(ouId)) return XListResult.success();
        //根据组织机构id获取所有的子级组织机构
        List<GetOuListTreeOutput> ouList = this.getAllOuListByOuId(context, ouId);
        if (CollectionUtils.isEmpty(ouList)) return XListResult.success();

        if (StringUtils.isBlank(input.getMenuId())) {
            return XListResult.success(this.createOuTree(ouList));
        }
        //获取用户授权的组织机构
        Map<String, List<SysOrganization>> pageOrgMap = session.getPageOrgMap();
        if (pageOrgMap != null) {
            //获取当前用户在指定菜单下被授权的组织机构
            List<SysOrganization> authList = pageOrgMap.get(input.getMenuId());
            if (!CollectionUtils.isEmpty(authList)) {
                List<String> authOutIds = new ArrayList<>();
                for (SysOrganization auth : authList) {
                    boolean isAdd = true;
                    for (GetOuListTreeOutput ou : ouList) {
                        if (ou.getOuId().equals(auth.getOuId())) {
                            isAdd = false;
                            break;
                        }
                    }
                    if (isAdd) {
                        authOutIds.add(auth.getOuId());
                    }
                }
                if (authOutIds.size() > 0) {
                    List<GetOuListTreeOutput> resultAuthList = this.getOuListByOuIds(context, authOutIds);
                    ouList.addAll(resultAuthList);
                }
            }
        }
        return XListResult.success(this.createOuTree(ouList));
    }

    /**
     * 根据组织机构id获取所有的子级组织机构
     */
    private List<GetOuListTreeOutput> getAllOuListByOuId(XContext context, String ouId) {
        List<GetOuListTreeOutput> outputs = new ArrayList<>();
        SysOrganizationViewMapper mapper = context.getBean(SysOrganizationViewMapper.class);
        SysOrganizationView view = new SysOrganizationView();
        view.setIsDeleted(1);
        view.setId(ouId);
        List<SysOrganizationView> list = mapper.selectAllListByOuId(view);
        if (list.size() > 0) {
            for (SysOrganizationView item : list) {
                GetOuListTreeOutput out = new GetOuListTreeOutput();
                out.setId(item.getId());
                out.setOuId(item.getId());
                out.setOuName(item.getOuName());
                out.setShortName(item.getShortName());
                out.setParentOuId(item.getParentOuId());
                outputs.add(out);
            }
        }
        return outputs;
    }

    /**
     * 根据组织机构id的集合，获取组织机构信息
     */
    private List<GetOuListTreeOutput> getOuListByOuIds(XContext context, List<String> ouIds) {
        List<GetOuListTreeOutput> outputs = new ArrayList<>();
        if (CollectionUtils.isEmpty(ouIds)) return outputs;
        SysOrganizationViewMapper mapper = context.getBean(SysOrganizationViewMapper.class);
        List<SysOrganizationView> list = mapper.selectListByOuIds(ouIds);
        if (list.size() > 0) {
            for (SysOrganizationView item : list) {
                GetOuListTreeOutput out = new GetOuListTreeOutput();
                out.setId(item.getId());
                out.setOuId(item.getId());
                out.setOuName(item.getOuName());
                out.setShortName(item.getShortName());
                out.setParentOuId(item.getParentOuId());
                outputs.add(out);
            }
        }
        return outputs;
    }

    public List<GetOuListTreeOutput> createOuTree(List<GetOuListTreeOutput> list) {
        List<GetOuListTreeOutput> treeList = new ArrayList<>();
        if (list.size() > 0) {
            for (GetOuListTreeOutput tree : list) {
                boolean isHaveParent = false;
                for (GetOuListTreeOutput item : list) {
                    //子集
                    if (tree.getOuId().equals(item.getParentOuId())) {
                        if (tree.getChildren() == null) {
                            tree.setChildren(new ArrayList<>());
                        }
                        tree.getChildren().add(item);
                    }
                    //父集
                    if (tree.getParentOuId().equals(item.getOuId())) {
                        isHaveParent = true;
                    }
                }
                if (!isHaveParent) {
                    treeList.add(tree);
                }
            }
        }
        return treeList;
    }


}
