package pps.core.system.service;

import java.math.BigDecimal;
import java.util.*;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.springframework.util.CollectionUtils;
import pps.core.system.mapper.*;
import pps.core.system.entity.*;
import pps.core.system.service.data.sys_role_menu_auth.*;
import pps.core.common.session.PpsUserSession;
import xstartup.annotation.XService;
import xstartup.base.exception.XServiceException;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.error.XError;
import xstartup.base.data.CustomQueryInput;
import xstartup.feature.mybatis.helper.XMapperHelper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import xstartup.helper.XTransactionHelper;

import java.util.stream.Collectors;

@XText("角色菜单管理")
@XService
public class SysRoleMenuAuthService {

    @XText("角色绑定多菜单")
    @XApiAnonymous
    @XApiPost
    public XServiceResult createSysRoleMenuAuth(XContext context, CreateSysRoleMenuAuthInput input) throws XServiceException {

        return XTransactionHelper.begin(context, () -> {
            //删除原关系
            SysRoleMenuAuthMapper mapper = context.getBean(SysRoleMenuAuthMapper.class);
            QueryWrapper<SysRoleMenuAuthEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(SysRoleMenuAuthEnt::getRoleId, input.getRoleId());
            PageHelper.startPage(1, 1000);
            List<SysRoleMenuAuthEnt> sysUserRoleRelEnts = mapper.selectList(queryWrapper);
            if (!CollectionUtils.isEmpty(sysUserRoleRelEnts)) {
                List<String> collect = sysUserRoleRelEnts.stream().map(SysRoleMenuAuthEnt::getId).collect(Collectors.toList());
                mapper.deleteBatchIds(collect);
            }

            //创建新关系
            SysRoleMenuAuthViewMapper viewMapper = context.getBean(SysRoleMenuAuthViewMapper.class);
            List<String> menuIdList = input.getMenuIds();
            Date now = new Date();
            List<SysRoleMenuAuthView> viewList = new ArrayList<>();
            for (String menuId : menuIdList) {
                SysRoleMenuAuthView entity = new SysRoleMenuAuthView();
                entity.setId(UUID.randomUUID().toString());
                entity.setMenuId(menuId);
                entity.setRoleId(input.getRoleId());
                PpsUserSession session = context.getSession(PpsUserSession.class);
                if (session != null) {
                    entity.setCreateById(session.getId());
                    entity.setCreateByName(session.getUserName());
                }
                entity.setCreateTime(now);
                viewList.add(entity);

            }
            if(!CollectionUtils.isEmpty(viewList)){
                viewMapper.batchInsert(viewList);
            }

            //返回执行成功的结果，事务提交
            return XServiceResult.OK;
        });
    }


    @XText("查询角色对应多个菜单")
    @XApiAnonymous
    @XApiPost
    public XListResult<QuerySysRoleMenuAuthViewOutput> querySysRoleMenuAuthByRoleId(XContext context, GetSysRoleMenuAuthViewInput input){
        SysRoleMenuAuthViewMapper mapper = context.getBean(SysRoleMenuAuthViewMapper.class);
        SysRoleMenuAuthView record = new SysRoleMenuAuthView();
        XCopyUtils.copyObject(input, record);
        List<SysRoleMenuAuthView> list = mapper.selectListByRoleId(record);
        PageInfo<SysRoleMenuAuthView> pageInfo = new PageInfo<>(list);
        List<QuerySysRoleMenuAuthViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySysRoleMenuAuthViewOutput.class);
        return XListResult.success(outputs);
    }

    @XApiAnonymous
    @XApiPost
    public XServiceResult updateSysRoleMenuAuth(XContext context, UpdateSysRoleMenuAuthInput input){
        SysRoleMenuAuthMapper mapper = context.getBean(SysRoleMenuAuthMapper.class);
        QueryWrapper<SysRoleMenuAuthEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysRoleMenuAuthEnt::getId, input.getId());
        SysRoleMenuAuthEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiPost
    public XServiceResult deleteSysRoleMenuAuth(XContext context, DeleteSysRoleMenuAuthInput input){
        SysRoleMenuAuthMapper mapper = context.getBean(SysRoleMenuAuthMapper.class);
        QueryWrapper<SysRoleMenuAuthEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysRoleMenuAuthEnt::getId, input.getId());
        SysRoleMenuAuthEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        mapper.deleteById(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetSysRoleMenuAuthOutput> getSysRoleMenuAuth(XContext context, GetSysRoleMenuAuthInput input){
        SysRoleMenuAuthMapper mapper = context.getBean(SysRoleMenuAuthMapper.class);
        QueryWrapper<SysRoleMenuAuthEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysRoleMenuAuthEnt::getId, input.getId());
        SysRoleMenuAuthEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetSysRoleMenuAuthOutput output = new GetSysRoleMenuAuthOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QuerySysRoleMenuAuthOutput> querySysRoleMenuAuth(XContext context, QuerySysRoleMenuAuthInput input){
        SysRoleMenuAuthMapper mapper = context.getBean(SysRoleMenuAuthMapper.class);
        QueryWrapper<SysRoleMenuAuthEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().like(SysRoleMenuAuthEnt::getId, input.getId())
            .like(SysRoleMenuAuthEnt::getRoleId, input.getRoleId())
            .like(SysRoleMenuAuthEnt::getMenuId, input.getMenuId())
            .like(SysRoleMenuAuthEnt::getCreateById, input.getCreateById())
            .like(SysRoleMenuAuthEnt::getCreateByName, input.getCreateByName())
            .eq(SysRoleMenuAuthEnt::getCreateTime, input.getCreateTime())
            .like(SysRoleMenuAuthEnt::getModifyById, input.getModifyById())
            .like(SysRoleMenuAuthEnt::getModifyByName, input.getModifyByName())
            .eq(SysRoleMenuAuthEnt::getModifyTime, input.getModifyTime());
        return XMapperHelper.query(mapper, input, queryWrapper, QuerySysRoleMenuAuthOutput.class);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QuerySysRoleMenuAuthOutput> dynamicQuerySysRoleMenuAuth(XContext context, CustomQueryInput input){
        SysRoleMenuAuthMapper mapper = context.getBean(SysRoleMenuAuthMapper.class);
        return XMapperHelper.query(mapper, input, SysRoleMenuAuthEnt.class, QuerySysRoleMenuAuthOutput.class);
    }

    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetSysRoleMenuAuthViewOutput> getSysRoleMenuAuthView(XContext context, GetSysRoleMenuAuthViewInput input){
        SysRoleMenuAuthViewMapper mapper = context.getBean(SysRoleMenuAuthViewMapper.class);
        SysRoleMenuAuthView record = new SysRoleMenuAuthView();
        XCopyUtils.copyObject(input, record);
        SysRoleMenuAuthView view = mapper.selectOne(record);
        if(view == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetSysRoleMenuAuthViewOutput output = new GetSysRoleMenuAuthViewOutput();
        XCopyUtils.copyObject(view, output);
        return XSingleResult.success(output);
    }



}
