package pps.core.system.service;

import java.util.*;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.CollectionUtils;
import pps.core.common.session.PpsUserSession;
import pps.core.system.mapper.*;
import pps.core.system.entity.*;
import pps.core.system.service.data.sys_auth_ou_scope.CreateSysAuthOuScopeInput;
import pps.core.system.service.data.sys_user_group.*;
import pps.core.system.service.data.sys_user_group_role_rel.CreateSysUserGroupRoleRelInput;
import pps.core.system.service.data.sys_user_with_user_group_rel.CreateSysUserWithUserGroupRelInput;
import xstartup.annotation.XService;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.data.XPageResult;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.error.XError;
import xstartup.base.data.CustomQueryInput;
import xstartup.feature.mybatis.helper.XMapperHelper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import xstartup.helper.XTransactionHelper;

import java.util.stream.Collectors;

@XText("用户组管理")
@XService
public class SysUserGroupService {

    @XText("创建用户组")
    @XApiPost
    public XServiceResult createSysUserGroup(XContext context, CreateSysUserGroupInput input){
        return XTransactionHelper.begin(context,()-> {
            SysUserGroupMapper mapper = context.getBean(SysUserGroupMapper.class);
            PpsUserSession session = context.getSession(PpsUserSession.class);
            SysUserGroupEnt entity = new SysUserGroupEnt();
            XCopyUtils.copyObject(input, entity);
            entity.setId(UUID.randomUUID().toString());
            //entity.setCreateById(context.getSession(PpsUserSession.class).getId());
            entity.setCreateByName(context.getUser().getName());
            entity.setCreateTime(new Date());
            mapper.insert(entity);
            return XServiceResult.OK;
        });
    }


    @XText("创建用户组和角色关系")
    @XApiPost
    public XServiceResult createSysUserGroupRoleRel(XContext context, CreateSysUserGroupRoleRelInput input){
        return XTransactionHelper.begin(context,()-> {
            SysUserGroupRoleRelMapper userGroupWithRole = context.getBean(SysUserGroupRoleRelMapper.class);
            SysUserGroupRoleRelViewMapper viewUserGroupWithRole = context.getBean(SysUserGroupRoleRelViewMapper.class);
            //删除原关系
            QueryWrapper<SysUserGroupRoleRelEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(SysUserGroupRoleRelEnt::getUserGroupId, input.getUserGroupId());
            PageHelper.startPage(1, 1000);
            List<SysUserGroupRoleRelEnt> sysUserGroupRoleRelEnts = userGroupWithRole.selectList(queryWrapper);
            if (!CollectionUtils.isEmpty(sysUserGroupRoleRelEnts)) {
                List<String> collect = sysUserGroupRoleRelEnts.stream().map(SysUserGroupRoleRelEnt::getId).collect(Collectors.toList());
                userGroupWithRole.deleteBatchIds(collect);
            }
            //创建新关系
            List<String> items = input.getItems();
            Date now = new Date();
            List<SysUserGroupRoleRelView> viewList = new ArrayList<>();
            for (String role : items) {
                SysUserGroupRoleRelView entity = new SysUserGroupRoleRelView();
                entity.setRoleId(role);
                entity.setUserGroupId(input.getUserGroupId());
                entity.setId(UUID.randomUUID().toString());
                //entity.setCreateById(context.getUser().getId());
                entity.setCreateByName(context.getUser().getName());
                entity.setCreateTime(now);
                viewList.add(entity);
            }
            if(ObjectUtils.isNotEmpty(viewList)){
                viewUserGroupWithRole.batchInsert(viewList);
            }
            return XServiceResult.OK;
        });
    }
    //SysAuthOuScope
    @XText("创建用户组和菜单、组织机构关系表")
    @XApiPost
    public XServiceResult createSysAuthOuScope(XContext context, CreateSysAuthOuScopeInput input){
        return XTransactionHelper.begin(context,()-> {
            SysAuthOuScopeMapper userGroupWithRole = context.getBean(SysAuthOuScopeMapper.class);
            SysAuthOuScopeViewMapper viewUserGroupWithRole = context.getBean(SysAuthOuScopeViewMapper.class);
            //删除原关系
            QueryWrapper<SysAuthOuScopeEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(SysAuthOuScopeEnt::getUserGroupId, input.getUserGroupId());
            PageHelper.startPage(1, 1000);
            List<SysAuthOuScopeEnt> sysAuthOuScopeEnts = userGroupWithRole.selectList(queryWrapper);
            if (!CollectionUtils.isEmpty(sysAuthOuScopeEnts)) {
                List<String> collect = sysAuthOuScopeEnts.stream().map(SysAuthOuScopeEnt::getId).collect(Collectors.toList());
                userGroupWithRole.deleteBatchIds(collect);
            }
            //创建新关系
            List<SysAuthOuScopeView> viewList = new ArrayList<>();
            List<String> ouIds = input.getOuIds();
            for (String ouId : ouIds) {
                List<String> menuIds = input.getMenuIds();
                Date now = new Date();
                for (String menuId : menuIds) {
                    SysAuthOuScopeView entity = new SysAuthOuScopeView();
                    entity.setMenuId(menuId);
                    entity.setOuId(ouId);
                    entity.setUserGroupId(input.getUserGroupId());
                    entity.setId(UUID.randomUUID().toString());
                    //entity.setCreateById(context.getUser().getId());
                    entity.setCreateByName(context.getUser().getName());
                    entity.setCreateTime(now);
                    viewList.add(entity);
                }
            }

            if(ObjectUtils.isNotEmpty(viewList)){
                viewUserGroupWithRole.batchInsert(viewList);
            }
            return XServiceResult.OK;
        });
    }

    @XText("创建用户组和用户关系表")
    @XApiPost
    public XServiceResult createSysUserWithUserGroupRel(XContext context, CreateSysUserWithUserGroupRelInput input){
        return XTransactionHelper.begin(context,()-> {
            //SysUserWithUserGroupRel
            SysUserWithUserGroupRelMapper userGroupWithRole = context.getBean(SysUserWithUserGroupRelMapper.class);
            SysUserWithUserGroupRelViewMapper userWithUserGroupRelViewMapper = context.getBean(SysUserWithUserGroupRelViewMapper.class);
            //删除原关系
            QueryWrapper<SysUserWithUserGroupRelEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(SysUserWithUserGroupRelEnt::getUserGroupId, input.getUserGroupId());
            PageHelper.startPage(1, 1000);
            List<SysUserWithUserGroupRelEnt> sysAuthOuScopeEnts = userGroupWithRole.selectList(queryWrapper);
            if (!CollectionUtils.isEmpty(sysAuthOuScopeEnts)) {
                List<String> collect = sysAuthOuScopeEnts.stream().map(SysUserWithUserGroupRelEnt::getId).collect(Collectors.toList());
                userGroupWithRole.deleteBatchIds(collect);
            }
            //创建新关系
            List<SysUserWithUserGroupRelView> items = input.getItems();
            Date now = new Date();
            List<SysUserWithUserGroupRelView> viewList = new ArrayList<>();
            for (SysUserWithUserGroupRelView obj : items) {
                SysUserWithUserGroupRelView entity = new SysUserWithUserGroupRelView();
                entity.setUserId(obj.getUserId());
                entity.setUserGroupId(input.getUserGroupId());
                entity.setId(UUID.randomUUID().toString());
                //entity.setCreateById(context.getUser().getId());
                entity.setCreateByName(context.getUser().getName());
                entity.setCreateTime(now);
                viewList.add(entity);
            }
            if(ObjectUtils.isNotEmpty(viewList)){
                userWithUserGroupRelViewMapper.batchInsert(viewList);
            }
            return XServiceResult.OK;
        });
    }

    @XText("修改用户组")
    @XApiPost
    public XServiceResult updateSysUserGroup(XContext context, UpdateSysUserGroupInput input){
        SysUserGroupMapper mapper = context.getBean(SysUserGroupMapper.class);
        QueryWrapper<SysUserGroupEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysUserGroupEnt::getId, input.getId());
        SysUserGroupEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XText("删除用户组")
    @XApiPost
    public XServiceResult deleteSysUserGroup(XContext context, DeleteSysUserGroupInput input){
        return XTransactionHelper.begin(context,()-> {
            SysUserGroupMapper mapper = context.getBean(SysUserGroupMapper.class);
            SysUserGroupRoleRelMapper groupRoleMapper = context.getBean(SysUserGroupRoleRelMapper.class);
            SysAuthOuScopeMapper groupMenuMapper = context.getBean(SysAuthOuScopeMapper.class);
            SysUserWithUserGroupRelMapper groupUserMapper = context.getBean(SysUserWithUserGroupRelMapper.class);
            QueryWrapper<SysUserGroupEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(SysUserGroupEnt::getId, input.getId());
            SysUserGroupEnt entity = mapper.selectOne(queryWrapper);
            if (entity == null) {
                return XServiceResult.error(context, XError.NotFound);
            }
            mapper.deleteById(entity);
            //删除角色用户组信息
            QueryWrapper<SysUserGroupRoleRelEnt> roleWrapper = new QueryWrapper<>();
            roleWrapper.lambda().eq(SysUserGroupRoleRelEnt::getUserGroupId, entity.getId());
            List<SysUserGroupRoleRelEnt> sysUserGroupEnts = groupRoleMapper.selectList(roleWrapper);
            if (sysUserGroupEnts != null && sysUserGroupEnts.size() > 0) {
                for (SysUserGroupRoleRelEnt obj : sysUserGroupEnts) {
                    mapper.deleteById(obj);
                }
            }
            //删除菜单用户组信息
            QueryWrapper<SysAuthOuScopeEnt> menuWrapper = new QueryWrapper<>();
            menuWrapper.lambda().eq(SysAuthOuScopeEnt::getUserGroupId, entity.getId());
            List<SysAuthOuScopeEnt> menuGroupEnts = groupMenuMapper.selectList(menuWrapper);
            if (menuGroupEnts != null && menuGroupEnts.size() > 0) {
                for (SysAuthOuScopeEnt obj : menuGroupEnts) {
                    groupMenuMapper.deleteById(obj);
                }
            }
            //删除用户用户组信息
            QueryWrapper<SysUserWithUserGroupRelEnt> userWrapper = new QueryWrapper<>();
            userWrapper.lambda().eq(SysUserWithUserGroupRelEnt::getUserGroupId, entity.getId());
            List<SysUserWithUserGroupRelEnt> userGroupEnts = groupUserMapper.selectList(userWrapper);
            if (userGroupEnts != null && userGroupEnts.size() > 0) {
                for (SysUserWithUserGroupRelEnt obj : userGroupEnts) {
                    groupUserMapper.deleteById(obj);
                }
            }
            return XServiceResult.OK;
        });
    }

    @XText("根据id查询用户组详情")
    @XApiPost
    public XSingleResult<GetSysUserGroupOutput> getSysUserGroup(XContext context, GetSysUserGroupInput input){
        SysUserGroupMapper mapper = context.getBean(SysUserGroupMapper.class);
        QueryWrapper<SysUserGroupEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysUserGroupEnt::getId, input.getId());
        SysUserGroupEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetSysUserGroupOutput output = new GetSysUserGroupOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

    @XText("分页查询用户组信息")
    @XApiPost
    public XPageResult<QuerySysUserGroupOutput> querySysUserGroup(XContext context, QuerySysUserGroupInput input){
        SysUserGroupMapper mapper = context.getBean(SysUserGroupMapper.class);
        QueryWrapper<SysUserGroupEnt> queryWrapper = new QueryWrapper<>();
        if(StringUtils.isNotBlank(input.getUserGroupName())){
            queryWrapper.lambda().like(SysUserGroupEnt::getUserGroupName, input.getUserGroupName());
        }
        if(StringUtils.isNotBlank(input.getOuId())){
            queryWrapper.lambda().like(SysUserGroupEnt::getOuId, input.getOuId());
        }
        return XMapperHelper.query(mapper, input, queryWrapper, QuerySysUserGroupOutput.class);
    }

    @XApiPost
    public XPageResult<QuerySysUserGroupOutput> dynamicQuerySysUserGroup(XContext context, CustomQueryInput input){
        SysUserGroupMapper mapper = context.getBean(SysUserGroupMapper.class);
        return XMapperHelper.query(mapper, input, SysUserGroupEnt.class, QuerySysUserGroupOutput.class);
    }

    @XApiPost
    public XSingleResult<GetSysUserGroupViewOutput> getSysUserGroupView(XContext context, GetSysUserGroupViewInput input){
        SysUserGroupViewMapper mapper = context.getBean(SysUserGroupViewMapper.class);
        SysUserGroupView record = new SysUserGroupView();
        XCopyUtils.copyObject(input, record);
        SysUserGroupView view = mapper.selectOne(record);
        if(view == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetSysUserGroupViewOutput output = new GetSysUserGroupViewOutput();
        XCopyUtils.copyObject(view, output);
        return XSingleResult.success(output);
    }

    @XText("分页查询用户组信息（带组组织机构名称）")
    @XApiPost
    public XPageResult<QuerySysUserGroupViewOutput> querySysUserGroupView(XContext context, QuerySysUserGroupViewInput input){
        SysUserGroupViewMapper mapper = context.getBean(SysUserGroupViewMapper.class);
        SysUserGroupView record = new SysUserGroupView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<SysUserGroupView> list = mapper.selectList(record);
        PageInfo<SysUserGroupView> pageInfo = new PageInfo<>(list);
        List<QuerySysUserGroupViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySysUserGroupViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

}
