package pps.core.system.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.util.CollectionUtils;
import pps.cloud.system.service.SystemDictionaryService;
import pps.cloud.system.service.data.QuerySysDictionaryViewInput;
import pps.cloud.system.service.data.QuerySysDictionaryViewOutput;
import pps.core.common.session.PpsUserSession;
import pps.core.common.utils.AESUtil;
import pps.core.common.utils.ExportExcelUtils;
import pps.core.common.utils.ImportExcelUtils;
import pps.core.common.utils.MD5Util;
import pps.core.system.constant.SysOrganizationCode;
import pps.core.system.entity.*;
import pps.core.system.mapper.*;
import pps.core.system.service.data.sys_user_import_log.*;
import pps.core.system.utils.UserInfoUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.tool.XStorageTool;
import xstartup.base.util.XCopyUtils;
import xstartup.base.util.XDateUtils;
import xstartup.base.util.XStringUtils;
import xstartup.core.base.helper.XThreadHelper;
import xstartup.data.*;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.api.annotation.XApiUpload;
import xstartup.feature.mybatis.helper.XMapperHelper;

import java.io.InputStream;
import java.util.*;


@XText("用户批量导入管理")
@XService
public class SysUserImportLogService {
    private HashMap<Integer,Boolean> isNeedNotEmptyMap=new HashMap<>();

    @XText("查询用户批量导入列表")
    @XApiGet
    public XPageResult<QuerySysUserImportLogViewOutput> querySysUserImportLogViewPage(XContext context, QuerySysUserImportLogViewInput input){
        SysUserImportLogMapper mapper = context.getBean(SysUserImportLogMapper.class);
        //PpsUserSession session = context.getSession(PpsUserSession.class);
        QueryWrapper<SysUserImportLogEnt> queryWrapper = new QueryWrapper<>();
        //queryWrapper.lambda().eq(SysUserImportLogEnt::getOuId , session.getOuId());
        if (input.getStatus() != null) {
            queryWrapper.lambda().eq(SysUserImportLogEnt::getStatus, input.getStatus());
        }
        if (StringUtils.isNotBlank(input.getOuId())) {
            queryWrapper.lambda().eq(SysUserImportLogEnt::getOuId, input.getOuId());
        }
        if (StringUtils.isNotBlank(input.getStartDataDate())) {
            queryWrapper.lambda().ge(SysUserImportLogEnt::getCreateTime, input.getStartDataDate());
        }
        if (StringUtils.isNotBlank(input.getEndDataDate())) {
            queryWrapper.lambda().le(SysUserImportLogEnt::getCreateTime, input.getEndDataDate());
        }
        if (StringUtils.isNotBlank(input.getCreateByName())) {
            queryWrapper.lambda().like(SysUserImportLogEnt::getCreateByName, input.getCreateByName());
        }
        queryWrapper.lambda().orderByDesc(SysUserImportLogEnt::getCreateTime);
        XPageResult<QuerySysUserImportLogViewOutput> query = XMapperHelper.query(mapper, input, queryWrapper, QuerySysUserImportLogViewOutput.class);
        return query;
    }

    @XApiPost
    @XText("删除油品信息")
    public XServiceResult deleteBaseOilType(XContext context, DeleteSysUserImportLogViewInput input) {
        SysUserImportLogMapper mapper = context.getBean(SysUserImportLogMapper.class);
        QueryWrapper<SysUserImportLogEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysUserImportLogEnt::getId, input.getId());
        SysUserImportLogEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        //使用逻辑删除
        PpsUserSession session = context.getSession(PpsUserSession.class);
        entity.setModifyById(session.getId());
        entity.setModifyByName(session.getUserName());
        entity.setModifyTime(new Date());
        entity.setIsDelete(0);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XText("根据日志id，查询用户批量导入数据列表")
    @XApiGet
    public XPageResult<QuerySysUserImportLogDetailOutput> querySysUserImportUserDetailViewPage(XContext context, QuerySysUserImportLogDetailInput input){
        SysUserImportLogDetailMapper mapper = context.getBean(SysUserImportLogDetailMapper.class);
        SysUserImportLogDetailEnt ent = XCopyUtils.copyNewObject(input, SysUserImportLogDetailEnt.class);
        QueryWrapper<SysUserImportLogDetailEnt> queryWrapper = setQueryWrapper(ent);
        XPageResult<QuerySysUserImportLogDetailOutput> query = XMapperHelper.query(mapper, input, queryWrapper, QuerySysUserImportLogDetailOutput.class);
        return query;
    }

    @XText("导出用户信息数据")
    @XApiGet
    public XFileResult exportSysUserImportLogDetailData(XContext context, GetSysUserImportLogDetailInput input) {
        try {
            List<SysUserImportLogDetailEnt> sysUserImportLogDetailEnts = new ArrayList<>();
            String fileSavePath = XStorageTool.getAbsolutePath("/temp/excel/用户信息.xlsx");
            String templateFilePath = "template/exportSysUserImportLogDetailTemplate.xlsx";
            if(input.getType() != null && input.getType() == 1){//导出模板
                SysUserImportLogDetailEnt example = new SysUserImportLogDetailEnt();
                example.setOuName("国家石油天然气管网集团有限公司");
                example.setLoginName("zhangsan");
                example.setIamLoginName("zhangsan");
                example.setUserCode("userCode");
                example.setUserName("张三");
                example.setPassword("111111");
                example.setIsOuter("是");
                example.setIsInner("否");
                example.setDuty("董事长");
                example.setPhone("13333333333");
                example.setEmail("example@example.com");
                example.setPostalCode("100010");
                example.setAddress("北京市");
                example.setUserId("zhangsan");
                sysUserImportLogDetailEnts.add(example);
            }else {
                templateFilePath = "template/exportSysUserImportLogDetailMsgTemplate.xlsx";
                SysUserImportLogDetailMapper mapper = context.getBean(SysUserImportLogDetailMapper.class);
                SysUserImportLogDetailEnt ent = XCopyUtils.copyNewObject(input, SysUserImportLogDetailEnt.class);
                QueryWrapper<SysUserImportLogDetailEnt> queryWrapper = setQueryWrapper(ent);
                sysUserImportLogDetailEnts = mapper.selectList(queryWrapper);
            }
            if (!CollectionUtils.isEmpty(sysUserImportLogDetailEnts)) {
                Map<String, Object> params = new HashMap<>();
                params.put("list", sysUserImportLogDetailEnts);
                InputStream templateInputStream = this.getClass().getClassLoader().getResourceAsStream(templateFilePath);
                byte[] excelContent = ExportExcelUtils.genSingleExcelFileData(templateInputStream, params);
                ExportExcelUtils.outputFileData(excelContent, fileSavePath);
                return XFileResult.success(XFileType.APPLICATION_XLSX, fileSavePath);
            } else {
                return XFileResult.error(200, "暂无数据导出");
            }
        } catch (Exception ex) {
            context.getLogger().error("导出出错", ex);
            return XFileResult.error(500, "导出失败");
        }
    }

    @XText("导入用户信息数据")
    @XApiUpload
    public XServiceResult importSysUserImportLogDetailData(XContext context, ImportSysUserImportLogDetailInput input) {
        try {
            if (input.getFile() == null)
                return XServiceResult.error(500, "未获取上传文件");
            PpsUserSession session = context.getSession(PpsUserSession.class);
            SysUserImportLogMapper mapper = context.getBean(SysUserImportLogMapper.class);
            SysUserImportLogEnt logEnt = new SysUserImportLogEnt();
            logEnt.setId(UUID.randomUUID().toString());
            logEnt.setOuId(session.getOuId());
            logEnt.setOuName(session.getOuName());
            logEnt.setCreateById(session.getId());
            logEnt.setCreateByName(session.getUserName());
            logEnt.setCreateTime(new Date());
            logEnt.setIsDelete(1);
            List<SysUserImportLogDetailEnt> successEntList = new ArrayList<>();
            List<SysUserImportLogDetailEnt> errorEntList = new ArrayList<>();
            this.importExcelData(context, input , successEntList , errorEntList);
            if(CollectionUtils.isEmpty(successEntList))
                logEnt.setStatus(2);
            if(CollectionUtils.isEmpty(errorEntList))
                logEnt.setStatus(0);
            if(!CollectionUtils.isEmpty(errorEntList) && !CollectionUtils.isEmpty(successEntList))
                logEnt.setStatus(1);
            mapper.insert(logEnt);
            SysUserImportLogDetailMapper detailMapper = context.getBean(SysUserImportLogDetailMapper.class);
            List<QuerySysDictionaryViewOutput> dictList = getDictList(context , "DUTY");
            SysUserMapper userMapper = context.getBean(SysUserMapper.class);
            SysUserOrganizationRelMapper userOrganizationRelMapper = context.getBean(SysUserOrganizationRelMapper.class);
            SysUserRoleRelMapper userRoleRelMapper = context.getBean(SysUserRoleRelMapper.class);
            XThreadHelper.async(new Runnable() {
                @Override
                public void run() {
                    for (int i = 0; i < successEntList.size(); i++) {
                        Date currentDate = new Date();
                        SysUserImportLogDetailEnt record = successEntList.get(i);
                        record.setId(UUID.randomUUID().toString());
                        record.setLogId(logEnt.getId());
                        record.setCreateById(session.getId());
                        record.setCreateByName(session.getUserName());
                        record.setCreateTime(currentDate);
                        detailMapper.insert(record);
                        //插入用户
                        SysUserEnt sysUserEnt = XCopyUtils.copyNewObject(record , SysUserEnt.class);
                        if(record.getIsInner().equals("是"))
                            sysUserEnt.setIsInner(1);
                        else
                            sysUserEnt.setIsInner(2);
                        if(record.getIsOuter().equals("是"))
                            sysUserEnt.setIsOuter(1);
                        else
                            sysUserEnt.setIsOuter(2);
                        sysUserEnt.setEndDate(XDateUtils.tryParse("2099-12-31"));
                        sysUserEnt.setIsSuperAdmin(0);
                        if(XStringUtils.isNotEmpty(sysUserEnt.getPassword()))
                            sysUserEnt.setPassword(MD5Util.encryptPassword(sysUserEnt.getPassword()));

                        else
                            sysUserEnt.setPassword("2e7832f9c3a0ca243c559017c07cceed");
                        sysUserEnt.setDuty(getDictKey(sysUserEnt.getDuty() ,dictList));
                        userMapper.insert(sysUserEnt);
                        //插入用户组织机构关系
                        SysUserOrganizationRelEnt sysUserOrganizationRelEnt = new SysUserOrganizationRelEnt();
                        sysUserOrganizationRelEnt.setId(UUID.randomUUID().toString());
                        sysUserOrganizationRelEnt.setOuId(record.getOuId());
                        sysUserOrganizationRelEnt.setUserId(sysUserEnt.getId());
                        sysUserOrganizationRelEnt.setCreateById(session.getId());
                        sysUserOrganizationRelEnt.setCreateByName(session.getUserName());
                        sysUserOrganizationRelEnt.setCreateTime(currentDate);
                        sysUserOrganizationRelEnt.setEffectTime(new Date());
                        sysUserOrganizationRelEnt.setEndTime(XDateUtils.tryParse("2099-12-31"));
                        userOrganizationRelMapper.insert(sysUserOrganizationRelEnt);
                        //插入用户角色关系
                        QueryWrapper<SysUserRoleRelEnt> queryWrapper = new QueryWrapper<>();
                        queryWrapper.lambda().eq(SysUserRoleRelEnt::getUserId , record.getUserIdAct());
                        List<SysUserRoleRelEnt> userRoleRelEnts = userRoleRelMapper.selectList(queryWrapper);
                        if(!CollectionUtils.isEmpty(userRoleRelEnts)){
                            userRoleRelEnts.stream().forEach(item->{
                                item.setId(UUID.randomUUID().toString());
                                item.setUserId(sysUserEnt.getId());
                                userRoleRelMapper.insert(item);
                            });
                        }
                    }

                    for (int i = 0; i < errorEntList.size(); i++) {
                        Date currentDate = new Date();
                        SysUserImportLogDetailEnt record = errorEntList.get(i);
                        record.setId(UUID.randomUUID().toString());
                        record.setLogId(logEnt.getId());
                        record.setCreateById(session.getId());
                        record.setCreateByName(session.getUserName());
                        record.setCreateTime(currentDate);
                        detailMapper.insert(record);
                    }
                }
            });
            return XServiceResult.OK;
        } catch (Exception ex) {
            context.getLogger().error("导入失败", ex);
            return XServiceResult.error(500, "导入失败：" + ex.toString().replace("java.lang.Exception:", ""));
        }
    }

    private void importExcelData(XContext context, ImportSysUserImportLogDetailInput param ,
                                                 List<SysUserImportLogDetailEnt> successEntList , List<SysUserImportLogDetailEnt> errorEntList) throws Exception {
        String fileName = param.getFile().getFileName();
        InputStream input = param.getFile().getInputStream();
        this.checkError(!fileName.endsWith(".xlsx"), "导入模板文件格式不正确，请核对(文档格式应为.xlsx)");
        Workbook wb = new XSSFWorkbook(input);
        Sheet sheet = wb.getSheetAt(0);
        Iterator<Row> rows = sheet.rowIterator();
        this.checkError(rows == null || !rows.hasNext(), "未导入任何数据");

        DataFormatter formatter = new DataFormatter();
        List<String> tempRowTitle = createTitles();

        Row titleRowFromExcel = rows.next();
        String titleErrorMessage = "";
        isNeedNotEmptyMap.clear();

        for (int i = 0; i < tempRowTitle.size(); i++) {
            String titleFormExcel = formatter.formatCellValue(titleRowFromExcel.getCell(i)).trim();
            isNeedNotEmptyMap.put(i, titleFormExcel.startsWith("*"));
            titleFormExcel = titleFormExcel.replace("*", "");

            if (!tempRowTitle.get(i).equals(titleFormExcel)) {
                titleErrorMessage = titleErrorMessage.concat("第" + (i + 1) + "列表头不正确应为“" + tempRowTitle.get(i) + "”,");
            }
        }
        this.checkError(StringUtils.isNotBlank(titleErrorMessage), titleErrorMessage + "请根据模板修改；");

        SysOrganizationViewMapper mapper = context.getBean(SysOrganizationViewMapper.class);
        SysUserMapper userMapper = context.getBean(SysUserMapper.class);
        QueryWrapper<SysUserEnt> userEntQueryWrapper = new QueryWrapper<>();
        while (rows.hasNext()) {
            SysUserImportLogDetailEnt resultItem = new SysUserImportLogDetailEnt();
            resultItem.setId(UUID.randomUUID().toString());
            resultItem.setStatus(0);
            resultItem.setMsg("插入成功");
            Row row = rows.next();
            if (ImportExcelUtils.isRowEmpty(row)) {
                resultItem.setMsg("数据不能为空；");
                resultItem.setStatus(1);
                errorEntList.add(resultItem);
                continue;
            }
//            String dateErrorMessage = this.checkEmptyData(row, tempRowTitle);
//            if (StringUtils.isNotBlank(dateErrorMessage)) {
//                resultItem.setMsg(dateErrorMessage);
//                resultItem.setStatus(1);
//                errorEntList.add(resultItem);
//                continue;
//            }
            StringBuilder errorMessageBuilder = new StringBuilder();
            for (int i = 0; i < tempRowTitle.size(); i++) {
                Cell cell = row.getCell(i);
                String cellText = formatter.formatCellValue(cell).trim();
                switch (i) {
                    case 0: //组织机构名称
                        try {
                            if (StringUtils.isBlank(cellText)){
                                if(this.checkEmptyData(row , tempRowTitle , 0));
                                    errorMessageBuilder.append("组织机构名称不能为空；");
                            }else {
                                SysOrganizationView record = new SysOrganizationView();
                                record.setOuName(cellText);
                                record.setIsDeleted(SysOrganizationCode.NORMAL);
                                record.setEndTime(new Date());
                                List<SysOrganizationView> viewList = mapper.selectOrganizationEffectiveByOuName(record);
                                resultItem.setOuName(cellText);
                                if(viewList.size()> 1)
                                    errorMessageBuilder.append("组织机构名称匹配多个；");
                                else if(viewList.size() == 1){
                                    resultItem.setOuId(viewList.get(0).getId());
                                }
                                else
                                    errorMessageBuilder.append("组织机构未匹配；");
                            }
                        } catch (Exception ex) {

                        }
                        break;
                    case 1: //登录账户
                        try {
                            if (StringUtils.isBlank(cellText)){
                                if(this.checkEmptyData(row , tempRowTitle , 1))
                                    errorMessageBuilder.append("登录账号不能为空；");
                            }else {
                                userEntQueryWrapper.clear();
                                userEntQueryWrapper.lambda().eq(SysUserEnt::getLoginName , cellText);
                                List<SysUserEnt> userEntList = userMapper.selectList(userEntQueryWrapper);
                                resultItem.setLoginName(cellText);
                                if(userEntList.size() > 0)
                                    errorMessageBuilder.append("登录账号匹配多个；");
                            }
                        } catch (Exception ex) {

                        }
                        break;
                    case 2: //IAM登录账户
                        try {
                            if (StringUtils.isBlank(cellText)){
                                if(this.checkEmptyData(row , tempRowTitle , 2))
                                    errorMessageBuilder.append("IAM登录账号不能为空；");
                            }else {
                                userEntQueryWrapper.clear();
                                userEntQueryWrapper.lambda().eq(SysUserEnt::getIamLoginName, cellText);
                                List<SysUserEnt> userEntList = userMapper.selectList(userEntQueryWrapper);
                                resultItem.setIamLoginName(cellText);
                                if (userEntList.size() > 0)
                                    errorMessageBuilder.append("IAM登录账号匹配多个；");
                            }
                        } catch (Exception ex) {

                        }
                        break;
                    case 3: //员工编码
                        try {
                            if (StringUtils.isBlank(cellText)){
                                if(this.checkEmptyData(row , tempRowTitle , 3))
                                    errorMessageBuilder.append("员工编码不能为空；");
                            }else {
                                resultItem.setUserCode(cellText);
                            }
                        } catch (Exception ex) {

                        }
                        break;
                    case 4: //用户姓名
                        try {
                            if (StringUtils.isBlank(cellText)){
                                if(this.checkEmptyData(row , tempRowTitle , 4))
                                    errorMessageBuilder.append("用户姓名不能为空；");
                            }else {
                                resultItem.setUserName(cellText);
                            }
                        } catch (Exception ex) {

                        }
                        break;
                    case 5: //登录密码
                        try {
                            if (StringUtils.isBlank(cellText)){
                                if(this.checkEmptyData(row , tempRowTitle , 5))
                                    errorMessageBuilder.append("登录密码不能为空；");
                            }else {
                                resultItem.setPassword(cellText);
                            }
                        } catch (Exception ex) {

                        }
                        break;
                    case 6: //内网访问
                        try {
                            if (StringUtils.isBlank(cellText)){
                                if(this.checkEmptyData(row , tempRowTitle , 6))
                                    errorMessageBuilder.append("内网访问不能为空；");
                            }else {
                                resultItem.setIsInner(cellText);
                            }
                        } catch (Exception ex) {

                        }
                        break;
                    case 7: //外网访问
                        try {
                            if (StringUtils.isBlank(cellText)){
                                if(this.checkEmptyData(row , tempRowTitle , 7))
                                    errorMessageBuilder.append("外网访问不能为空；");
                            }else {
                                resultItem.setIsOuter(cellText);
                            }
                        } catch (Exception ex) {

                        }
                        break;
                    case 8: //职务
                        try {
                            if (StringUtils.isBlank(cellText)){
                                if(this.checkEmptyData(row , tempRowTitle , 8))
                                    errorMessageBuilder.append("职务不能为空；");
                            }else {
                                resultItem.setDuty(cellText);
                            }
                        } catch (Exception ex) {

                        }
                        break;
                    case 9: //邮编
                        try {
                            if (StringUtils.isBlank(cellText)){
                                if(this.checkEmptyData(row , tempRowTitle , 9))
                                    errorMessageBuilder.append("邮编不能为空；");
                            }else {
                                resultItem.setPostalCode(cellText);
                            }
                        } catch (Exception ex) {

                        }
                        break;
                    case 10: //手机号
                        try {
                            if (StringUtils.isBlank(cellText)){
                                if(this.checkEmptyData(row , tempRowTitle , 10))
                                    errorMessageBuilder.append("手机号不能为空；");
                            }else {
                                resultItem.setPhone(UserInfoUtil.encrypt(cellText));
                            }
                        } catch (Exception ex) {

                        }
                        break;
                    case 11: //用户邮箱
                        try {
                            if (StringUtils.isBlank(cellText)){
                                if(this.checkEmptyData(row , tempRowTitle , 11))
                                    errorMessageBuilder.append("用户邮箱不能为空；");
                            }else {
                                resultItem.setEmail(cellText);
                            }
                        } catch (Exception ex) {

                        }
                        break;
                    case 12: //通讯地址
                        try {
                            if (StringUtils.isBlank(cellText)){
                                if(this.checkEmptyData(row , tempRowTitle , 12))
                                    errorMessageBuilder.append("通讯地址不能为空；");
                            }else {
                                resultItem.setAddress(cellText);
                            }
                        } catch (Exception ex) {

                        }
                        break;
                    case 13: //角色用户id
                        try {
                            if (StringUtils.isBlank(cellText)){
                                if(this.checkEmptyData(row , tempRowTitle , 13))
                                    errorMessageBuilder.append("角色用户IAM账户不能为空；");
                            }else {
                                userEntQueryWrapper.clear();
                                userEntQueryWrapper.lambda().eq(SysUserEnt::getIamLoginName , cellText);
                                List<SysUserEnt> userEntList = userMapper.selectList(userEntQueryWrapper);
                                resultItem.setUserId(cellText);
                                if(userEntList.size() == 0)
                                    errorMessageBuilder.append("角色用户IAM账户未匹配；");
                                else
                                    resultItem.setUserIdAct(userEntList.get(0).getId());
                            }
                        } catch (Exception ex) {

                        }
                        break;
                    default:
                        break;
                }
            }
            if (errorMessageBuilder.length() > 0){
                resultItem.setMsg(errorMessageBuilder.toString());
                resultItem.setStatus(1);
                errorEntList.add(resultItem);
            }else
                successEntList.add(resultItem);
        }
    }

    private List<String> createTitles() {
        List<String> titles = new ArrayList<String>();
        titles.add("所属单位");
        titles.add("登录账号");
        titles.add("IAM登录账号");
        titles.add("员工编码");
        titles.add("用户姓名");
        titles.add("登录密码");
        titles.add("内网访问");
        titles.add("外网访问");
        titles.add("职务");
        titles.add("邮编");
        titles.add("手机号");
        titles.add("用户邮箱");
        titles.add("通讯地址");
        titles.add("角色用户IAM账号");
        return titles;
    }

    private String checkEmptyData(Row row, List<String> tempRowTitle) {
        String result = "";
        DataFormatter formatter = new DataFormatter();

        for (int i = 0; i < isNeedNotEmptyMap.size(); i++) {
            Cell cell = row.getCell(i);
            Boolean isNeedNotEmpty = isNeedNotEmptyMap.get(i);
            Boolean isEmptyValue = StringUtils.isBlank(formatter.formatCellValue(cell));

            if (isNeedNotEmpty && isEmptyValue) {
                result = result.concat(String.format("“%s”", tempRowTitle.get(i).replace("*", "")));
            }
        }
        if (StringUtils.isNotBlank(result)) {
            result =  result + "不能为空；";
        }
        return result;
    }

    private Boolean checkEmptyData(Row row, List<String> tempRowTitle , int index) {
        Boolean isNeed = false;
        DataFormatter formatter = new DataFormatter();
        Cell cell = row.getCell(index);
        Boolean isNeedNotEmpty = isNeedNotEmptyMap.get(index);
        Boolean isEmptyValue = StringUtils.isBlank(formatter.formatCellValue(cell));
        if (isNeedNotEmpty && isEmptyValue) {
            isNeed = true;
        }
        return isNeed;
    }
    private void checkError(boolean isError, String errorMessage) throws Exception {
        if (isError)
            throw new Exception(errorMessage);
    }

    public QueryWrapper<SysUserImportLogDetailEnt> setQueryWrapper(SysUserImportLogDetailEnt input) {
        QueryWrapper<SysUserImportLogDetailEnt> queryWrapper = new QueryWrapper<>();
        if (input.getStatus() != null) {
            queryWrapper.lambda().eq(SysUserImportLogDetailEnt::getStatus, input.getStatus());
        }
        if (StringUtils.isNotBlank(input.getLogId())) {
            queryWrapper.lambda().eq(SysUserImportLogDetailEnt::getLogId, input.getLogId());
        }
        queryWrapper.lambda().orderByDesc(SysUserImportLogDetailEnt::getCreateTime);
        return queryWrapper;
    }

    public List<QuerySysDictionaryViewOutput> getDictList(XContext context , String alias ){
        SystemDictionaryService dictionaryService = context.getBean(SystemDictionaryService.class);
        QuerySysDictionaryViewInput dictInput = new QuerySysDictionaryViewInput();
        dictInput.setAlias(alias);
        dictInput.setPage(1);
        dictInput.setLimit(999);
        XListResult<QuerySysDictionaryViewOutput> dictPageResult = dictionaryService.queryChildSysDictionarysByParentAlias(context, dictInput);
        dictPageResult.throwIfFail();
        return dictPageResult.getResult();
    }

    public String getDictKey(String dictName,List<QuerySysDictionaryViewOutput> dictList){
        if(StringUtils.isBlank(dictName) || CollectionUtils.isEmpty(dictList)){
            return "";
        }
        for (QuerySysDictionaryViewOutput dict : dictList) {
            if(dict.getDicName().equals(dictName)){
                return dict.getDicKey();
            }
        }
        return "";
    }

    @XText("更新手机号码")
    @XApiPost
    @XApiAnonymous
    public XServiceResult updateUserPhone(XContext context){
        SysUserMapper sysUserMapper = context.getBean(SysUserMapper.class);
        QueryWrapper<SysUserEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().last("where length(phone) = 11");
        List<SysUserEnt> sysUserEnts = sysUserMapper.selectList(queryWrapper);
        if(!sysUserEnts.isEmpty()){
            sysUserEnts.stream().forEach(item->{
                item.setPhone(UserInfoUtil.encrypt(item.getPhone()));
                sysUserMapper.updateById(item);
            });
        }
        return XServiceResult.OK;
    }
}
