package pps.core.system.service;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import pps.cloud.system.service.SystemScadaIhdService;
import pps.cloud.system.service.data.*;
import pps.core.system.entity.ScadaEnvironmentInfo;
import pps.core.system.entity.ScadaGatherBoxRelDetail;
import pps.core.system.entity.ScadaGatherConfView;
import pps.core.system.entity.ScadaGatherData;
import pps.core.system.mapper.ScadaEnvironmentInfoMapper;
import pps.core.system.mapper.ScadaGatherBoxRelDetailMapper;
import pps.core.system.mapper.ScadaGatherConfViewMapper;
import pps.core.system.mapper.ScadaGatherDataMapper;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

@XService
public class SystemScadaIhdServiceImpl implements SystemScadaIhdService {

    @Override
    public XListResult<ScadaEnvironmentInfoOutput> getScadaEnvList(XContext context, ScadaEnvironmentInfoInput input) {
        ScadaEnvironmentInfoMapper mapper = context.getBean(ScadaEnvironmentInfoMapper.class);
        QueryWrapper<ScadaEnvironmentInfo> scadaEnvironmentInfoMapperQueryWrapper = new QueryWrapper<>();
        scadaEnvironmentInfoMapperQueryWrapper.eq("type", input.getScadaEnvType());
        scadaEnvironmentInfoMapperQueryWrapper.eq("is_enable", true);
        List<ScadaEnvironmentInfo> ts = mapper.selectList(scadaEnvironmentInfoMapperQueryWrapper);
        List<ScadaEnvironmentInfoOutput> outputList = XCopyUtils.copyNewList(ts, ScadaEnvironmentInfoOutput.class);

        return XListResult.success(outputList);
    }

    @Override
    public XSingleResult<ScadaGatherConfViewResultOutput> getScadaTagList(XContext context, ScadaTagInput input) {
        Date beforeMinute = input.getStartDate();
        ScadaGatherConfViewMapper viewMapper = context.getBean(ScadaGatherConfViewMapper.class);
        int currentPage = input.getCurrentPage();
        int pageSize = input.getPageSize();

        int totalRecord = viewMapper.selectTotal(input.getScadaEnvId(), input.getStartDate());
        int totalPageNum = (totalRecord + pageSize - 1) / pageSize;
        int start = (currentPage - 1) * pageSize;

        List<ScadaGatherConfView> scadaGatherConfViews = viewMapper.selectByPage(start, pageSize, input.getScadaEnvId(), beforeMinute);

        List<ScadaGatherConfViewOutput> outputList = XCopyUtils.copyNewList(scadaGatherConfViews, ScadaGatherConfViewOutput.class);

        ScadaGatherConfViewResultOutput scadaGatherConfViewResultOutput = new ScadaGatherConfViewResultOutput();
        scadaGatherConfViewResultOutput.setOutputList(outputList);
        scadaGatherConfViewResultOutput.setTotalPageNum(totalPageNum);

        return XSingleResult.success(scadaGatherConfViewResultOutput);
    }

    @Override
    public XServiceResult insertScadaGatherData(XContext context, ScadaGatherDataOutput input) {
        ScadaGatherDataMapper mapper = context.getBean(ScadaGatherDataMapper.class);
        ScadaGatherData scadaGatherData = XCopyUtils.copyNewObject(input, ScadaGatherData.class);
        mapper.insert(scadaGatherData);
        return XServiceResult.OK;
    }

    @Override
    public XServiceResult updateScadaGatherData(XContext context, ScadaGatherDataOutput input) {
        ScadaGatherDataMapper mapper = context.getBean(ScadaGatherDataMapper.class);
        ScadaGatherData scadaGatherData = XCopyUtils.copyNewObject(input, ScadaGatherData.class);
        mapper.updateById(scadaGatherData);
        return XServiceResult.OK;
    }

    @Override
    public XListResult<ScadaGatherBoxRelDetailOutput> selectListDetails(XContext context, ScadaGatherBoxRelDetailInput input) {
        ScadaGatherBoxRelDetailMapper detailMapper = context.getBean(ScadaGatherBoxRelDetailMapper.class);
        QueryWrapper<ScadaGatherBoxRelDetail> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("scada_gather_box_rel_id", input.getRelId());
        List<ScadaGatherBoxRelDetail> scadaGatherBoxRelDetails = detailMapper.selectList(queryWrapper);
        List<ScadaGatherBoxRelDetailOutput> scadaGatherBoxRelDetailOutputs = XCopyUtils.copyNewList(scadaGatherBoxRelDetails, ScadaGatherBoxRelDetailOutput.class);
        return XListResult.success(scadaGatherBoxRelDetailOutputs);
    }

    @Override
    public XServiceResult insertDefaultTable(XContext context, DefaultTableInput input) {
        ScadaGatherConfViewMapper bean = context.getBean(ScadaGatherConfViewMapper.class);
//        ScadaGatherConfViewMapper
        bean.insertDefaultData(input.getSql());

        return XServiceResult.OK;
    }

    @Override
    public XServiceResult updateDefaultTable(XContext context, DefaultTableInput input) {
        ScadaGatherConfViewMapper bean = context.getBean(ScadaGatherConfViewMapper.class);
        bean.updateDefaultData(input.getSql());
        return XServiceResult.OK;
    }

    @Override
    public XSingleResult<DefaultTableOutput> selectDefaultTable(XContext context, DefaultTableInput input) {
        ScadaGatherConfViewMapper bean = context.getBean(ScadaGatherConfViewMapper.class);
        HashMap<String, Object> stringStringHashMap = bean.selectBusinessModuleTable(input.getSql());
        DefaultTableOutput defaultTableOutput = new DefaultTableOutput();
        defaultTableOutput.setResultData(stringStringHashMap);

        return XSingleResult.success(defaultTableOutput);
    }

    @Override
    public XSingleResult<ScadaGatherDataOutput> selectScadaGatherDataByDataDate(XContext context, selectScadaGatherDataByDataDateOutput output) {
        ScadaGatherDataMapper bean = context.getBean(ScadaGatherDataMapper.class);
        ScadaGatherData data = bean.selectDataByDataDate(output.getScadaTag(), output.getDateData());
        ScadaGatherDataOutput scadaGatherDataOutput = XCopyUtils.copyNewObject(data, ScadaGatherDataOutput.class);
        return XSingleResult.success(scadaGatherDataOutput);
    }

    @Override
    public XListResult<ScadaGatherDataOutput> getScadaRemedyDataTagList(XContext context, ScadaEnvironmentInfoOutput scadaEnv) {
        ScadaGatherDataMapper bean = context.getBean(ScadaGatherDataMapper.class);
        List<ScadaGatherData> dataList = bean.selectRemedyList(scadaEnv.getId());

        List<ScadaGatherDataOutput> scadaGatherDataOutputs = XCopyUtils.copyNewList(dataList, ScadaGatherDataOutput.class);

        return XListResult.success(scadaGatherDataOutputs);
    }

}
