package pps.core.base.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import pps.core.base.constant.ScadaConstant;
import pps.core.base.entity.BasePhotovoltaicPlantEnt;
import pps.core.base.entity.BasePhotovoltaicPlantView;
import pps.core.base.entity.BaseWellheadEnt;
import pps.core.base.mapper.BasePhotovoltaicPlantMapper;
import pps.core.base.mapper.BasePhotovoltaicPlantViewMapper;
import pps.core.base.service.data.base_photovoltaic_plant.*;
import pps.core.base.utils.SessionSimulation;
import pps.core.common.session.PpsUserSession;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.data.CustomQueryInput;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;

import java.util.Date;
import java.util.List;

/**
 * @Author luoxiangyang
 * @Description: 光伏电站配置
 * @Date 2023/8/24
 **/
@XService
public class BasePhotovoltaicPlantService {
    //todo 模拟登陆
    private final PpsUserSession session = SessionSimulation.getSession();

    @XText("新增")
    @XApiAnonymous
    @XApiPost
    public XServiceResult createBasePhotovoltaicPlant(XContext context, CreateBasePhotovoltaicPlantInput input) {
        BasePhotovoltaicPlantMapper mapper = context.getBean(BasePhotovoltaicPlantMapper.class);
        BasePhotovoltaicPlantEnt entity = new BasePhotovoltaicPlantEnt();
        XCopyUtils.copyObject(input, entity);
        entity.setCreateById(session.getId());
        entity.setCreateByName(session.getUserName());
        entity.setCreateTime(new Date());
        entity.setModifyById(session.getId());
        entity.setModifyByName(session.getUserName());
        entity.setModifyTime(new Date());
        mapper.insert(entity);
        return XServiceResult.OK;
    }
    @XText("更新")
    @XApiAnonymous
    @XApiPost
    public XServiceResult updateBasePhotovoltaicPlant(XContext context, UpdateBasePhotovoltaicPlantInput input) {
        BasePhotovoltaicPlantMapper mapper = context.getBean(BasePhotovoltaicPlantMapper.class);
        QueryWrapper<BasePhotovoltaicPlantEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda()
                .eq(BasePhotovoltaicPlantEnt::getId, input.getId())
                .eq(BasePhotovoltaicPlantEnt::getIsDeleted, ScadaConstant.IS_DELETE_FLASE);
        BasePhotovoltaicPlantEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        entity.setModifyById(session.getId());
        entity.setModifyByName(session.getUserName());
        entity.setModifyTime(new Date());
        mapper.updateById(entity);
        return XServiceResult.OK;
    }
    @XText("删除")
    @XApiAnonymous
    @XApiPost
    public XServiceResult deleteBasePhotovoltaicPlant(XContext context, DeleteBasePhotovoltaicPlantInput input) {
        BasePhotovoltaicPlantMapper mapper = context.getBean(BasePhotovoltaicPlantMapper.class);
        QueryWrapper<BasePhotovoltaicPlantEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda()
                .eq(BasePhotovoltaicPlantEnt::getId, input.getId())
                .eq(BasePhotovoltaicPlantEnt::getIsDeleted, ScadaConstant.IS_DELETE_FLASE);
        BasePhotovoltaicPlantEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        entity = new BasePhotovoltaicPlantEnt();
        entity.setId(input.getId());
        entity.setIsDeleted(ScadaConstant.IS_DELETE_TRUE);
        //PpsUserSession session = context.getSession(PpsUserSession.class);
        entity.setModifyById(session.getId());
        entity.setModifyByName(session.getUserName());
        entity.setModifyTime(new Date());
        mapper.updateById(entity);
        return XServiceResult.OK;
    }
    @XText("根据id获取详情")
    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetBasePhotovoltaicPlantOutput> getBasePhotovoltaicPlant(XContext context, GetBasePhotovoltaicPlantInput input) {
        BasePhotovoltaicPlantMapper mapper = context.getBean(BasePhotovoltaicPlantMapper.class);
        QueryWrapper<BasePhotovoltaicPlantEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda()
                .eq(BasePhotovoltaicPlantEnt::getId, input.getId())
                .eq(BasePhotovoltaicPlantEnt::getIsDeleted, ScadaConstant.IS_DELETE_FLASE);
        BasePhotovoltaicPlantEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBasePhotovoltaicPlantOutput output = new GetBasePhotovoltaicPlantOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }
    @XText("分页查询")
    @XApiAnonymous
    @XApiGet
    public XPageResult<QueryBasePhotovoltaicPlantOutput> queryBasePhotovoltaicPlant(XContext context, QueryBasePhotovoltaicPlantInput input) {
        BasePhotovoltaicPlantMapper mapper = context.getBean(BasePhotovoltaicPlantMapper.class);
        String stationName = input.getStationName();
        String ouId = input.getOuId();
        String excludeIdFlag = input.getExcludeIdFlag();
        QueryWrapper<BasePhotovoltaicPlantEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda()
                .notExists("1".equals(excludeIdFlag), "SELECT 1 FROM base_power_line_plant w WHERE w.ou_id = {0} AND base_photovoltaic_plant.id = w.plant_id", input.getOuId())
                .eq(BasePhotovoltaicPlantEnt::getIsDeleted, ScadaConstant.IS_DELETE_FLASE)
                .eq(StrUtil.isNotEmpty(ouId),BasePhotovoltaicPlantEnt::getOuId, ouId)
                .like(StrUtil.isNotEmpty(stationName),BasePhotovoltaicPlantEnt::getStationName,stationName )
                .orderByDesc(BasePhotovoltaicPlantEnt::getModifyTime);
        return XMapperHelper.query(mapper, input, queryWrapper, QueryBasePhotovoltaicPlantOutput.class);
    }

}
