package pps.core.base.service;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.apache.commons.lang3.StringUtils;
import pps.cloud.system.service.SysOrganizationCloudService;
import pps.cloud.system.service.data.GetSysOrganizationViewInput;
import pps.cloud.system.service.data.GetSysOrganizationViewOutput;
import pps.cloud.system.service.data.QuerySysDictionaryViewOutput;
import pps.core.base.entity.*;
import pps.core.base.mapper.*;
import pps.core.base.service.data.base_power_line.*;
import pps.core.base.service.data.base_power_line_diesel.GetBasePowerLineDieselViewOutput;
import pps.core.base.service.data.base_power_line_plant.GetBasePowerLinePlantViewOutput;
import pps.core.base.service.data.base_power_line_storage.GetBasePowerLineStorageViewOutput;
import pps.core.base.service.data.base_power_line_wellhead.DynamicQueryBasePowerLineWellheadInput;
import pps.core.base.service.data.base_power_line_wellhead.DynamicQueryBasePowerLineWellheadViewOutput;
import pps.core.base.service.data.base_power_line_wellhead.GetBasePowerLineWellheadViewOutput;
import pps.core.base.utils.DictUtils;
import pps.core.common.entity.BaseEnt;
import pps.core.common.session.PpsUserSession;
import pps.core.common.utils.BaseEntUtils;
import pps.core.system.constant.DictTypeConstant;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.helper.XTransactionHelper;

import java.util.List;
import java.util.Objects;
import java.util.Optional;

/**
 * 输电线路配置模块
 *
 * @author ZWT
 * @date 2023/08/25
 */
@XService
@XText("输电线路配置模块")
public class BasePowerLineService {

    /**
     * 输电线路配置--新增
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XApiAnonymous
    @XApiPost
    @XText("输电线路配置--新增")
    public XServiceResult createBasePowerLine(XContext context, CreateBasePowerLineInput input) {
        PpsUserSession session = context.getSession(PpsUserSession.class);
        return XTransactionHelper.begin(context, () -> {
            BasePowerLineMapper mapper = context.getBean(BasePowerLineMapper.class);
            BasePowerLineEnt entity = XCopyUtils.copyNewObject(input, BasePowerLineEnt.class);
            //新增输电线路
            BaseEntUtils.setBaseEntDefault(entity, session);
            mapper.insert(entity);
            String lineId = entity.getId();
            String ouId = entity.getOuId();
            //新增关联信息
            this.saveLineRelation(lineId, ouId, session, input.getWellheadInputs(), input.getPlantInputs(), input.getStorageInputs(), input.getDieselInputs());
            return XServiceResult.OK;
        });
    }

    /**
     * 输电线路配置--修改
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XApiAnonymous
    @XApiPost
    @XText("输电线路配置--修改")
    public XServiceResult updateBasePowerLine(XContext context, UpdateBasePowerLineInput input) {
        PpsUserSession session = context.getSession(PpsUserSession.class);
        String lineId = input.getId();
        String ouId = input.getOuId();
        return XTransactionHelper.begin(context, () -> {
            BasePowerLineMapper mapper = context.getBean(BasePowerLineMapper.class);
            BasePowerLineEnt entity = this.selectOneByLineId(lineId, mapper);
            if (Objects.isNull(entity)) {
                return XServiceResult.error(context, XError.NotFound);
            }
            //删除线路关联信息
            this.deleteLineRelation(context, lineId);
            //新增关联信息
            this.saveLineRelation(lineId, ouId, session, input.getWellheadInputs(), input.getPlantInputs(), input.getStorageInputs(), input.getDieselInputs());
            XCopyUtils.copyObject(input, entity);
            BaseEntUtils.setBaseEntDefault(entity, session);
            mapper.updateById(entity);
            return XServiceResult.OK;
        });
    }

    /**
     * 输电线路配置--删除
     * (如果被引用,则不能删除)
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XApiAnonymous
    @XApiPost
    @XText("输电线路配置--删除")
    public XServiceResult deleteBasePowerLine(XContext context, DeleteBasePowerLineInput input) {
        String lineId = input.getId();
        //todo: 检查是否被其它功能引用
        return XTransactionHelper.begin(context, () -> {
            BasePowerLineMapper mapper = context.getBean(BasePowerLineMapper.class);
            BasePowerLineEnt entity = this.selectOneByLineId(lineId, mapper);
            if (Objects.isNull(entity)) {
                return XServiceResult.error(context, XError.NotFound);
            }
            //删除线路关联信息
            this.deleteLineRelation(context, lineId);
            mapper.deleteById(entity);
            return XServiceResult.OK;
        });
    }

    /**
     * 输电线路配置--详情
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link GetBasePowerLineOutput}>
     */
    @XApiAnonymous
    @XApiGet
    @XText("输电线路配置--详情")
    public XSingleResult<GetBasePowerLineViewOutput> getBasePowerLineView(XContext context, GetBasePowerLineInput input) {
        String lineId = input.getId();
        BasePowerLineViewMapper mapper = context.getBean(BasePowerLineViewMapper.class);
        BasePowerLineView entity = mapper.selectOne(BasePowerLineView.builder()
                .id(lineId)
                .build());
        if (Objects.isNull(entity)) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBasePowerLineViewOutput output = XCopyUtils.copyNewObject(entity, GetBasePowerLineViewOutput.class);
        //设置关联井口
        BasePowerLineWellheadViewMapper wellheadViewMapper = context.getBean(BasePowerLineWellheadViewMapper.class);
        List<BasePowerLineWellheadView> wellheadViewList = wellheadViewMapper.selectList(BasePowerLineWellheadView.builder()
                .lineId(lineId)
                .build());
        if (CollectionUtil.isNotEmpty(wellheadViewList)) {
            output.setWellheadOutputs(XCopyUtils.copyNewList(wellheadViewList, GetBasePowerLineWellheadViewOutput.class));
        }
        //设置关联光伏
        BasePowerLinePlantViewMapper plantViewMapper = context.getBean(BasePowerLinePlantViewMapper.class);
        List<BasePowerLinePlantView> plantViewList = plantViewMapper.selectList(BasePowerLinePlantView.builder()
                .lineId(lineId)
                .build());
        if (CollectionUtil.isNotEmpty(plantViewList)) {
            output.setPlantOutputs(XCopyUtils.copyNewList(plantViewList, GetBasePowerLinePlantViewOutput.class));
        }
        //设置关联储能
        BasePowerLineStorageViewMapper storageViewMapper = context.getBean(BasePowerLineStorageViewMapper.class);
        List<BasePowerLineStorageView> storageViewList = storageViewMapper.selectList(BasePowerLineStorageView.builder()
                .lineId(lineId)
                .build());
        if (CollectionUtil.isNotEmpty(storageViewList)) {
            output.setStorageOutputs(XCopyUtils.copyNewList(storageViewList, GetBasePowerLineStorageViewOutput.class));
        }
        //设置关联柴发
        BasePowerLineDieselViewMapper dieselViewMapper = context.getBean(BasePowerLineDieselViewMapper.class);
        List<BasePowerLineDieselView> dieselViewList = dieselViewMapper.selectList(BasePowerLineDieselView.builder()
                .lineId(lineId)
                .build());
        if (CollectionUtil.isNotEmpty(dieselViewList)) {
            output.setDieselOutputs(XCopyUtils.copyNewList(dieselViewList, GetBasePowerLineDieselViewOutput.class));
        }
        return XSingleResult.success(output);
    }

    /**
     * 输电线路配置--分页列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XPageResult}<{@link QueryBasePowerLineViewOutput}>
     */
    @XApiAnonymous
    @XApiGet
    @XText("输电线路配置--分页列表")
    public XPageResult<QueryBasePowerLineViewOutput> queryBasePowerLineView(XContext context, QueryBasePowerLineViewInput input) {
        BasePowerLineViewMapper mapper = context.getBean(BasePowerLineViewMapper.class);
        BasePowerLineView record = XCopyUtils.copyNewObject(input, BasePowerLineView.class);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<BasePowerLineView> list = mapper.selectList(record);
        PageInfo<BasePowerLineView> pageInfo = new PageInfo<>(list);
        List<QueryBasePowerLineViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBasePowerLineViewOutput.class);
        String ouName = this.getSysOrgNameById(context, input.getOuId());
        List<QuerySysDictionaryViewOutput> gridTypeList = DictUtils.getDictList(context, DictTypeConstant.GRID_TYPE);
        List<QuerySysDictionaryViewOutput> lineTypeList = DictUtils.getDictList(context, DictTypeConstant.LINE_TYPE);
        for (QueryBasePowerLineViewOutput output : outputs) {
            output.setOuName(ouName);
            output.setGridTypeName(gridTypeList.stream()
                    .filter(row -> StringUtils.equals(row.getDicKey(), output.getGridTypeKey()))
                    .map(QuerySysDictionaryViewOutput::getDicName)
                    .findAny()
                    .orElse(null));
            output.setLineTypeName(lineTypeList.stream()
                    .filter(row -> StringUtils.equals(row.getDicKey(), output.getLineTypeKey()))
                    .map(QuerySysDictionaryViewOutput::getDicName)
                    .findAny()
                    .orElse(null));
        }
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    /**
     * 输电线路配置--列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link QueryBasePowerLineOutput}>
     */
    @XApiAnonymous
    @XApiGet
    @XText("输电线路配置--列表")
    public XListResult<DynamicQueryBasePowerLineOutput> queryBasePowerLineList(XContext context, DynamicQueryBasePowerLineInput input) {
        String lineId = input.getId();
        String ouId = input.getOuId();
        String upperLineId = input.getUpperLineId();
        String lineName = input.getLineName();
        String gridTypeKey = input.getGridTypeKey();
        String lineTypeKey = input.getLineTypeKey();
        String strategyId = input.getStrategyId();
        BasePowerLineMapper mapper = context.getBean(BasePowerLineMapper.class);
        List<BasePowerLineEnt> selectList = mapper.selectList(new LambdaQueryWrapper<BasePowerLineEnt>()
                .select(BaseEnt::getId, BasePowerLineEnt::getLineName)
                .eq(StringUtils.isNotBlank(lineId), BaseEnt::getId, lineId)
                .eq(StringUtils.isNotBlank(ouId), BasePowerLineEnt::getOuId, ouId)
                .eq(StringUtils.isNotBlank(upperLineId), BasePowerLineEnt::getUpperLineId, upperLineId)
                .like(StringUtils.isNotBlank(lineName), BasePowerLineEnt::getLineName, lineName)
                .eq(StringUtils.isNotBlank(gridTypeKey), BasePowerLineEnt::getGridTypeKey, gridTypeKey)
                .eq(StringUtils.isNotBlank(lineTypeKey), BasePowerLineEnt::getLineTypeKey, lineTypeKey)
                .eq(StringUtils.isNotBlank(strategyId), BasePowerLineEnt::getStrategyId, strategyId)
                .orderByDesc(BaseEnt::getModifyTime)
        );
        List<DynamicQueryBasePowerLineOutput> lineOutputList = XCopyUtils.copyNewList(selectList, DynamicQueryBasePowerLineOutput.class);
        return XListResult.success(lineOutputList);
    }

    /**
     * 输电线路配置--井口列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQueryBasePowerLineWellheadViewOutput}>
     */
    @XApiAnonymous
    @XApiGet
    @XText("输电线路配置--井口列表")
    public XListResult<DynamicQueryBasePowerLineWellheadViewOutput> queryBasePowerLineWellheadList(XContext context, DynamicQueryBasePowerLineWellheadInput input) {
        BasePowerLineWellheadViewMapper wellheadViewMapper = context.getBean(BasePowerLineWellheadViewMapper.class);
        BasePowerLineWellheadView basePowerLineWellheadView = new BasePowerLineWellheadView();
        basePowerLineWellheadView.setLineId(input.getLineId());
        String wellheadId = input.getWellheadId();
        if (StringUtils.isNotBlank(wellheadId)) {
            basePowerLineWellheadView.setWellheadId(wellheadId);
        }
        List<BasePowerLineWellheadView> selectList = wellheadViewMapper.selectList(basePowerLineWellheadView);
        List<DynamicQueryBasePowerLineWellheadViewOutput> lineOutputList = XCopyUtils.copyNewList(selectList, DynamicQueryBasePowerLineWellheadViewOutput.class);
        return XListResult.success(lineOutputList);
    }

    /*-----------------------------------private-----------------------------------*/


    /**
     * 通过组织id获取系统组织名称
     *
     * @param context 上下文
     * @param ouId    组织id
     * @return {@link String}
     */
    private String getSysOrgNameById(XContext context, String ouId) {
        SysOrganizationCloudService orgCloudService = context.getBean(SysOrganizationCloudService.class);
        GetSysOrganizationViewInput viewInput = new GetSysOrganizationViewInput();
        viewInput.setId(ouId);
        XSingleResult<GetSysOrganizationViewOutput> organization = orgCloudService.getSysOrganizationById(context, viewInput);
        organization.throwIfFail();
        return Optional.ofNullable(organization.getResult())
                .map(GetSysOrganizationViewOutput::getOuName)
                .orElse(null);
    }


    /**
     * 通过线路ID获取线路信息
     *
     * @param lineId 行id
     * @param mapper 映射器
     * @return {@link BasePowerLineEnt}
     */
    private BasePowerLineEnt selectOneByLineId(String lineId, BasePowerLineMapper mapper) {
        BasePowerLineEnt entity = mapper.selectOne(new QueryWrapper<BasePowerLineEnt>()
                .lambda()
                .eq(BasePowerLineEnt::getId, lineId));
        return entity;
    }

    /**
     * 保存线路关联信息
     *
     * @param lineId         线路id
     * @param ouId           组织id
     * @param session        会话
     * @param wellheadInputs 井口列表
     * @param plantInputs    光伏列表
     * @param storageInputs  储能列表
     * @param dieselInputs   柴发列表
     */
    private void saveLineRelation(String lineId, String ouId, PpsUserSession session,
                                  List wellheadInputs, List plantInputs, List storageInputs, List dieselInputs) {
        //新增井口配置
        if (CollectionUtil.isNotEmpty(wellheadInputs)) {
            List<BasePowerLineWellheadEnt> wellheads = XCopyUtils.copyNewList(wellheadInputs, BasePowerLineWellheadEnt.class);
            for (BasePowerLineWellheadEnt wellhead : wellheads) {
                wellhead.setLineId(lineId);
                wellhead.setOuId(ouId);
                BaseEntUtils.setBaseEntDefault(wellhead, session);
            }
            BaseEntUtils.batchInsert(BasePowerLineWellheadMapper.class, BasePowerLineWellheadEnt.class, wellheads);
        }
        //新增光伏配置
        if (CollectionUtil.isNotEmpty(plantInputs)) {
            List<BasePowerLinePlantEnt> plants = XCopyUtils.copyNewList(plantInputs, BasePowerLinePlantEnt.class);
            for (BasePowerLinePlantEnt plant : plants) {
                plant.setLineId(lineId);
                plant.setOuId(ouId);
                BaseEntUtils.setBaseEntDefault(plant, session);
            }
            BaseEntUtils.batchInsert(BasePowerLinePlantMapper.class, BasePowerLinePlantEnt.class, plants);
        }
        //新增储能配置
        if (CollectionUtil.isNotEmpty(storageInputs)) {
            List<BasePowerLineStorageEnt> storages = XCopyUtils.copyNewList(storageInputs, BasePowerLineStorageEnt.class);
            for (BasePowerLineStorageEnt storage : storages) {
                storage.setLineId(lineId);
                storage.setOuId(ouId);
                BaseEntUtils.setBaseEntDefault(storage, session);
            }
            BaseEntUtils.batchInsert(BasePowerLineStorageMapper.class, BasePowerLineStorageEnt.class, storages);
        }
        //新增柴发配置
        if (CollectionUtil.isNotEmpty(dieselInputs)) {
            List<BasePowerLineDieselEnt> diesels = XCopyUtils.copyNewList(dieselInputs, BasePowerLineDieselEnt.class);
            for (BasePowerLineDieselEnt diesel : diesels) {
                diesel.setLineId(lineId);
                diesel.setOuId(ouId);
                BaseEntUtils.setBaseEntDefault(diesel, session);
            }
            BaseEntUtils.batchInsert(BasePowerLineDieselMapper.class, BasePowerLineDieselEnt.class, diesels);
        }
    }

    /**
     * 删除线路关系信息
     *
     * @param context 上下文
     * @param lineId  行id
     */
    private void deleteLineRelation(XContext context, String lineId) {
        //删除井口关联信息
        BasePowerLineWellheadMapper wellheadMapper = context.getBean(BasePowerLineWellheadMapper.class);
        wellheadMapper.delete(new LambdaQueryWrapper<BasePowerLineWellheadEnt>()
                .eq(BasePowerLineWellheadEnt::getLineId, lineId)
        );
        //删除光伏关联信息
        BasePowerLinePlantMapper plantMapper = context.getBean(BasePowerLinePlantMapper.class);
        plantMapper.delete(new LambdaQueryWrapper<BasePowerLinePlantEnt>()
                .eq(BasePowerLinePlantEnt::getLineId, lineId)
        );
        //删除储能关联信息
        BasePowerLineStorageMapper storageMapper = context.getBean(BasePowerLineStorageMapper.class);
        storageMapper.delete(new LambdaQueryWrapper<BasePowerLineStorageEnt>()
                .eq(BasePowerLineStorageEnt::getLineId, lineId)
        );
        //删除柴发关联信息
        BasePowerLineDieselMapper dieselMapper = context.getBean(BasePowerLineDieselMapper.class);
        dieselMapper.delete(new LambdaQueryWrapper<BasePowerLineDieselEnt>()
                .eq(BasePowerLineDieselEnt::getLineId, lineId)
        );
    }
}
