package pps.core.base.service;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.apache.commons.lang3.StringUtils;
import pps.cloud.system.service.SystemAreaService;
import pps.cloud.system.service.data.GetSysAreaInput;
import pps.cloud.system.service.data.GetSysAreaOutput;
import pps.core.base.entity.BasePriceStrategyDetailEnt;
import pps.core.base.entity.BasePriceStrategyEnt;
import pps.core.base.entity.BasePriceStrategyMonthEnt;
import pps.core.base.entity.BasePriceStrategyView;
import pps.core.base.mapper.BasePriceStrategyDetailMapper;
import pps.core.base.mapper.BasePriceStrategyMapper;
import pps.core.base.mapper.BasePriceStrategyMonthMapper;
import pps.core.base.mapper.BasePriceStrategyViewMapper;
import pps.core.base.service.data.base_price_strategy.*;
import pps.core.base.service.data.base_price_strategy_month.CreateBasePriceStrategyMonthInput;
import pps.core.base.service.data.base_price_strategy_month.GetBasePriceStrategyMonthOutput;
import pps.core.base.service.data.base_price_strategy_month.UpdateBasePriceStrategyMonthInput;
import pps.core.base.utils.DictUtils;
import pps.core.common.entity.BaseEnt;
import pps.core.common.session.PpsUserSession;
import pps.core.common.utils.BaseEntUtils;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.helper.XTransactionHelper;

import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * 市电峰谷配置模块
 *
 * @author ZWT
 * @date 2023/08/24
 */
@XService
@XText("市电峰谷配置模块")
public class BasePriceStrategyService {

    /**
     * 市电峰谷策略--新增
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XApiAnonymous
    @XApiPost
    @XText("市电峰谷策略--新增")
    public XServiceResult createBasePriceStrategy(XContext context, CreateBasePriceStrategyInput input) {
        PpsUserSession session = context.getSession(PpsUserSession.class);
        return XTransactionHelper.begin(context, () -> {
            BasePriceStrategyMapper mapper = context.getBean(BasePriceStrategyMapper.class);
            BasePriceStrategyEnt entity = XCopyUtils.copyNewObject(input, BasePriceStrategyEnt.class);
            BaseEntUtils.setBaseEntDefault(entity, session);
            mapper.insert(entity);
            String strategyId = entity.getId();
            List<CreateBasePriceStrategyMonthInput> inputMonths = input.getMonths();
            if (CollectionUtil.isNotEmpty(inputMonths)) {
                List<BasePriceStrategyMonthEnt> monthList = XCopyUtils.copyNewList(inputMonths, BasePriceStrategyMonthEnt.class);
                BasePriceStrategyMonthMapper monthMapper = context.getBean(BasePriceStrategyMonthMapper.class);
                this.saveStrategyMonthDetail(monthMapper, monthList, strategyId, session);
            }
            return XServiceResult.OK;
        });
    }

    /**
     * 市电峰谷策略--修改
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XApiAnonymous
    @XApiPost
    @XText("市电峰谷策略--修改")
    public XServiceResult updateBasePriceStrategy(XContext context, UpdateBasePriceStrategyInput input) {
        PpsUserSession session = context.getSession(PpsUserSession.class);
        String strategyId = input.getId();
        return XTransactionHelper.begin(context, () -> {
            BasePriceStrategyMapper mapper = context.getBean(BasePriceStrategyMapper.class);
            BasePriceStrategyEnt entity = mapper.selectOne(new LambdaQueryWrapper<BasePriceStrategyEnt>()
                    .eq(BaseEnt::getId, strategyId));
            if (Objects.isNull(entity)) {
                return XServiceResult.error(context, XError.NotFound);
            }
            List<UpdateBasePriceStrategyMonthInput> inputMonths = input.getMonths();
            if (CollectionUtil.isNotEmpty(inputMonths)) {
                List<BasePriceStrategyMonthEnt> monthList = XCopyUtils.copyNewList(inputMonths, BasePriceStrategyMonthEnt.class);
                List<Integer> collect = monthList.stream()
                        .map(BasePriceStrategyMonthEnt::getStrategyMonth)
                        .collect(Collectors.toList());
                BasePriceStrategyMonthMapper monthMapper = context.getBean(BasePriceStrategyMonthMapper.class);
                //删除月信息
                monthMapper.delete(new QueryWrapper<BasePriceStrategyMonthEnt>()
                        .lambda()
                        .eq(BasePriceStrategyMonthEnt::getStrategyId, strategyId)
                        .in(BasePriceStrategyMonthEnt::getStrategyMonth, collect)
                );
                BasePriceStrategyDetailMapper detailMapper = context.getBean(BasePriceStrategyDetailMapper.class);
                //删除详情
                detailMapper.delete(new QueryWrapper<BasePriceStrategyDetailEnt>()
                        .lambda()
                        .eq(BasePriceStrategyDetailEnt::getStrategyId, strategyId)
                        .in(BasePriceStrategyDetailEnt::getStrategyMonth, collect)
                );
                this.saveStrategyMonthDetail(monthMapper, monthList, strategyId, session);
            }
            XCopyUtils.copyObject(input, entity);
            BaseEntUtils.setBaseEntDefault(entity, session);
            mapper.updateById(entity);
            return XServiceResult.OK;
        });
    }

    /**
     * 市电峰谷策略--删除
     * (如果被引用,则不能删除)
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XApiAnonymous
    @XApiPost
    @XText("市电峰谷策略--删除")
    public XServiceResult deleteBasePriceStrategy(XContext context, DeleteBasePriceStrategyInput input) {
        String strategyId = input.getId();
        if (DictUtils.checkStrategyIsReference(context, strategyId)) {
            return XServiceResult.error(992, "当前策略已被引用");
        }
        return XTransactionHelper.begin(context, () -> {
            BasePriceStrategyMapper mapper = context.getBean(BasePriceStrategyMapper.class);
            BasePriceStrategyEnt entity = mapper.selectOne(new LambdaQueryWrapper<BasePriceStrategyEnt>()
                    .eq(BaseEnt::getId, strategyId));
            if (Objects.isNull(entity)) {
                return XServiceResult.error(context, XError.NotFound);
            }
            //删除月数据
            BasePriceStrategyMonthMapper monthMapper = context.getBean(BasePriceStrategyMonthMapper.class);
            monthMapper.delete(new QueryWrapper<BasePriceStrategyMonthEnt>()
                    .lambda()
                    .eq(BasePriceStrategyMonthEnt::getStrategyId, strategyId));
            //删除明细数据
            BasePriceStrategyDetailMapper detailMapper = context.getBean(BasePriceStrategyDetailMapper.class);
            detailMapper.delete(new QueryWrapper<BasePriceStrategyDetailEnt>()
                    .lambda()
                    .eq(BasePriceStrategyDetailEnt::getStrategyId, strategyId));
            mapper.deleteById(entity);
            return XServiceResult.OK;
        });
    }

    /**
     * 市电峰谷策略--详情
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link GetBasePriceStrategyOutput}>
     */
    @XApiAnonymous
    @XApiGet
    @XText("市电峰谷策略--详情")
    public XSingleResult<GetBasePriceStrategyOutput> getBasePriceStrategy(XContext context, GetBasePriceStrategyInput input) {
        BasePriceStrategyMapper mapper = context.getBean(BasePriceStrategyMapper.class);
        String strategyId = input.getId();
        QueryWrapper<BasePriceStrategyEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda()
                .select(BasePriceStrategyEnt::getId,
                        BasePriceStrategyEnt::getPolicyName,
                        BasePriceStrategyEnt::getAreaCode,
                        BasePriceStrategyEnt::getStrategyYear,
                        BasePriceStrategyEnt::getIsFixedPrice,
                        BasePriceStrategyEnt::getFixedElectricityPrice)
                .eq(BasePriceStrategyEnt::getId, strategyId);
        BasePriceStrategyEnt entity = mapper.selectOne(queryWrapper);
        if (Objects.isNull(entity)) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBasePriceStrategyOutput output = XCopyUtils.copyNewObject(entity, GetBasePriceStrategyOutput.class);
        //查月份
        BasePriceStrategyMonthMapper monthMapper = context.getBean(BasePriceStrategyMonthMapper.class);
        List<BasePriceStrategyMonthEnt> monthEntList = monthMapper.selectList(new QueryWrapper<BasePriceStrategyMonthEnt>()
                .lambda()
                .select(BasePriceStrategyMonthEnt::getId, BasePriceStrategyMonthEnt::getStrategyMonth)
                .eq(BasePriceStrategyMonthEnt::getStrategyId, strategyId));
        if (CollectionUtil.isNotEmpty(monthEntList)) {
            //查明细
            BasePriceStrategyDetailMapper detailMapper = context.getBean(BasePriceStrategyDetailMapper.class);
            List<BasePriceStrategyDetailEnt> detailEntList = detailMapper.selectList(new QueryWrapper<BasePriceStrategyDetailEnt>()
                    .lambda()
                    .select(BasePriceStrategyDetailEnt::getStrategyMonth,
                            BasePriceStrategyDetailEnt::getPeriodTypeKey,
                            BasePriceStrategyDetailEnt::getStartTime,
                            BasePriceStrategyDetailEnt::getEndTime,
                            BasePriceStrategyDetailEnt::getElectrovalence,
                            BasePriceStrategyDetailEnt::getInternalSettlementPrice)
                    .eq(BasePriceStrategyDetailEnt::getStrategyId, strategyId));
            if (CollectionUtil.isNotEmpty(detailEntList)) {
                Map<Integer, List<BasePriceStrategyDetailEnt>> collect = detailEntList.stream()
                        .collect(Collectors.groupingBy(BasePriceStrategyDetailEnt::getStrategyMonth));
                for (BasePriceStrategyMonthEnt monthEnt : monthEntList) {
                    monthEnt.setDetails(collect.get(monthEnt.getStrategyMonth()));
                }
            }
            List<GetBasePriceStrategyMonthOutput> months = XCopyUtils.copyNewList(monthEntList, GetBasePriceStrategyMonthOutput.class);
            output.setMonths(months);
        }
        return XSingleResult.success(output);
    }

    /**
     * 市电峰谷策略--分页列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XPageResult}<{@link QueryBasePriceStrategyViewOutput}>
     */
    @XApiAnonymous
    @XApiGet
    @XText("市电峰谷策略--分页列表")
    public XPageResult<QueryBasePriceStrategyViewOutput> queryBasePriceStrategyView(XContext context, QueryBasePriceStrategyViewInput input) {
        BasePriceStrategyViewMapper mapper = context.getBean(BasePriceStrategyViewMapper.class);
        BasePriceStrategyView record = XCopyUtils.copyNewObject(input, BasePriceStrategyView.class);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<BasePriceStrategyView> list = mapper.selectList(record);
        PageInfo<BasePriceStrategyView> pageInfo = new PageInfo<>(list);
        List<QueryBasePriceStrategyViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBasePriceStrategyViewOutput.class);
        List<GetSysAreaOutput> sysAreaOutputList = this.getSysAreaOutputList(context);
        for (QueryBasePriceStrategyViewOutput output : outputs) {
            output.setAreaName(sysAreaOutputList.stream()
                    .filter(area -> StringUtils.equals(output.getAreaCode(), String.valueOf(area.getId())))
                    .map(GetSysAreaOutput::getName)
                    .findAny()
                    .orElse("其他地区"));
        }
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    /**
     * 市电峰谷策略--列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQueryBasePriceStrategyOutput}>
     */
    @XApiAnonymous
    @XApiGet
    @XText("市电峰谷策略--列表")
    public XListResult<DynamicQueryBasePriceStrategyOutput> queryBasePriceStrategyList(XContext context, DynamicQueryBasePriceStrategyInput input) {
        String id = input.getId();
        String policyName = input.getPolicyName();
        String areaCode = input.getAreaCode();
        Integer strategyYear = input.getStrategyYear();
        Integer isFixedPrice = input.getIsFixedPrice();
        BasePriceStrategyMapper mapper = context.getBean(BasePriceStrategyMapper.class);
        List<BasePriceStrategyEnt> selectList = mapper.selectList(new LambdaQueryWrapper<BasePriceStrategyEnt>()
                .select(BaseEnt::getId, BasePriceStrategyEnt::getPolicyName)
                .eq(StringUtils.isNotBlank(id), BaseEnt::getId, id)
                .like(StringUtils.isNotBlank(policyName), BasePriceStrategyEnt::getPolicyName, policyName)
                .eq(StringUtils.isNotBlank(areaCode), BasePriceStrategyEnt::getAreaCode, areaCode)
                .eq(Objects.nonNull(strategyYear), BasePriceStrategyEnt::getStrategyYear, strategyYear)
                .eq(Objects.nonNull(isFixedPrice), BasePriceStrategyEnt::getIsFixedPrice, isFixedPrice)
                .orderByDesc(BaseEnt::getModifyTime)
        );
        List<DynamicQueryBasePriceStrategyOutput> strategyOutputList = XCopyUtils.copyNewList(selectList, DynamicQueryBasePriceStrategyOutput.class);
        return XListResult.success(strategyOutputList);
    }

    /*-----------------------------------private-----------------------------------*/

    /**
     * 获得系统输出列表区域
     *
     * @param context 上下文
     * @return {@link List}<{@link GetSysAreaOutput}>
     */
    private List<GetSysAreaOutput> getSysAreaOutputList(XContext context) {
        SystemAreaService systemAreaService = context.getBean(SystemAreaService.class);
        XListResult<GetSysAreaOutput> sysAreaList = systemAreaService.getSysAreaList(context, new GetSysAreaInput());
        sysAreaList.throwIfFail();
        return sysAreaList.getResult();
    }

    /**
     * 保存策略明细
     *
     * @param monthMapper 月映射器
     * @param monthList   月清单
     * @param strategyId  战略id
     * @param session     会话
     */
    private void saveStrategyMonthDetail(BasePriceStrategyMonthMapper monthMapper, List<BasePriceStrategyMonthEnt> monthList, String strategyId, PpsUserSession session) {
        Integer strategyMonth;
        List<BasePriceStrategyDetailEnt> detailList;
        for (BasePriceStrategyMonthEnt monthEnt : monthList) {
            monthEnt.setStrategyId(strategyId);
            BaseEntUtils.setBaseEntDefault(monthEnt, session);
            monthMapper.insert(monthEnt);
            strategyMonth = monthEnt.getStrategyMonth();
            detailList = monthEnt.getDetails();
            for (BasePriceStrategyDetailEnt detailEnt : detailList) {
                detailEnt.setStrategyId(strategyId);
                detailEnt.setStrategyMonth(strategyMonth);
                BaseEntUtils.setBaseEntDefault(detailEnt, session);
            }
            BaseEntUtils.batchInsert(BasePriceStrategyDetailMapper.class, BasePriceStrategyDetailEnt.class, detailList);
        }
    }
}