package pps.core.base.service.data.base_power_line;

import jakarta.validation.Valid;
import jakarta.validation.constraints.NotBlank;
import lombok.Data;
import pps.core.base.service.data.base_power_line_diesel.UpdateBasePowerLineDieselInput;
import pps.core.base.service.data.base_power_line_plant.UpdateBasePowerLinePlantInput;
import pps.core.base.service.data.base_power_line_storage.UpdateBasePowerLineStorageInput;
import pps.core.base.service.data.base_power_line_wellhead.UpdateBasePowerLineWellheadInput;
import xstartup.annotation.XText;

import java.util.List;

/**
 * 输电线路配置
 *
 * @author ZWT
 * @date 2023/08/25
 */
@Data
public class UpdateBasePowerLineInput {

    @XText("ID")
    @NotBlank(message = "缺少线路ID")
    private String id;

    @XText("组织机构ID")
    @NotBlank(message = "缺少组织机构ID")
    private String ouId;

    @XText("上级线路ID")
    private String upperLineId;

    @XText("线路名称")
    private String lineName;

    @XText("电网类型key(字典获取)")
    private String gridTypeKey;

    @XText("线路类型key(字典获取)")
    private String lineTypeKey;

    @XText("市电峰谷配置ID")
    private String strategyId;

    @XText("是否激活共享调度(0_激活;1_关闭)")
    private Integer isShareDispatch;

    @XText("是否激活返输调度(0_激活;1_关闭)")
    private Integer isReverseDispatch;

    /**
     * 输电线路柴发设备配置
     */
    @XText("输电线路柴发设备配置")
    @Valid
    private List<UpdateBasePowerLineDieselInput> dieselInputs;

    /**
     * 输电线路光伏配置
     */
    @XText("输电线路光伏配置")
    @Valid
    private List<UpdateBasePowerLinePlantInput> plantInputs;

    /**
     * 输电线路储能配置
     */
    @XText("输电线路储能配置")
    @Valid
    private List<UpdateBasePowerLineStorageInput> storageInputs;

    /**
     * 输电线路井口配置
     */
    @XText("输电线路井口配置")
    @Valid
    private List<UpdateBasePowerLineWellheadInput> wellheadInputs;
}
