package pps.core.space.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import pps.core.common.session.PpsUserSession;
import pps.core.space.entity.SpaceIntervalSystemEnt;
import pps.core.space.entity.SpaceIntervalSystemView;
import pps.core.space.mapper.SpaceIntervalSystemMapper;
import pps.core.space.mapper.SpaceIntervalSystemViewMapper;
import pps.core.space.service.data.space_interval_system.*;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.helper.XTransactionHelper;

import java.util.List;

/**
 * 基础间开制度
 *
 * @author ZWT
 * @date 2023/08/29
 */
@XService
@XText(value = "基础间开制度")
public class SpaceIntervalSystemService {

    /**
     * 基础间开制度--新增
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XApiAnonymous
    @XApiPost
    @XText(value = "基础间开制度--新增")
    public XServiceResult createSpaceIntervalSystem(XContext context, CreateSpaceIntervalSystemInput input) {
        //        PpsUserSession session = context.getSession(PpsUserSession.class);
        PpsUserSession session = new PpsUserSession();
        session.setId("123");
        session.setUserName("ceshi");
        return XTransactionHelper.begin(context, () -> {
            SpaceIntervalSystemMapper mapper = context.getBean(SpaceIntervalSystemMapper.class);
            SpaceIntervalSystemEnt entity = new SpaceIntervalSystemEnt();
            XCopyUtils.copyObject(input, entity);
            mapper.insert(entity);
            return XServiceResult.OK;
        });
    }

    @XApiAnonymous
    @XApiPost
    public XServiceResult updateSpaceIntervalSystem(XContext context, UpdateSpaceIntervalSystemInput input) {
        SpaceIntervalSystemMapper mapper = context.getBean(SpaceIntervalSystemMapper.class);
        QueryWrapper<SpaceIntervalSystemEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SpaceIntervalSystemEnt::getId, input.getId());
        SpaceIntervalSystemEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiPost
    public XServiceResult deleteSpaceIntervalSystem(XContext context, DeleteSpaceIntervalSystemInput input) {
        SpaceIntervalSystemMapper mapper = context.getBean(SpaceIntervalSystemMapper.class);
        QueryWrapper<SpaceIntervalSystemEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SpaceIntervalSystemEnt::getId, input.getId());
        SpaceIntervalSystemEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        mapper.deleteById(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetSpaceIntervalSystemViewOutput> getSpaceIntervalSystemView(XContext context, GetSpaceIntervalSystemViewInput input) {
        SpaceIntervalSystemViewMapper mapper = context.getBean(SpaceIntervalSystemViewMapper.class);
        SpaceIntervalSystemView record = new SpaceIntervalSystemView();
        XCopyUtils.copyObject(input, record);
        SpaceIntervalSystemView view = mapper.selectOne(record);
        if (view == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetSpaceIntervalSystemViewOutput output = new GetSpaceIntervalSystemViewOutput();
        XCopyUtils.copyObject(view, output);
        return XSingleResult.success(output);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QuerySpaceIntervalSystemViewOutput> querySpaceIntervalSystemView(XContext context, QuerySpaceIntervalSystemViewInput input) {
        SpaceIntervalSystemViewMapper mapper = context.getBean(SpaceIntervalSystemViewMapper.class);
        SpaceIntervalSystemView record = new SpaceIntervalSystemView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<SpaceIntervalSystemView> list = mapper.selectList(record);
        PageInfo<SpaceIntervalSystemView> pageInfo = new PageInfo<>(list);
        List<QuerySpaceIntervalSystemViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySpaceIntervalSystemViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }
}
